@@ -20,7 +20,6 @@ Author: Daniel Kroening, kroening@kroening.com
 #include "ansi_c_typecheck.h"
 #include "ansi_c_parser.h"
 #include "expr2c.h"
-#include "c_final.h"
 #include "trans_unit.h"
 #include "c_link.h"
 #include "c_preprocess.h"
@@ -126,7 +125,7 @@ bool ansi_c_languaget::parse(
   std::istringstream codestr(code);
 
   ansi_c_parser.clear();
-  ansi_c_parser.filename="<built-in>";
+  ansi_c_parser.filename=ID_built_in;
   ansi_c_parser.in=&codestr;
   ansi_c_parser.set_message_handler(message_handler);
   ansi_c_parser.grammar=ansi_c_parsert::LANGUAGE;
@@ -226,7 +225,6 @@ bool ansi_c_languaget::final(
   contextt &context,
   message_handlert &message_handler)
 {
-  if(c_final(context, message_handler)) return true;
   if(c_main(context, "c::", "c::main", message_handler)) return true;
   
   return false;
