@@ -75,6 +75,91 @@ Author: Daniel Kroening, kroening@kroening.com
 
 /*******************************************************************\
 
+Function: shell_quote
+
+  Inputs:
+
+ Outputs:
+
+ Purpose: quote a string for bash and CMD
+
+\*******************************************************************/
+
+std::string shell_quote(const std::string &src)
+{
+  #ifdef _WIN32
+  // first check if quoting is needed at all
+  
+  if(src.find(' ')==std::string::npos &&
+     src.find('"')==std::string::npos &&
+     src.find('&')==std::string::npos &&
+     src.find('|')==std::string::npos &&
+     src.find('(')==std::string::npos &&
+     src.find(')')==std::string::npos &&
+     src.find('<')==std::string::npos &&
+     src.find('>')==std::string::npos &&
+     src.find('^')==std::string::npos)
+  {
+    // seems fine -- return as is
+    return src;
+  }
+  
+  std::string result;
+  
+  result+='"';
+
+  for(unsigned i=0; i<src.size(); i++)
+  {
+    const char ch=src[i];
+    if(ch=='"') result+='"'; // quotes are doubled
+    result+=ch;
+  }  
+  
+  result+='"';
+
+  return result;
+
+  #else
+
+  // first check if quoting is needed at all
+  
+  if(src.find(' ')==std::string::npos &&
+     src.find('"')==std::string::npos &&
+     src.find('*')==std::string::npos &&
+     src.find('$')==std::string::npos &&
+     src.find('\\')==std::string::npos &&
+     src.find('?')==std::string::npos &&
+     src.find('&')==std::string::npos &&
+     src.find('|')==std::string::npos &&
+     src.find('>')==std::string::npos &&
+     src.find('<')==std::string::npos &&
+     src.find('^')==std::string::npos &&
+     src.find('\'')==std::string::npos)
+  {
+    // seems fine -- return as is
+    return src;
+  }
+  
+  std::string result;
+  
+  // the single quotes catch everything but themselves!
+  result+='\'';
+
+  for(unsigned i=0; i<src.size(); i++)
+  {
+    const char ch=src[i];
+    if(ch=='\'') result+="'\\''";
+    result+=ch;
+  }  
+  
+  result+='\'';
+
+  return result;
+  #endif
+}
+
+/*******************************************************************\
+
 Function: c_preprocess
 
   Inputs:
@@ -197,7 +282,7 @@ bool c_preprocess_visual_studio(
 
   message_streamt message_stream(message_handler);
 
-  // use VC98 CL
+  // use Visual Studio's CL
   
   std::string stderr_file=get_temporary_file("tmp.stderr", "");
   std::string command_file_name=get_temporary_file("tmp.cl-cmd", "");
@@ -229,19 +314,19 @@ bool c_preprocess_visual_studio(
         it=config.ansi_c.defines.begin();
         it!=config.ansi_c.defines.end();
         it++)
-      command_file << "/D\"" << *it << "\"" << std::endl;
+      command_file << "/D"+shell_quote(*it) << std::endl;
 
     for(std::list<std::string>::const_iterator
         it=config.ansi_c.include_paths.begin();
         it!=config.ansi_c.include_paths.end();
         it++)
-      command_file << "/I\"" << *it << "\"" << std::endl;
+      command_file << "/I"+shell_quote(*it) << std::endl;
 
-    command_file << file << std::endl;
+    command_file << "\"" << file << "\"" << std::endl;
   }
   
   std::string tmpi=get_temporary_file("tmp.cl", "");
-
+  
   std::string command="CL @"+command_file_name;
   command+=" > \""+tmpi+"\"";
   command+=" 2> \""+stderr_file+"\"";
@@ -256,7 +341,7 @@ bool c_preprocess_visual_studio(
   {
     unlink(tmpi.c_str());
     unlink(stderr_file.c_str());
-    message_stream.error("Preprocessing failed (fopen failed)");
+    message_stream.error("CL Preprocessing failed (fopen failed)");
     return true;
   }
 
@@ -283,7 +368,7 @@ bool c_preprocess_visual_studio(
   if(result!=0)
   {
     message_stream.error_parse(1);
-    message_stream.error("Preprocessing failed");
+    message_stream.error("CL Preprocessing failed");
     return true;
   }
   else
@@ -294,6 +379,53 @@ bool c_preprocess_visual_studio(
 
 /*******************************************************************\
 
+Function: postprocess_codewarrior
+
+  Inputs:
+
+ Outputs:
+
+ Purpose: post-processing specifically for CodeWarrior
+
+\*******************************************************************/
+
+void postprocess_codewarrior(
+  std::istream &instream,
+  std::ostream &outstream)
+{
+  // CodeWarrior prepends some header to the file,
+  // marked with '#' signs.
+  // We skip over it.
+  //
+  // CodeWarrior has an ugly way of marking lines, e.g.:
+  //
+  // /* #line 1      "__ppc_eabi_init.cpp"   /* stack depth 0 */
+  //
+  // We remove the initial '/* ' prefix
+  
+  std::string line;
+  
+  while(instream)
+  {
+    str_getline(instream, line);
+    
+    if(line.size()>=2 &&
+       line[0]=='#' && (line[1]=='#' || line[1]==' ' || line[1]=='\t'))
+    {
+      // skip the line!
+    }
+    else if(line.size()>=3 &&
+            line[0]=='/' && line[1]=='*' && line[2]==' ')
+    {
+      outstream << line.c_str()+3 << std::endl; // strip the '/* '
+    }
+    else
+      outstream << line << std::endl;
+  }
+}
+
+/*******************************************************************\
+
 Function: c_preprocess_codewarrior
 
   Inputs:
@@ -309,41 +441,94 @@ bool c_preprocess_codewarrior(
   std::ostream &outstream,
   message_handlert &message_handler)
 {
+  // check extension
+  if(is_dot_i_file(file))
+    return c_preprocess_none(file, outstream, message_handler);
+
+  // preprocessing
   message_streamt message_stream(message_handler);
 
-  // CodeWarrior prepends some header to the file,
-  // marked with '#' signs.
-  // We skip over it.
-  //
-  // CodeWarrior has an ugly way of marking lines, e.g.:
-  //
-  // /* #line 1      "__ppc_eabi_init.cpp"   /* stack depth 0 */
-  //
-  // We remove the initial '/* ' prefix
+  std::string stderr_file=get_temporary_file("tmp.stderr", "");
+
+  std::string command;
   
-  std::ifstream in(file.c_str());
-  
-  std::string line;
-  
-  while(in)
-  {
-    str_getline(in, line);
+  command="mwcceppc -E -P -D__CPROVER__ -ppopt line -ppopt full";
+
+  for(std::list<std::string>::const_iterator
+      it=config.ansi_c.defines.begin();
+      it!=config.ansi_c.defines.end();
+      it++)
+    command+=" -D'"+*it+"'";
+
+  for(std::list<std::string>::const_iterator
+      it=config.ansi_c.include_paths.begin();
+      it!=config.ansi_c.include_paths.end();
+      it++)
+    command+=" -I'"+*it+"'";
+
+  for(std::list<std::string>::const_iterator
+      it=config.ansi_c.include_files.begin();
+      it!=config.ansi_c.include_files.end();
+      it++)
+    command+=" -include '"+*it+"'";
+
+  for(std::list<std::string>::const_iterator
+      it=config.ansi_c.preprocessor_options.begin();
+      it!=config.ansi_c.preprocessor_options.end();
+      it++)
+    command+=" "+*it;
     
-    if(line.size()>=1 &&
-       line[0]=='#' && (line[1]=='#' || line[1]==' ' || line[1]=='\t'))
-    {
-      // skip the line!
-    }
-    else if(line.size()>=3 &&
-            line[0]=='/' && line[1]=='*' && line[2]==' ')
-    {
-      outstream << line.c_str()+3 << std::endl; // strip the '/* '
-    }
-    else
-      outstream << line << std::endl;
+  int result;
+
+  std::string tmpi=get_temporary_file("tmp.cl", "");
+  command+=" \""+file+"\"";
+  command+=" -o \""+tmpi+"\"";
+  command+=" 2> \""+stderr_file+"\"";
+
+  result=system(command.c_str());
+
+  std::ifstream stream_i(tmpi.c_str());
+
+  if(stream_i)
+  {
+    postprocess_codewarrior(stream_i, outstream);
+
+    stream_i.close();
+    unlink(tmpi.c_str());
   }
-  
+  else
+  {
+    unlink(tmpi.c_str());
+    unlink(stderr_file.c_str());
+    message_stream.error("Preprocessing failed (fopen failed)");
+    return true;
+  }
+
+  // errors/warnings
+  {
+    std::ifstream stderr_stream(stderr_file.c_str());
+    char ch;
+    while((stderr_stream.read(&ch, 1))!=NULL)
+      message_stream.str << ch;
+  }
+
+  unlink(stderr_file.c_str());
+
+  if(result!=0)
+  {
+    message_stream.error_parse(1);
+    message_stream.error("Preprocessing failed");
+    return true;
+  }
+  else
+    message_stream.error_parse(2);
+
   return false;
+
+  #if 0
+  message_streamt message_stream(message_handler);
+
+  #endif
 }
 
 /*******************************************************************\
@@ -390,7 +575,6 @@ bool c_preprocess_gcc(
     command+=" -D__FLT_MIN__=1.17549435e-38F";
     command+=" -D__DEC64_DEN__=0.000000000000001E-383DD";
     command+=" -D__CHAR_BIT__=8";
-    command+=" -D__WCHAR_MAX__=2147483647";
     command+=" -D__DBL_DENORM_MIN__=4.9406564584124654e-324";
     command+=" -D__FLT_EVAL_METHOD__=0";
     command+=" -D__DBL_MIN_10_EXP__=\"(-307)\"";
@@ -419,7 +603,6 @@ bool c_preprocess_gcc(
     command+=" -D__DBL_MAX_EXP__=1024";
     command+=" -D__DEC128_EPSILON__=1E-33DL";
     command+=" -D__SSE2_MATH__=1";
-    command+=" -D__LONG_LONG_MAX__=9223372036854775807LL";
     command+=" -D__GXX_ABI_VERSION=1002";
     command+=" -D__FLT_MIN_EXP__=\"(-125)\"";
     command+=" -D__DBL_MIN__=2.2250738585072014e-308";
@@ -450,7 +633,6 @@ bool c_preprocess_gcc(
     command+=" -D__k8__=1";
     command+=" -D__pic__=2";
     command+=" -D__FLT_DIG__=6";
-    command+=" -D__INT_MAX__=2147483647";
     command+=" -D__FLT_MAX_EXP__=128";
     //command+=" -D__BLOCKS__=1";
     command+=" -D__DBL_MANT_DIG__=53";
@@ -533,19 +715,19 @@ bool c_preprocess_gcc(
       it=config.ansi_c.defines.begin();
       it!=config.ansi_c.defines.end();
       it++)
-    command+=" -D'"+*it+"'";
+    command+=" -D"+shell_quote(*it);
 
   for(std::list<std::string>::const_iterator
       it=config.ansi_c.include_paths.begin();
       it!=config.ansi_c.include_paths.end();
       it++)
-    command+=" -I'"+*it+"'";
+    command+=" -I"+shell_quote(*it);
 
   for(std::list<std::string>::const_iterator
       it=config.ansi_c.include_files.begin();
       it!=config.ansi_c.include_files.end();
       it++)
-    command+=" -include '"+*it+"'";
+    command+=" -include "+shell_quote(*it);
 
   for(std::list<std::string>::const_iterator
       it=config.ansi_c.preprocessor_options.begin();
@@ -580,7 +762,7 @@ bool c_preprocess_gcc(
   {
     unlink(tmpi.c_str());
     unlink(stderr_file.c_str());
-    message_stream.error("Preprocessing failed (fopen failed)");
+    message_stream.error("GCC preprocessing failed (fopen failed)");
     return true;
   }
   #else
@@ -600,7 +782,7 @@ bool c_preprocess_gcc(
   else
   {
     unlink(stderr_file.c_str());
-    message_stream.error("Preprocessing failed (popen failed)");
+    message_stream.error("GCC preprocessing failed (popen failed)");
     return true;
   }
   #endif
@@ -618,7 +800,7 @@ bool c_preprocess_gcc(
   if(result!=0)
   {
     message_stream.error_parse(1);
-    message_stream.error("Preprocessing failed");
+    message_stream.error("GCC preprocessing failed");
     return true;
   }
   else
@@ -730,7 +912,7 @@ bool c_preprocess_arm(
   {
     unlink(tmpi.c_str());
     unlink(stderr_file.c_str());
-    message_stream.error("Preprocessing failed (fopen failed)");
+    message_stream.error("ARMCC preprocessing failed (fopen failed)");
     return true;
   }
   #else
@@ -750,7 +932,7 @@ bool c_preprocess_arm(
   else
   {
     unlink(stderr_file.c_str());
-    message_stream.error("Preprocessing failed (popen failed)");
+    message_stream.error("ARMCC preprocessing failed (popen failed)");
     return true;
   }
   #endif
@@ -768,7 +950,7 @@ bool c_preprocess_arm(
   if(result!=0)
   {
     message_stream.error_parse(1);
-    message_stream.error("Preprocessing failed");
+    message_stream.error("ARMCC preprocessing failed");
     return true;
   }
   else
@@ -802,11 +984,19 @@ bool c_preprocess_none(
     message_stream.error("failed to open `"+file+"'");
     return true;
   }
+  
+  if(config.ansi_c.mode==configt::ansi_ct::MODE_CODEWARRIOR)
+  {
+    // special treatment for "/* #line"
+    postprocess_codewarrior(infile, outstream);
+  }
+  else
+  {
+    char ch;
 
-  char ch;
-
-  while(infile.read(&ch, 1))
-    outstream << ch;
+    while(infile.read(&ch, 1))
+      outstream << ch;
+  }
 
   return false;
 }
