@@ -23,23 +23,24 @@ public:
 
   exprt operator()(const typet &type)
   {
-    return c_sizeof(type);
+    return sizeof_rec(type);
   }
 
+  exprt c_offsetof(
+    const struct_typet &type,
+    const irep_idt &component_name);
+
 protected:
   const namespacet &ns;
 
   virtual exprt sizeof_rec(const typet &type);
-  
-  exprt c_sizeof(const typet &type)
-  {
-    exprt result(sizeof_rec(type));
-    
-    if(result.is_nil()) return result;
-    
-    result.set("#c_sizeof_type", type);
-    return result;
-  }
 };
 
-exprt c_sizeof(const typet &src, const namespacet &ns);
+exprt c_sizeof(
+  const typet &src,
+  const namespacet &ns);
+
+exprt c_offsetof(
+  const struct_typet &src,
+  const irep_idt &component_name,
+  const namespacet &ns);
