@@ -6,13 +6,14 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
+#include <cstdlib>
 
 #include <config.h>
-#include <simplify_expr_class.h>
 #include <expr_util.h>
 #include <std_expr.h>
 #include <base_type.h>
+#include <symbol.h>
 
 #include "c_typecast.h"
 #include "c_types.h"
@@ -72,7 +73,7 @@ Function: c_implicit_typecast_arithmetic
 
  Outputs:
 
- Purpose:
+ Purpose: perform arithmetic prompotions and conversions
 
 \*******************************************************************/
 
@@ -213,7 +214,6 @@ bool check_c_implicit_typecast(
     if(dest_type.id()==ID_fixedbv) return false;
   }
   else if(src_type_id==ID_array ||
-          src_type_id==ID_incomplete_array ||
           src_type_id==ID_pointer)
   {
     if(dest_type.id()==ID_pointer)
@@ -228,9 +228,8 @@ bool check_c_implicit_typecast(
         return false;
     }
     
-    if((dest_type.id()==ID_array ||
-        dest_type.id()==ID_incomplete_array) &&
-       (src_type.subtype()==dest_type.subtype())) return false;
+    if(dest_type.id()==ID_array &&
+       src_type.subtype()==dest_type.subtype()) return false;
 
     if(dest_type.id()==ID_bool) return false;
     if(dest_type.id()==ID_unsignedbv) return false;
@@ -306,6 +305,8 @@ c_typecastt::c_typet c_typecastt::get_c_type(
   {
     if(width<=config.ansi_c.char_width)
       return CHAR;
+    else if(width<=config.ansi_c.short_int_width)
+      return SHORT;
     else if(width<=config.ansi_c.int_width)
       return INT;
     else if(width<=config.ansi_c.long_int_width)
@@ -317,6 +318,8 @@ c_typecastt::c_typet c_typecastt::get_c_type(
   {
     if(width<=config.ansi_c.char_width)
       return UCHAR;
+    else if(width<=config.ansi_c.short_int_width)
+      return USHORT;
     else if(width<=config.ansi_c.int_width)
       return UINT;
     else if(width<=config.ansi_c.long_int_width)
@@ -343,8 +346,7 @@ c_typecastt::c_typet c_typecastt::get_c_type(
     else
       return PTR;
   }
-  else if(type.id()==ID_array ||
-          type.id()==ID_incomplete_array)
+  else if(type.id()==ID_array)
   {
     return PTR;
   }
@@ -386,8 +388,7 @@ void c_typecastt::implicit_typecast_arithmetic(
   switch(c_type)
   {
   case PTR:
-    if(expr_type.id()==ID_array ||
-       expr_type.id()==ID_incomplete_array)
+    if(expr_type.id()==ID_array)
     {
       new_type.id(ID_pointer);
       new_type.subtype()=expr_type.subtype();
@@ -396,8 +397,10 @@ void c_typecastt::implicit_typecast_arithmetic(
     return;
 
   case BOOL:       new_type=bool_typet(); break;
-  case CHAR:       assert(false); // should always be promoted
-  case UCHAR:      assert(false); // should always be promoted
+  case CHAR:       assert(false); // should always be promoted to int
+  case UCHAR:      assert(false); // should always be promoted to int
+  case SHORT:      assert(false); // should always be promoted to int
+  case USHORT:     assert(false); // should always be promoted to int
   case INT:        new_type=int_type(); break;
   case UINT:       new_type=uint_type(); break;
   case LONG:       new_type=long_int_type(); break;
@@ -416,8 +419,7 @@ void c_typecastt::implicit_typecast_arithmetic(
   if(new_type!=expr_type)
   {
     if(new_type.id()==ID_pointer &&
-       (expr_type.id()==ID_array ||
-        expr_type.id()==ID_incomplete_array))
+       expr_type.id()==ID_array)
     {
       exprt index_expr(ID_index, expr_type.subtype());
       index_expr.reserve_operands(2);
@@ -530,8 +532,7 @@ void c_typecastt::implicit_typecast_followed(
     }
   
     if(src_type.id()==ID_pointer ||
-       src_type.id()==ID_array ||
-       src_type.id()==ID_incomplete_array)
+       src_type.id()==ID_array)
     {
       // we are quite generous about pointers
       
@@ -612,8 +613,23 @@ void c_typecastt::implicit_typecast_arithmetic(
           c_type2=get_c_type(type2);
 
   c_typet max_type=std::max(c_type1, c_type2);
-  max_type=std::max(max_type, INT); // minimum promotion
 
+  // "If an int can represent all values of the original type, the
+  // value is converted to an int; otherwise, it is converted to
+  // an unsigned int."
+  
+  // The second case can arise if we promote any unsigned type
+  // that is as large as unsigned int.
+
+  if(config.ansi_c.short_int_width==config.ansi_c.int_width &&
+     max_type==USHORT)
+    max_type=UINT;
+  else if(config.ansi_c.char_width==config.ansi_c.int_width &&
+          max_type==UCHAR)
+    max_type=UINT;
+  else
+    max_type=std::max(max_type, INT);
+  
   implicit_typecast_arithmetic(expr1, max_type);
   implicit_typecast_arithmetic(expr2, max_type);
   
@@ -646,8 +662,7 @@ void c_typecastt::do_typecast(exprt &dest, const typet &type)
   
   const typet &dest_type=ns.follow(dest.type());
 
-  if(dest_type.id()==ID_array || 
-     dest_type.id()==ID_incomplete_array)
+  if(dest_type.id()==ID_array)
   {
     index_exprt index;
     index.array()=dest;
@@ -660,20 +675,5 @@ void c_typecastt::do_typecast(exprt &dest, const typet &type)
   }
 
   if(dest_type!=type)
-  {
     dest.make_typecast(type);
-    
-    if(dest.op0().is_constant())
-    {
-      // preserve #c_sizeof_type -- don't make it a reference!
-      const irept c_sizeof_type=
-        dest.op0().find("#c_sizeof_type");
-
-      simplify_exprt simplify_expr(ns);
-      simplify_expr.simplify_typecast(dest);
-
-      if(c_sizeof_type.is_not_nil())
-        dest.add("#c_sizeof_type")=c_sizeof_type;
-    }
-  }
 }
