@@ -170,6 +170,7 @@ protected:
   virtual void typecheck_computed_goto(codet &code);
   virtual void typecheck_ifthenelse(code_ifthenelset &code);
   virtual void typecheck_label(code_labelt &code);
+  virtual void typecheck_gcc_local_label(codet &code);
   virtual void typecheck_return(codet &code);
   virtual void typecheck_switch(codet &code);
   virtual void typecheck_while(codet &code);
@@ -224,27 +225,22 @@ protected:
   virtual void typecheck_compound_type(struct_union_typet &type);
   virtual void typecheck_code_type(code_typet &type);
   virtual void typecheck_symbol_type(typet &type);
-  virtual void typecheck_c_bitfield_type(typet &type);
+  virtual void typecheck_c_bit_field_type(typet &type);
   virtual void typecheck_typeof_type(typet &type);
   virtual void typecheck_array_type(array_typet &type);
   virtual void typecheck_vector_type(vector_typet &type);
   virtual void adjust_function_argument(typet &type) const;
-  virtual void add_padding(struct_typet &type);
-  virtual unsigned alignment(const typet &type) const;
   virtual bool is_complete_type(const typet &type) const;
 
   // this cleans expressions in array types
   virtual void clean_type(
-    const symbolt &base_symbol,
-    typet &type);
+    const irep_idt &base_symbol_identifier,
+    typet &type,
+    std::list<codet> &code);
   
   typedef hash_set_cont<irep_idt, irep_id_hash> already_cleanedt;
   already_cleanedt already_cleaned;
 
-  // this is for storing side-effects found in types  
-  typedef hash_map_cont<irep_idt, codet, irep_id_hash> clean_codet;
-  clean_codet clean_code;
-  
   void make_index_type(exprt &expr);
 
   // environment
