@@ -87,11 +87,10 @@ void c_typecheck_baset::typecheck_code(codet &code)
   }
   else if(statement==ID_asm)
     typecheck_asm(code);
-  else if(statement=="start_thread")
+  else if(statement==ID_start_thread)
     typecheck_start_thread(code);
   else if(statement==ID_gcc_local_label)
-  {
-  }
+    typecheck_gcc_local_label(code);
   else if(statement==ID_msc_try_finally)
   {
     assert(code.operands().size()==2);
@@ -173,6 +172,29 @@ void c_typecheck_baset::typecheck_decl_block(codet &code)
 {
   Forall_operands(it, code)
     typecheck_code(to_code(*it));
+
+  // unwind any decl-blocks inside the decl block
+  
+  exprt::operandst new_ops;
+  new_ops.reserve(code.operands().size());
+
+  Forall_operands(it1, code)
+  {
+    if(it1->is_nil()) continue;
+
+    codet &code_op=to_code(*it1);
+  
+    if(code_op.get_statement()==ID_decl_block)
+    {
+      Forall_operands(it2, code_op)
+        if(it2->is_not_nil())
+          new_ops.push_back(*it2);
+    }
+    else
+      new_ops.push_back(code_op);
+  }
+
+  code.operands().swap(new_ops);
 }
 
 /*******************************************************************\
@@ -302,7 +324,24 @@ Function: c_typecheck_baset::typecheck_decl
 void c_typecheck_baset::typecheck_decl_type(codet &code)
 {
   assert(code.operands().size()==0);
-  // type only!
+  // Type only! May have side-effects in it!
+  
+  std::list<codet> clean_code;
+  
+  typet &type=static_cast<typet &>(code.add(ID_type_arg));
+
+  // replace if needed
+  replace_symbol(type);
+  
+  clean_type(irep_idt(), type, clean_code);
+  
+  if(!clean_code.empty())
+  {
+    // build a decl-block
+    codet decl_block(ID_decl_block);
+    decl_block.copy_to_operands(code_blockt(clean_code), code);
+    code.swap(decl_block);
+  }
 }
 
 /*******************************************************************\
@@ -325,7 +364,7 @@ void c_typecheck_baset::typecheck_decl(codet &code)
      code.operands().size()!=2)
   {
     err_location(code);
-    throw "decl expected to have 0-2 arguments";
+    throw "decl expected to have 1 or 2 arguments";
   }
 
   // op0 must be symbol
@@ -350,7 +389,26 @@ void c_typecheck_baset::typecheck_decl(codet &code)
   }
 
   symbolt &symbol=s_it->second;
+  
+  // There may be side-effects in the type.  
+  {
+    std::list<codet> clean_code;
+    clean_type(symbol.name, symbol.type, clean_code);
+    code.type()=symbol.type;
+  
+    if(!clean_code.empty())
+    {
+      // do again
+      typecheck_decl(code);
 
+      // build a decl-block
+      codet decl_block(ID_decl_block);
+      decl_block.copy_to_operands(code_blockt(clean_code), code);
+      code.swap(decl_block);
+      return;
+    }
+  }
+  
   // see if it's a typedef
   // or a function
   // or static
@@ -372,7 +430,8 @@ void c_typecheck_baset::typecheck_decl(codet &code)
     typecheck_expr(code.op1());
     do_initializer(code.op1(), symbol.type, false);
 
-    if(follow(symbol.type).id()==ID_incomplete_array)
+    if(follow(symbol.type).id()==ID_array &&
+       to_array_type(follow(symbol.type)).size().is_nil())
       symbol.type=code.op1().type();
   }
   
@@ -402,11 +461,13 @@ Function: c_typecheck_baset::is_complete_type
 bool c_typecheck_baset::is_complete_type(const typet &type) const
 {
   if(type.id()==ID_incomplete_struct ||
-     type.id()==ID_incomplete_union ||
-     type.id()==ID_incomplete_array)
+     type.id()==ID_incomplete_union)
     return false;
   else if(type.id()==ID_array)
+  {
+    if(to_array_type(type).size().is_nil()) return false;
     return is_complete_type(type.subtype());
+  }
   else if(type.id()==ID_struct || type.id()==ID_union)
   {
     const struct_union_typet::componentst &components=
@@ -609,6 +670,24 @@ void c_typecheck_baset::typecheck_label(code_labelt &code)
 
 /*******************************************************************\
 
+Function: c_typecheck_baset::typecheck_gcc_local_label
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void c_typecheck_baset::typecheck_gcc_local_label(codet &code)
+{
+  // these are just declarations, e.g.,
+  // __label__ here, there;
+}
+
+/*******************************************************************\
+
 Function: c_typecheck_baset::typecheck_goto
 
   Inputs:
@@ -733,23 +812,23 @@ void c_typecheck_baset::typecheck_return(codet &code)
 {
   if(code.operands().size()==0)
   {
-    if(return_type.id()!=ID_empty)
+    if(follow(return_type).id()!=ID_empty)
     {
-      err_location(code);
-      throw "function expected to return a value";
+      // gcc doesn't actually complain, it just warns!
+      // We'll put a zero here.
+      exprt zero=zero_initializer(return_type, code.location());
+      code.copy_to_operands(zero);
     }
   }
   else if(code.operands().size()==1)
   {
     typecheck_expr(code.op0());
 
-    if(return_type.id()==ID_empty)
+    if(follow(return_type).id()==ID_empty)
     {
-      if(code.op0().type().id()!=ID_empty)
-      {
-        err_location(code);
-        throw "function not expected to return a value";
-      }
+      // gcc doesn't actually complain, it just warns!
+      if(follow(code.op0().type()).id()!=ID_empty)
+        code.op0().make_typecast(return_type);
     }
     else
       implicit_typecast(code.op0(), return_type);
