@@ -26,6 +26,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #include "c_sizeof.h"
 #include "string_constant.h"
 #include "anonymous_member.h"
+#include "padding.h"
 
 /*******************************************************************\
 
@@ -296,56 +297,82 @@ void c_typecheck_baset::typecheck_expr_builtin_offsetof(exprt &expr)
       
         const struct_union_typet &struct_union_type=
           to_struct_union_type(type);
-        
-        if(!has_component_rec(struct_union_type, component_name, *this))
+          
+        // direct member?
+        if(struct_union_type.has_component(component_name))
         {
-          err_location(expr);    
-          throw "offset-of of member failed to find component `"+
-                id2string(component_name)+"' in `"+to_string(type)+"'";
-        }
+          found=true;
 
-        const struct_union_typet::componentst &components=
-          struct_union_type.components();
-        
-        for(struct_union_typet::componentst::const_iterator
-            c_it=components.begin();
-            c_it!=components.end();
-            c_it++)
-        {
-          if(c_it->get_name()==component_name)
-          {
-            typet tmp=follow(c_it->type());
-            type=tmp;
-            found=true;
-            break;
-          }
-          else if(c_it->get_anonymous())
-          {
-            if(has_component_rec(c_it->type(), component_name, *this))
-            {
-              typet tmp=follow(c_it->type());
-              type=tmp;
-              assert(type.id()==ID_union || type.id()==ID_struct);
-              break; // we run into another iteration of the outer loop
-            }
-          }
-         
           if(type.id()==ID_struct)
           {
-            exprt sub_size=c_sizeof(c_it->type(), *this);
+            exprt o=c_offsetof(to_struct_type(type), component_name, *this);
 
-            if(sub_size.is_nil())
+            if(o.is_nil())
             {
               err_location(expr);
-              str << "offsetof failed to determine size of `"
-                  << to_string(c_it->type()) << "'";
+              str << "offsetof failed to determine offset of `"
+                  << component_name << "'";
               throw 0;
             }
           
-            if(sub_size.type()!=size_type())
-              sub_size.make_typecast(size_type());
+            if(o.type()!=size_type())
+              o.make_typecast(size_type());
 
-            result=plus_exprt(result, sub_size);
+            result=plus_exprt(result, o);
+          }
+          
+          type=struct_union_type.get_component(component_name).type();
+        }
+        else
+        {
+          // maybe anonymous?
+          
+          const struct_union_typet::componentst &components=
+            struct_union_type.components();
+            
+          bool found2=false;
+        
+          for(struct_union_typet::componentst::const_iterator
+              c_it=components.begin();
+              c_it!=components.end();
+              c_it++)
+          {
+            if(c_it->get_anonymous())
+            {
+              if(has_component_rec(c_it->type(), component_name, *this))
+              {
+                if(type.id()==ID_struct)
+                {
+                  exprt o=c_offsetof(to_struct_type(type), c_it->get_name(), *this);
+
+                  if(o.is_nil())
+                  {
+                    err_location(expr);
+                    str << "offsetof failed to determine offset of `"
+                        << component_name << "'";
+                    throw 0;
+                  }
+                
+                  if(o.type()!=size_type())
+                    o.make_typecast(size_type());
+
+                  result=plus_exprt(result, o);
+                }
+
+                typet tmp=follow(c_it->type());
+                type=tmp;
+                assert(type.id()==ID_union || type.id()==ID_struct);
+                found2=true;
+                break; // we run into another iteration of the outer loop
+              }
+            }
+          }
+         
+          if(!found2)
+          {
+            err_location(expr);    
+            throw "offset-of of member failed to find component `"+
+                  id2string(component_name)+"' in `"+to_string(type)+"'";
           }
         }
       }
@@ -354,8 +381,7 @@ void c_typecheck_baset::typecheck_expr_builtin_offsetof(exprt &expr)
     {
       assert(m_it->operands().size()==1);
 
-      if(type.id()!=ID_array &&
-         type.id()!=ID_incomplete_array)
+      if(type.id()!=ID_array)
       {
         err_location(expr);
         throw "offsetof of index expects array type";
@@ -371,6 +397,7 @@ void c_typecheck_baset::typecheck_expr_builtin_offsetof(exprt &expr)
     }
   }
 
+  // we make an effort to produce a constant
   simplify(result, *this);
   result.location()=expr.location();
 
@@ -648,7 +675,7 @@ void c_typecheck_baset::typecheck_expr_sizeof(exprt &expr)
 
   new_expr.swap(expr);
 
-  expr.add("#c_sizeof_type")=type;
+  expr.add(ID_C_c_sizeof_type)=type;
 }
 
 /*******************************************************************\
@@ -677,7 +704,7 @@ void c_typecheck_baset::typecheck_expr_alignof(exprt &expr)
   }
 
   // we only care about the type
-  unsigned a=alignment(argument_type);
+  unsigned a=alignment(argument_type, *this);
   
   exprt tmp=from_integer(a, size_type());
   tmp.location()=expr.location();
@@ -702,18 +729,46 @@ void c_typecheck_baset::typecheck_expr_typecast(exprt &expr)
   if(expr.operands().size()!=1)
   {
     err_location(expr);
-    error("typecast operator expects one operand");
+    str << "typecast operator expects one operand";
     throw 0;
   }
 
   exprt &op=expr.op0();
 
   typecheck_type(expr.type());
+  
+  {
+    // first clean the type of any side-effects
+    std::list<codet> clean_code;
+    clean_type(irep_idt(), expr.type(), clean_code);
+
+    if(!clean_code.empty())
+    {
+      sideeffect_exprt sideeffect_expr(ID_statement_expression, empty_typet());
+      sideeffect_expr.copy_to_operands(code_blockt(clean_code));
+    
+      // We merge the side-effect into the operand, using
+      // a comma-expression.
+      // I.e., (type)e becomes (type)(side-effect, e)
+    
+      exprt comma_expr(ID_comma, op.type());
+      comma_expr.copy_to_operands(sideeffect_expr, op);
+      op.swap(comma_expr);
+    }
+  }
 
   const typet expr_type=follow(expr.type());
   
   if(expr_type.id()==ID_struct)
   {
+    // Already the same? Just remove.
+    if(base_type_eq(expr_type, op.type(), *this))
+    {
+      exprt tmp=op;
+      expr=tmp;
+      return;
+    }
+
     // typecast to struct: this is a GCC extension
     if(op.id()!=ID_initializer_list)
     {
@@ -721,10 +776,10 @@ void c_typecheck_baset::typecheck_expr_typecast(exprt &expr)
       str << "type cast to struct requires initializer_list argument";
       throw 0;
     }
-    
+  
     // just do a normal initialization    
     do_initializer(op, expr_type, false);
-    
+  
     // just remove typecast
     exprt tmp=op;
     expr=tmp;
@@ -733,9 +788,17 @@ void c_typecheck_baset::typecheck_expr_typecast(exprt &expr)
   }
   else if(expr_type.id()==ID_union)
   {
-    // this is a GCC extension. It's either a 'temporary union',
+    // Already the same? Just remove.
+    if(base_type_eq(expr_type, op.type(), *this))
+    {
+      exprt tmp=op;
+      expr=tmp;
+      return;
+    }
+
+    // This is a GCC extension. It's either a 'temporary union',
     // where the argument is expected to be a 'initializer_list',
-    // or a cast from one of the member types
+    // or a cast from one of the member types.
 
     if(op.id()==ID_initializer_list)
     {
@@ -769,14 +832,13 @@ void c_typecheck_baset::typecheck_expr_typecast(exprt &expr)
         return;
       }
     }
-
+    
     err_location(expr);
     str << "type cast to union either requires initializer list "
            "argument or an expression with a member type";
     throw 0;
   }
-  else if(expr_type.id()==ID_array ||
-          expr_type.id()==ID_incomplete_array)
+  else if(expr_type.id()==ID_array)
   {
     // this is a GCC extension called 'array constructor'
     // the argument is expected to be an 'initializer_list'
@@ -824,8 +886,7 @@ void c_typecheck_baset::typecheck_expr_typecast(exprt &expr)
      op_type.id()==ID_pointer)
   {
   }
-  else if(op_type.id()==ID_array ||
-          op_type.id()==ID_incomplete_array)
+  else if(op_type.id()==ID_array)
   {
     index_exprt index;
     index.array()=op;
@@ -922,10 +983,8 @@ void c_typecheck_baset::typecheck_expr_index(exprt &expr)
     const typet &index_full_type=follow(index_expr.type());
 
     if(array_full_type.id()!=ID_array &&
-       array_full_type.id()!=ID_incomplete_array &&
        array_full_type.id()!=ID_pointer &&
        (index_full_type.id()==ID_array ||
-        index_full_type.id()==ID_incomplete_array ||
         index_full_type.id()==ID_pointer))
       std::swap(array_expr, index_expr);
   }
@@ -934,8 +993,7 @@ void c_typecheck_baset::typecheck_expr_index(exprt &expr)
 
   const typet &final_array_type=follow(array_expr.type());
 
-  if(final_array_type.id()==ID_array ||
-     final_array_type.id()==ID_incomplete_array)
+  if(final_array_type.id()==ID_array)
   {
     if(array_expr.get_bool(ID_C_lvalue))
       expr.set(ID_C_lvalue, true);
@@ -1024,7 +1082,6 @@ void c_typecheck_baset::typecheck_expr_rel(exprt &expr)
     {
       const typet &final_type=follow(o_type0);
       if(final_type.id()!=ID_array &&
-         final_type.id()!=ID_incomplete_array &&
          final_type.id()!=ID_incomplete_struct)
       {
         adjust_float_rel(expr);
@@ -1470,8 +1527,7 @@ void c_typecheck_baset::typecheck_expr_dereference(exprt &expr)
 
   const typet op_type=follow(op.type());
 
-  if(op_type.id()==ID_array ||
-     op_type.id()==ID_incomplete_array)
+  if(op_type.id()==ID_array)
   {
     // *a is the same as a[0]
     expr.id(ID_index);
@@ -1481,12 +1537,6 @@ void c_typecheck_baset::typecheck_expr_dereference(exprt &expr)
   }
   else if(op_type.id()==ID_pointer)
   {
-    if(op_type.subtype().id()==ID_empty)
-    {
-      err_location(expr);
-      warning("operand of unary * is a void pointer");
-    }
-
     expr.type()=op_type.subtype();
   }
   else
@@ -1654,6 +1704,9 @@ void c_typecheck_baset::typecheck_side_effect_function_call(
       // We do a bit of return-type guessing, but just a bit
       typet return_type=int_type();
       
+      // The following isn't really right and sound, but there
+      // are too many idiots out there who use malloc and the like
+      // without the right header file.
       if(identifier=="c::malloc" ||
          identifier=="c::realloc" ||
          identifier=="c::reallocf" ||
@@ -1742,6 +1795,7 @@ void c_typecheck_baset::do_special_functions(
   side_effect_expr_function_callt &expr)
 {
   const exprt &f_op=expr.function();
+  const locationt &location=expr.location();
 
   // some built-in functions
   if(f_op.id()==ID_symbol)
@@ -1758,6 +1812,7 @@ void c_typecheck_baset::do_special_functions(
 
       exprt same_object_expr("same-object", bool_typet());
       same_object_expr.operands()=expr.arguments();
+      same_object_expr.location()=location;
       expr.swap(same_object_expr);
     }
     else if(identifier==CPROVER_PREFIX "buffer_size")
@@ -1770,6 +1825,7 @@ void c_typecheck_baset::do_special_functions(
 
       exprt buffer_size_expr("buffer_size", uint_type());
       buffer_size_expr.operands()=expr.arguments();
+      buffer_size_expr.location()=location;
       expr.swap(buffer_size_expr);
     }
     else if(identifier==CPROVER_PREFIX "is_zero_string")
@@ -1783,6 +1839,7 @@ void c_typecheck_baset::do_special_functions(
       exprt is_zero_string_expr("is_zero_string", bool_typet());
       is_zero_string_expr.operands()=expr.arguments();
       is_zero_string_expr.set(ID_C_lvalue, true); // make it an lvalue
+      is_zero_string_expr.location()=location;
       expr.swap(is_zero_string_expr);
     }
     else if(identifier==CPROVER_PREFIX "zero_string_length")
@@ -1796,6 +1853,7 @@ void c_typecheck_baset::do_special_functions(
       exprt zero_string_length_expr("zero_string_length", uint_type());
       zero_string_length_expr.operands()=expr.arguments();
       zero_string_length_expr.set(ID_C_lvalue, true); // make it an lvalue
+      zero_string_length_expr.location()=location;
       expr.swap(zero_string_length_expr);
     }
     else if(identifier==CPROVER_PREFIX "DYNAMIC_OBJECT")
@@ -1805,6 +1863,7 @@ void c_typecheck_baset::do_special_functions(
 
       exprt dynamic_object_expr=exprt(ID_dynamic_object, expr.type());
       dynamic_object_expr.operands()=expr.arguments();
+      dynamic_object_expr.location()=location;
       expr.swap(dynamic_object_expr);
     }
     else if(identifier==CPROVER_PREFIX "POINTER_OFFSET")
@@ -1814,6 +1873,7 @@ void c_typecheck_baset::do_special_functions(
 
       exprt pointer_offset_expr=exprt(ID_pointer_offset, expr.type());
       pointer_offset_expr.operands()=expr.arguments();
+      pointer_offset_expr.location()=location;
       expr.swap(pointer_offset_expr);
     }
     else if(identifier==CPROVER_PREFIX "POINTER_OBJECT")
@@ -1823,6 +1883,7 @@ void c_typecheck_baset::do_special_functions(
 
       exprt pointer_object_expr=exprt(ID_pointer_object, expr.type());
       pointer_object_expr.operands()=expr.arguments();
+      pointer_object_expr.location()=location;
       expr.swap(pointer_object_expr);
     }
     else if(identifier==CPROVER_PREFIX "isnan")
@@ -1835,6 +1896,7 @@ void c_typecheck_baset::do_special_functions(
 
       exprt isnan_expr(ID_isnan, bool_typet());
       isnan_expr.operands()=expr.arguments();
+      isnan_expr.location()=location;
       expr.swap(isnan_expr);
     }
     else if(identifier==CPROVER_PREFIX "isfinite")
@@ -1847,6 +1909,7 @@ void c_typecheck_baset::do_special_functions(
 
       exprt isfinite_expr(ID_isfinite, bool_typet());
       isfinite_expr.operands()=expr.arguments();
+      isfinite_expr.location()=location;
       expr.swap(isfinite_expr);
     }
     else if(identifier==CPROVER_PREFIX "abs" ||
@@ -1863,6 +1926,7 @@ void c_typecheck_baset::do_special_functions(
 
       exprt abs_expr(ID_abs, expr.type());
       abs_expr.operands()=expr.arguments();
+      abs_expr.location()=location;
       expr.swap(abs_expr);
     }
     else if(identifier==CPROVER_PREFIX "malloc")
@@ -1875,7 +1939,7 @@ void c_typecheck_baset::do_special_functions(
 
       exprt malloc_expr=side_effect_exprt(ID_malloc);
       malloc_expr.type()=expr.type();
-      malloc_expr.location()=expr.location();
+      malloc_expr.location()=location;
       malloc_expr.operands()=expr.arguments();
       expr.swap(malloc_expr);
     }
@@ -1889,6 +1953,7 @@ void c_typecheck_baset::do_special_functions(
 
       exprt isinf_expr(ID_isinf, bool_typet());
       isinf_expr.operands()=expr.arguments();
+      isinf_expr.location()=location;
       expr.swap(isinf_expr);
     }
     else if(identifier==CPROVER_PREFIX "isnormal")
@@ -1901,6 +1966,7 @@ void c_typecheck_baset::do_special_functions(
 
       exprt isnormal_expr(ID_isnormal, bool_typet());
       isnormal_expr.operands()=expr.arguments();
+      isnormal_expr.location()=location;
       expr.swap(isnormal_expr);
     }
     else if(identifier==CPROVER_PREFIX "sign")
@@ -1913,7 +1979,7 @@ void c_typecheck_baset::do_special_functions(
 
       exprt sign_expr(ID_sign, bool_typet());
       sign_expr.operands()=expr.arguments();
-      sign_expr.location()=expr.location();
+      sign_expr.location()=location;
       expr.swap(sign_expr);
     }
     else if(identifier==CPROVER_PREFIX "equal")
@@ -1924,9 +1990,9 @@ void c_typecheck_baset::do_special_functions(
         throw "equal expects two operands";
       }
       
-      equality_exprt equality_expr;
+      equal_exprt equality_expr;
       equality_expr.operands()=expr.arguments();
-      equality_expr.location()=expr.location();
+      equality_expr.location()=location;
       
       if(!base_type_eq(equality_expr.lhs().type(),
                        equality_expr.rhs().type(), *this))
@@ -1981,12 +2047,31 @@ void c_typecheck_baset::do_special_functions(
         throw "__builtin_constant_p expects one argument";
       }
 
+      // try to produce constant
       exprt tmp1=expr.arguments().front();
       simplify(tmp1, *this);
 
       exprt tmp2=from_integer(tmp1.is_constant(), expr.type());      
+      tmp2.location()=location;
       expr.swap(tmp2);
     }
+    else if(identifier==CPROVER_PREFIX "float_debug1" ||
+            identifier==CPROVER_PREFIX "float_debug2")
+    {
+      if(expr.arguments().size()!=2)
+      {
+        err_location(f_op);
+        throw "float_debug expects two operands";
+      }
+
+      const irep_idt &id=
+        identifier==CPROVER_PREFIX "float_debug1"?
+        "float_debug1":"float_debug2";
+      exprt float_debug_expr(id, expr.type());
+      float_debug_expr.operands()=expr.arguments();
+      float_debug_expr.location()=location;
+      expr.swap(float_debug_expr);
+    }
   }
 }
 
@@ -2062,7 +2147,7 @@ void c_typecheck_baset::typecheck_function_call_arguments(
       // don't know type, just do standard conversion
       
       const typet &type=follow(op.type());
-      if(type.id()==ID_array || type.id()==ID_incomplete_array)
+      if(type.id()==ID_array)
       {
         pointer_typet dest_type;
         dest_type.subtype()=empty_typet();
@@ -2166,6 +2251,12 @@ void c_typecheck_baset::typecheck_expr_unary_boolean(exprt &expr)
   exprt &operand=expr.op0();
 
   implicit_typecast_bool(operand);
+  
+  // This is not quite accurate: the standard says the result
+  // should be of type 'int'.
+  // We do 'bool' anyway to get more compact formulae. Eventually,
+  // this should be achieved by means of simplification, and not
+  // in the frontend.
   expr.type()=typet(ID_bool);
 }
 
@@ -2438,6 +2529,11 @@ void c_typecheck_baset::typecheck_expr_binary_boolean(exprt &expr)
   implicit_typecast_bool(expr.op0());
   implicit_typecast_bool(expr.op1());
 
+  // This is not quite accurate: the standard says the result
+  // should be of type 'int'.
+  // We do 'bool' anyway to get more compact formulae. Eventually,
+  // this should be achieved by means of simplification, and not
+  // in the frontend.
   expr.type()=typet(ID_bool);
 }
 
@@ -2485,6 +2581,13 @@ void c_typecheck_baset::typecheck_side_effect_assignment(exprt &expr)
   const typet &final_type0=follow(type0);
 
   expr.type()=type0;
+  
+  if(final_type0.id()==ID_empty)
+  {
+    err_location(expr);
+    str << "cannot assign void";
+    throw 0;
+  }
 
   if(!op0.get_bool(ID_C_lvalue))
   {
