@@ -22,6 +22,8 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <fixedbv.h>
 #include <prefix.h>
 #include <lispirep.h>
+#include <lispexpr.h>
+#include <symbol.h>
 
 #include "expr2c.h"
 #include "c_types.h"
@@ -301,6 +303,9 @@ std::string expr2ct::convert_rec(
   }
   else if(src.id()==ID_array)
   {
+    if(to_array_type(src).size().is_nil())
+      return convert(src.subtype())+" []";
+
     std::string size_string=convert(to_array_type(src).size());
     return convert(src.subtype())+" ["+size_string+"]";
   }
@@ -589,7 +594,7 @@ std::string expr2ct::convert_with(
         
       assert(comp_expr.is_not_nil());
         
-      op1=comp_expr.get_string(ID_pretty_name);
+      op1="."+comp_expr.get_string(ID_pretty_name);
       p1=10;
     }
     else
@@ -834,7 +839,7 @@ std::string expr2ct::convert_statement_expression(
   if(src.operands().size()!=1)
     return convert_norep(src, precedence);
 
-  return "{"+convert_code(to_code(src.op0()), 2)+"}";
+  return "({"+convert_code(to_code(src.op0()), 0)+"})";
 }
 
 /*******************************************************************\
@@ -1366,6 +1371,9 @@ std::string expr2ct::convert_symbol(
 
   if(ns_collision.find(id)==ns_collision.end())
     dest=id_shorthand(src);
+  else if(src.operands().size()==1 &&
+        src.op0().id()==ID_predicate_passive_symbol)
+    dest=src.op0().get(ID_identifier).as_string();
   else
     dest=id2string(id);
 
@@ -1437,6 +1445,26 @@ std::string expr2ct::convert_predicate_next_symbol(
 
 /*******************************************************************\
 
+Function: expr2ct::convert_predicate_passive_symbol
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+std::string expr2ct::convert_predicate_passive_symbol(
+  const exprt &src,
+  unsigned &precedence)
+{
+  const std::string &id=src.get_string(ID_identifier);
+  return "pps("+id+")";
+}
+
+/*******************************************************************\
+
 Function: expr2ct::convert_quantified_symbol
 
   Inputs:
@@ -1599,6 +1627,8 @@ std::string expr2ct::convert_constant(
       if(src.type()==float_type())
         dest+="f";
       else if(src.type()==double_type())
+        dest+=""; // ANSI-C: double is default
+      else if(src.type()==long_double_type())
         dest+="l";
     }
   }
@@ -2152,6 +2182,12 @@ std::string expr2ct::convert_overflow(
   std::string dest="overflow(\"";
   dest+=src.id().c_str()+9;
   dest+="\"";
+  
+  if(!src.operands().empty())
+  {
+    dest+=", ";
+    dest+=convert(src.op0().type());
+  }
 
   forall_operands(it, src)
   {
@@ -2189,7 +2225,7 @@ std::string expr2ct::indent_str(unsigned indent)
 
 /*******************************************************************\
 
-Function: expr2ct::convert_code_while
+Function: expr2ct::convert_code_asm
 
   Inputs:
 
@@ -2207,7 +2243,7 @@ std::string expr2ct::convert_code_asm(
   dest+="asm(";
   if(src.operands().size()==1)
     dest+=convert(src.op0());
-  dest+=");\n";
+  dest+=");";
   return dest;
 }
 
@@ -2224,7 +2260,7 @@ Function: expr2ct::convert_code_while
 \*******************************************************************/
 
 std::string expr2ct::convert_code_while(
-  const codet &src,
+  const code_whilet &src,
   unsigned indent)
 {
   if(src.operands().size()!=2)
@@ -2234,18 +2270,16 @@ std::string expr2ct::convert_code_while(
   }
 
   std::string dest=indent_str(indent);
-  dest+="while("+convert(src.op0());
+  dest+="while("+convert(src.cond());
 
-  if(src.op1().is_nil())
-    dest+=");\n";
+  if(src.body().is_nil())
+    dest+=");";
   else
   {
     dest+=")\n";
-    dest+=convert_code(to_code(src.op1()), indent+2);
+    dest+=convert_code(src.body(), indent+2);
   }
 
-  dest+="\n";
-
   return dest;
 }
 
@@ -2274,17 +2308,16 @@ std::string expr2ct::convert_code_dowhile(
   std::string dest=indent_str(indent);
 
   if(src.op1().is_nil())
-    dest+="do; ";
+    dest+="do;";
   else
   {
     dest+="do\n";
     dest+=convert_code(to_code(src.op1()), indent+2);
     dest+=indent_str(indent);
+    dest+="\n";
   }
 
-  dest+="while("+convert(src.op0())+");\n";
-
-  dest+="\n";
+  dest+="while("+convert(src.op0())+");";
 
   return dest;
 }
@@ -2302,7 +2335,7 @@ Function: expr2ct::convert_code_ifthenelse
 \*******************************************************************/
 
 std::string expr2ct::convert_code_ifthenelse(
-  const codet &src,
+  const code_ifthenelset &src,
   unsigned indent)
 {
   if(src.operands().size()!=3 &&
@@ -2313,26 +2346,23 @@ std::string expr2ct::convert_code_ifthenelse(
   }
 
   std::string dest=indent_str(indent);
-  dest+="if("+convert(src.op0())+")\n";
+  dest+="if("+convert(src.cond())+")\n";
 
-  if(src.op1().is_nil())
+  if(src.then_case().is_nil())
   {
     dest+=indent_str(indent+2);
     dest+=";\n";
   }
   else
-    dest+=convert_code(to_code(src.op1()), indent+2);
+    dest+=convert_code(src.then_case(), indent+2);
 
-  if(src.operands().size()==3 &&
-     !src.operands().back().is_nil())
+  if(src.operands().size()==3 && !src.else_case().is_nil())
   {
     dest+=indent_str(indent);
     dest+="else\n";
-    dest+=convert_code(to_code(src.operands().back()), indent+2);
+    dest+=convert_code(src.else_case(), indent+2);
   }
 
-  dest+="\n";
-
   return dest;
 }
 
@@ -2365,7 +2395,7 @@ std::string expr2ct::convert_code_return(
   if(src.operands().size()==1)
     dest+=" "+convert(src.op0());
 
-  dest+=";\n";
+  dest+=";";
 
   return dest;
 }
@@ -2389,7 +2419,7 @@ std::string expr2ct::convert_code_goto(
   std:: string dest=indent_str(indent);
   dest+="goto ";
   dest+=src.get_string(ID_destination);
-  dest+=";\n";
+  dest+=";";
 
   return dest;
 }
@@ -2412,7 +2442,7 @@ std::string expr2ct::convert_code_break(
 {
   std::string dest=indent_str(indent);
   dest+="break";
-  dest+=";\n";
+  dest+=";";
 
   return dest;
 }
@@ -2464,7 +2494,7 @@ std::string expr2ct::convert_code_switch(
   }
 
   dest+=indent_str(indent);
-  dest+="}\n";
+  dest+="}";
 
   return dest;
 }
@@ -2487,7 +2517,7 @@ std::string expr2ct::convert_code_continue(
 {
   std::string dest=indent_str(indent);
   dest+="continue";
-  dest+=";\n";
+  dest+=";";
 
   return dest;
 }
@@ -2526,7 +2556,7 @@ std::string expr2ct::convert_code_decl(
   if(src.operands().size()==2)
     dest+=" = "+convert(src.op1());
 
-  dest+=";\n";
+  dest+=";";
 
   return dest;
 }
@@ -2544,7 +2574,7 @@ Function: expr2ct::convert_code_for
 \*******************************************************************/
 
 std::string expr2ct::convert_code_for(
-  const codet &src,
+  const code_fort &src,
   unsigned indent)
 {
   if(src.operands().size()!=4)
@@ -2568,16 +2598,14 @@ std::string expr2ct::convert_code_for(
     if(i!=2) dest+=";";
   }
 
-  if(src.op3().is_nil())
+  if(src.body().is_nil())
     dest+=");\n";
   else
   {
     dest+=")\n";
-    dest+=convert_code(to_code(src.op3()), indent+2);
+    dest+=convert_code(src.body(), indent+2);
   }
 
-  dest+="\n";
-
   return dest;
 }
 
@@ -2594,23 +2622,25 @@ Function: expr2ct::convert_code_block
 \*******************************************************************/
 
 std::string expr2ct::convert_code_block(
-  const codet &src,
+  const code_blockt &src,
   unsigned indent)
 {
-  assert(indent>=2);
-  std::string dest=indent_str(indent-2);
+  assert(indent>=0);
+  std::string dest=indent_str(indent);
   dest+="{\n";
 
   forall_operands(it, src)
   {
-    if(it->get(ID_statement)==ID_block)
-      dest+=convert_code_block(to_code(*it), indent+2);
-    else
+    if(it->get(ID_statement)==ID_label)
       dest+=convert_code(to_code(*it), indent);
+    else
+      dest+=convert_code(to_code(*it), indent+2);
+
+    dest+="\n";
   }
 
-  dest+=indent_str(indent-2);
-  dest+="}\n";
+  dest+=indent_str(indent);
+  dest+="}";
 
   return dest;
 }
@@ -2643,8 +2673,6 @@ std::string expr2ct::convert_code_expression(
   }
 
   dest+=expr_str+";";
-
-  dest+="\n";
   return dest;
 }
 
@@ -2670,28 +2698,28 @@ std::string expr2ct::convert_code(
     return convert_code_expression(src, indent);
 
   if(statement==ID_block)
-    return convert_code_block(src, indent);
+    return convert_code_block(to_code_block(src), indent);
 
   if(statement==ID_switch)
     return convert_code_switch(src, indent);
 
   if(statement==ID_for)
-    return convert_code_for(src, indent);
+    return convert_code_for(to_code_for(src), indent);
 
   if(statement==ID_while)
-    return convert_code_while(src, indent);
+    return convert_code_while(to_code_while(src), indent);
 
   if(statement==ID_asm)
     return convert_code_asm(src, indent);
 
   if(statement==ID_skip)
-    return indent_str(indent)+";\n";
+    return indent_str(indent)+";";
 
   if(statement==ID_dowhile)
     return convert_code_dowhile(src, indent);
 
   if(statement==ID_ifthenelse)
-    return convert_code_ifthenelse(src, indent);
+    return convert_code_ifthenelse(to_code_ifthenelse(src), indent);
 
   if(statement==ID_return)
     return convert_code_return(src, indent);
@@ -2721,7 +2749,7 @@ std::string expr2ct::convert_code(
     return convert_code_decl(src, indent);
 
   if(statement==ID_assign)
-    return convert_code_assign(src, indent);
+    return convert_code_assign(to_code_assign(src), indent);
 
   if(statement==ID_init)
     return convert_code_init(src, indent);
@@ -2736,7 +2764,7 @@ std::string expr2ct::convert_code(
     return convert_code_function_call(to_code_function_call(src), indent);
 
   if(statement==ID_label)
-    return convert_code_label(src, indent);
+    return convert_code_label(to_code_label(src), indent);
 
   if(statement==ID_free)
     return convert_code_free(src, indent);
@@ -2764,12 +2792,12 @@ Function: expr2ct::convert_code_assign
 \*******************************************************************/
 
 std::string expr2ct::convert_code_assign(
-  const codet &src,
+  const code_assignt &src,
   unsigned indent)
 {
   std::string tmp=convert_binary(src, "=", 2, true);
 
-  std::string dest=indent_str(indent)+tmp+";\n";
+  std::string dest=indent_str(indent)+tmp+";";
 
   return dest;
 }
@@ -2892,7 +2920,7 @@ std::string expr2ct::convert_code_function_call(
     return convert_norep(src, precedence);
   }
 
-  std::string dest;
+  std::string dest=indent_str(indent);
 
   if(src.lhs().is_not_nil())
   {
@@ -3128,26 +3156,26 @@ Function: expr2ct::convert_code_label
 \*******************************************************************/
 
 std::string expr2ct::convert_code_label(
-  const codet &src,
+  const code_labelt &src,
   unsigned indent)
 {
   bool first=true;
   std::string labels_string;
 
   {
-    const irept::named_subt &labels=src.find(ID_labels).get_named_sub();
-
-    forall_named_irep(it, labels)
+    irep_idt label=src.get_label();
+    
+    if(label!=irep_idt())
     {
       if(first) { labels_string+="\n"; first=false; }
       labels_string+=indent_str(indent);
-      labels_string+=name2string(it->first);
+      labels_string+=name2string(label);
       labels_string+=":\n";
     }
 
-    const exprt &case_expr=(exprt &)src.find(ID_case);
+    const exprt::operandst &case_op=src.case_op();
 
-    forall_operands(it, case_expr)
+    forall_expr(it, case_op)
     {
       if(first) { labels_string+="\n"; first=false; }
       labels_string+=indent_str(indent);
@@ -3156,7 +3184,7 @@ std::string expr2ct::convert_code_label(
       labels_string+=":\n";
     }
 
-    if(src.get_bool(ID_default))
+    if(src.is_default())
     {
       if(first) { labels_string+="\n"; first=false; }
       labels_string+=indent_str(indent);
@@ -3170,7 +3198,7 @@ std::string expr2ct::convert_code_label(
     return convert_norep(src, precedence);
   }
 
-  std::string tmp=convert_code(to_code(src.op0()), indent);
+  std::string tmp=convert_code(src.code(), indent+2);
 
   return labels_string+tmp;
 }
@@ -3189,7 +3217,7 @@ Function: expr2ct::convert_code
 
 std::string expr2ct::convert_code(const codet &src)
 {
-  return convert_code(src, 2);
+  return convert_code(src, 0);
 }
 
 /*******************************************************************\
@@ -3276,6 +3304,35 @@ std::string expr2ct::convert_extractbit(
 
 /*******************************************************************\
 
+Function: expr2ct::convert_extractbits
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+std::string expr2ct::convert_extractbits(
+  const exprt &src,
+  unsigned precedence)
+{
+  if(src.operands().size()!=3)
+    return convert_norep(src, precedence);
+
+  std::string dest=convert(src.op0(), precedence);
+  dest+='[';
+  dest+=convert(src.op1(), precedence);
+  dest+=", ";
+  dest+=convert(src.op2(), precedence);
+  dest+=']';
+
+  return dest;
+}
+
+/*******************************************************************\
+
 Function: expr2ct::convert
 
   Inputs:
@@ -3589,15 +3646,12 @@ std::string expr2ct::convert(
   else if(src.id()==ID_width)
     return convert_function(src, "WIDTH", precedence=15);
 
+  else if(src.id()==ID_concatenation)
+    return convert_function(src, "CONCATENATION", precedence=15);
+
   else if(src.id()==ID_ieee_float_notequal)
     return convert_function(src, "IEEE_FLOAT_NOTEQUAL", precedence=15);
 
-  else if(src.id()==ID_byte_update_little_endian)
-    return convert_function(src, "BYTE_UPDATE_LITTLE_ENDIAN", precedence=15);
-
-  else if(src.id()==ID_byte_update_big_endian)
-    return convert_function(src, "BYTE_UPDATE_BIG_ENDIAN", precedence=15);
-
   else if(src.id()==ID_abs)
     return convert_function(src, "abs", precedence=15);
 
@@ -3643,19 +3697,22 @@ std::string expr2ct::convert(
   else if(src.id()==ID_nondet_symbol)
     return convert_nondet_symbol(src, precedence);
 
-  else if(src.id()=="predicate_symbol")
+  else if(src.id()==ID_predicate_symbol)
     return convert_predicate_symbol(src, precedence);
 
-  else if(src.id()=="predicate_next_symbol")
+  else if(src.id()==ID_predicate_next_symbol)
     return convert_predicate_next_symbol(src, precedence);
 
-  else if(src.id()=="quantified_symbol")
+  else if(src.id()==ID_predicate_passive_symbol)
+    return convert_predicate_passive_symbol(src, precedence);
+
+  else if(src.id()=="quant_symbol")
     return convert_quantified_symbol(src, precedence);
 
-  else if(src.id()=="nondet_bool")
+  else if(src.id()==ID_nondet_bool)
     return convert_nondet_bool(src, precedence);
 
-  else if(src.id()=="object_descriptor")
+  else if(src.id()==ID_object_descriptor)
     return convert_object_descriptor(src, precedence);
 
   else if(src.id()=="Hoare")
@@ -3700,18 +3757,24 @@ std::string expr2ct::convert(
   else if(src.id()==ID_cond)
     return convert_cond(src, precedence);
 
-  else if(std::string(src.id_string(), 0, 9)=="overflow-")
+  else if(src.id()==ID_overflow_unary_minus ||
+      src.id()==ID_overflow_minus ||
+      src.id()==ID_overflow_mult ||
+      src.id()==ID_overflow_plus)
     return convert_overflow(src, precedence);
 
   else if(src.id()==ID_unknown)
     return "*";
 
-  else if(src.id()=="invalid")
+  else if(src.id()==ID_invalid)
     return "#";
 
   else if(src.id()==ID_extractbit)
     return convert_extractbit(src, precedence);
 
+  else if(src.id()==ID_extractbits)
+    return convert_extractbits(src, precedence);
+
   else if(src.id()==ID_initializer_list)
     return convert_initializer_list(src, precedence=15);
 
