@@ -49,6 +49,7 @@ void ansi_c_internal_additions(std::string &code)
   code+=
     "typedef __typeof__(sizeof(int)) __CPROVER_size_t;\n"
     "void __CPROVER_assume(_Bool assumption);\n"
+    "void __VERIFIER_assume(_Bool assumption);\n"
     "void assert(_Bool assertion);\n"
     "void __CPROVER_assert(_Bool assertion, const char *description);\n"
     "_Bool __CPROVER_equal();\n"
@@ -81,6 +82,7 @@ void ansi_c_internal_additions(std::string &code)
     "const void *__CPROVER_deallocated=0;\n"
     "const void *__CPROVER_malloc_object=0;\n"
     "__CPROVER_size_t __CPROVER_malloc_size;\n"
+    "_Bool __CPROVER_malloc_is_new_array=0;\n" // for the benefit of C++
 
     // this is ANSI-C
     "extern __CPROVER_thread_local const char __func__[__CPROVER_constant_infinity_uint];\n"
@@ -115,6 +117,8 @@ void ansi_c_internal_additions(std::string &code)
     
 	// manual specification of predicates
     "void __CPROVER_predicate(_Bool predicate);\n"
+	"void __CPROVER_parameter_predicates();\n"
+	"void __CPROVER_return_predicates();\n"
 
     // GCC junk stuff
     GCC_BUILTIN_HEADERS
@@ -134,7 +138,7 @@ void ansi_c_internal_additions(std::string &code)
 
 /*******************************************************************\
 
-Function: architecture_string
+Function: architecture_strings
 
   Inputs:
 
