@@ -8,12 +8,14 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <fstream>
 #include <memory>
+#include <cstdlib>
 
 #include <config.h>
 #include <expr_util.h>
+#include <language.h>
 
 #include <goto-programs/goto_convert_functions.h>
-#include <goto-programs/goto_function_pointers.h>
+#include <goto-programs/remove_function_pointers.h>
 #include <goto-programs/goto_check.h>
 #include <goto-programs/goto_inline.h>
 #include <goto-programs/show_claims.h>
@@ -157,11 +159,11 @@ void cbmc_parseoptionst::get_command_line_options(optionst &options)
   if(cmdline.isset("unwindset"))
     options.set_option("unwindset", cmdline.getval("unwindset"));
 
-  // substitution previous expressions
-  if(cmdline.isset("no-substitution"))
-    options.set_option("substitution", false);
+  // constant propagation
+  if(cmdline.isset("no-propagation"))
+    options.set_option("propagation", false);
   else
-    options.set_option("substitution", true);
+    options.set_option("propagation", true);
 
   // check array bounds
   if(cmdline.isset("bounds-check"))
@@ -176,10 +178,16 @@ void cbmc_parseoptionst::get_command_line_options(optionst &options)
     options.set_option("div-by-zero-check", false);
 
   // check overflow/underflow
-  if(cmdline.isset("overflow-check"))
-    options.set_option("overflow-check", true);
+  if(cmdline.isset("signed-overflow-check"))
+    options.set_option("signed-overflow-check", true);
   else
-    options.set_option("overflow-check", false);
+    options.set_option("signed-overflow-check", false);
+
+  // check overflow/underflow
+  if(cmdline.isset("unsigned-overflow-check"))
+    options.set_option("unsigned-overflow-check", true);
+  else
+    options.set_option("unsigned-overflow-check", false);
 
   // check for NaN (not a number)
   if(cmdline.isset("nan-check"))
@@ -243,6 +251,9 @@ void cbmc_parseoptionst::get_command_line_options(optionst &options)
   if(cmdline.isset("boolector"))
     options.set_option("boolector", true);
 
+  if(cmdline.isset("mathsat"))
+    options.set_option("mathsat", true);
+
   if(cmdline.isset("cvc"))
     options.set_option("cvc", true);
 
@@ -252,6 +263,9 @@ void cbmc_parseoptionst::get_command_line_options(optionst &options)
   if(cmdline.isset("smt2"))
     options.set_option("smt2", true);
 
+  if(cmdline.isset("fpa"))
+    options.set_option("fpa", true);
+
   if(cmdline.isset("yices"))
     options.set_option("yices", true);
 
@@ -306,13 +320,14 @@ int cbmc_parseoptionst::doit()
   
   register_languages();
 
-  bmct bmc(context, ui_message_handler);
-
   //
   // command line options
   //
 
-  get_command_line_options(bmc.options);
+  optionst options;
+  get_command_line_options(options);
+
+  bmct bmc(options, context, ui_message_handler);
   set_verbosity(bmc);
   set_verbosity(*this);
   
@@ -324,7 +339,7 @@ int cbmc_parseoptionst::doit()
 
   goto_functionst goto_functions;
 
-  if(get_goto_program(bmc, goto_functions))
+  if(get_goto_program(options, bmc, goto_functions))
     return 6;
 
   if(cmdline.isset("show-claims"))
@@ -394,7 +409,8 @@ Function: cbmc_parseoptionst::get_goto_program
 \*******************************************************************/
   
 bool cbmc_parseoptionst::get_goto_program(
-  bmc_baset &bmc,
+  const optionst &options,
+  bmct &bmc,
   goto_functionst &goto_functions)
 {
   if(cmdline.args.size()==0)
@@ -453,23 +469,16 @@ bool cbmc_parseoptionst::get_goto_program(
       status("Generating GOTO Program");
 
       goto_convert(
-        context, bmc.options, goto_functions,
+        context, options, goto_functions,
         ui_message_handler);
-        
-      // finally add the library
-      status("Adding CPROVER library");      
-      link_to_library(
-        context, goto_functions, bmc.options, ui_message_handler);
     }
 
-    if(cmdline.isset("interpreter"))
-    {
-      status("Starting interpreter");
-      interpreter(context, goto_functions);
-      return true;
-    }
+    // finally add the library
+    status("Adding CPROVER library");      
+    link_to_library(
+      context, goto_functions, options, ui_message_handler);
 
-    if(process_goto_program(bmc, goto_functions))
+    if(process_goto_program(options, goto_functions))
       return true;
   }
 
@@ -579,7 +588,7 @@ Function: cbmc_parseoptionst::process_goto_program
 \*******************************************************************/
   
 bool cbmc_parseoptionst::process_goto_program(
-  bmc_baset &bmc,
+  const optionst &options,
   goto_functionst &goto_functions)
 {
   try
@@ -591,7 +600,8 @@ bool cbmc_parseoptionst::process_goto_program(
         context, get_message_handler(), goto_functions);
 
     status("Function Pointer Removal");
-    remove_function_pointers(ns, goto_functions);
+    remove_function_pointers(ns, goto_functions,
+      cmdline.isset("pointer-check"));
 
     status("Partial Inlining");
     // do partial inlining
@@ -599,7 +609,7 @@ bool cbmc_parseoptionst::process_goto_program(
     
     status("Generic Property Instrumentation");
     // add generic checks
-    goto_check(ns, bmc.options, goto_functions);
+    goto_check(ns, options, goto_functions);
     
     if(cmdline.isset("string-abstraction"))
     {
@@ -630,7 +640,7 @@ bool cbmc_parseoptionst::process_goto_program(
 
       // add pointer checks
       pointer_checks(
-        goto_functions, ns, bmc.options, value_set_analysis);
+        goto_functions, context, options, value_set_analysis);
     }
 
     // recalculate numbers, etc.
@@ -693,7 +703,7 @@ Function: cbmc_parseoptionst::do_bmc
 \*******************************************************************/
 
 int cbmc_parseoptionst::do_bmc(
-  bmc_baset &bmc,
+  bmct &bmc,
   const goto_functionst &goto_functions)
 {
   bmc.set_ui(get_ui());
@@ -721,7 +731,13 @@ void cbmc_parseoptionst::help()
 {
   std::cout <<
     "\n"
-    "* *            CBMC " CBMC_VERSION " - Copyright (C) 2001-2008           * *\n"
+    "* *   CBMC " CBMC_VERSION " - Copyright (C) 2001-2011 ";
+    
+  std::cout << "(" << (sizeof(void *)*8) << "-bit version)";
+    
+  std::cout << "   * *\n";
+    
+  std::cout <<
     "* *              Daniel Kroening, Edmund Clarke             * *\n"
     "* * Carnegie Mellon University, Computer Science Department * *\n"
     "* *                 kroening@kroening.com                   * *\n"
@@ -769,13 +785,13 @@ void cbmc_parseoptionst::help()
     " --round-to-plus-inf          IEEE floating point rounding mode\n"
     " --round-to-minus-inf         IEEE floating point rounding mode\n"
     " --round-to-zero              IEEE floating point rounding mode\n"
-    " --interpreter                do concrete execution\n"
     "\n"
     "Program instrumentation options:\n"
     " --bounds-check               enable array bounds checks\n"
     " --div-by-zero-check          enable division by zero checks\n"
     " --pointer-check              enable pointer checks\n"
-    " --overflow-check             enable arithmetic over- and underflow checks\n"
+    " --signed-overflow-check      enable arithmetic over- and underflow checks\n"
+    " --unsigned-overflow-check    enable arithmetic over- and underflow checks\n"
     " --nan-check                  check floating-point for NaN\n"
     " --all-claims                 keep all claims\n"
     " --show-claims                only show claims\n"
@@ -803,6 +819,7 @@ void cbmc_parseoptionst::help()
     " --smt1                       output subgoals in SMT1 syntax (experimental)\n"
     " --smt2                       output subgoals in SMT2 syntax (experimental)\n"
     " --boolector                  use Boolector (experimental)\n"
+    " --mathsat                    use MathSAT (experimental)\n"
     " --cvc                        use CVC3 (experimental)\n"
     " --yices                      use Yices (experimental)\n"
     " --z3                         use Z3 (experimental)\n"
