@@ -121,14 +121,15 @@ protected:
   void convert_function(symbolt &symbol);
 
   void convert_pmop(exprt& expr);
-  void convert_anonymous_union(
-        cpp_declarationt &declaration,
-        codet& new_code);
 
-  void convert_compound_ano_union(
-       const cpp_declarationt &declaration,
-       const irep_idt &access,
-       struct_typet::componentst &components);
+  void convert_anonymous_union(
+    cpp_declarationt &declaration,
+    codet &new_code);
+
+  void convert_anon_struct_union_member(
+    const cpp_declarationt &declaration,
+    const irep_idt &access,
+    struct_typet::componentst &components);
 
   //
   // Templates
@@ -151,14 +152,16 @@ protected:
 
   void typecheck_template_class(cpp_declarationt &declaration);
 
-  void typecheck_template_function(cpp_declarationt &declaration);
+  void typecheck_function_template(cpp_declarationt &declaration);
+
+  void typecheck_template_member_function(cpp_declarationt &declaration);
 
   std::string template_class_identifier(
     const irep_idt &base_name,
     const template_typet &template_type,
     const cpp_template_args_non_tct &partial_specialization_args);
     
-  std::string template_function_identifier(
+  std::string function_template_identifier(
     const irep_idt &base_name,
     const template_typet &template_type,
     const typet &function_type);
@@ -342,7 +345,7 @@ protected:
   cpp_scopet &typecheck_template_parameters(
     template_typet &type);
 
-  void typecheck_compound_type(typet &type);
+  void typecheck_compound_type(struct_union_typet &type);
   void check_array_types(typet &type);
   void typecheck_enum_type(typet &type);
 
@@ -373,6 +376,7 @@ protected:
   void typecheck_enum_body(symbolt &symbol);
   void typecheck_function_bodies();
   void typecheck_compound_bases(struct_typet &type);
+  void add_anonymous_members_to_scope(const symbolt &struct_union_symbol);
 
   void move_member_initializers(
     irept &initializers,
