@@ -6,8 +6,9 @@ Author:
 
 \*******************************************************************/
 
+#include <cstdlib>
+
 #include <config.h>
-#include <bitvector.h>
 #include <arith_tools.h>
 #include <expr_util.h>
 #include <std_types.h>
@@ -53,10 +54,10 @@ bool cpp_typecheckt::standard_conversion_lvalue_to_rvalue(
 {
   assert(expr.get_bool(ID_C_lvalue));
 
-  if(expr.type().id()== ID_code ||
-     expr.type().id()== ID_incomplete_array || 
-     expr.type().id()== ID_incomplete_struct ||
-     expr.type().id()== ID_incomplete_union)
+  if(expr.type().id()==ID_code ||
+     expr.type().id()==ID_incomplete_array || 
+     expr.type().id()==ID_incomplete_struct ||
+     expr.type().id()==ID_incomplete_union)
     return false;
 
   new_expr=expr;
@@ -213,7 +214,6 @@ Function: standard_conversion_integral_promotion
   Outputs: True iff the integral pormotion is possible.
            The result of the conversion is stored in 'new_expr'.
 
-
   Purpose:  Integral-promotion conversion
 
   An rvalue of type char, signed char, unsigned char, short int,
@@ -255,7 +255,7 @@ bool cpp_typecheckt::standard_conversion_integral_promotion(
 
   if(expr.type().id()==ID_signedbv)
   {
-    unsigned width = bv_width(expr.type());
+    unsigned width=to_signedbv_type(expr.type()).get_width();
     if(width >= config.ansi_c.int_width)
       return false;
     new_expr = expr;
@@ -265,7 +265,7 @@ bool cpp_typecheckt::standard_conversion_integral_promotion(
 
   if(expr.type().id()==ID_unsignedbv)
   {
-    unsigned width=bv_width(expr.type());
+    unsigned width=to_unsignedbv_type(expr.type()).get_width();
     if(width >= config.ansi_c.int_width)
       return false;
     new_expr = expr;
@@ -285,7 +285,6 @@ bool cpp_typecheckt::standard_conversion_integral_promotion(
   return false;
 }
 
-
 /*******************************************************************\
 
 Function: standard_conversion_floating_point_promotion
@@ -314,7 +313,7 @@ bool cpp_typecheckt::standard_conversion_floating_point_promotion(
   if(expr.type()!=float_type())
     return false;
 
-  unsigned width=bv_width(expr.type());
+  unsigned width=to_floatbv_type(expr.type()).get_width();
 
   if(width!=config.ansi_c.single_width)
     return false;
@@ -339,7 +338,6 @@ Function: standard_conversion_integral_conversion
   Outputs: True iff the integral pormotion is possible.
            The result of the conversion is stored in 'new_expr'.
 
-
   Purpose:  Integral conversion
 
   An rvalue of type char, signed char, unsigned char, short int,
@@ -748,19 +746,19 @@ bool cpp_typecheckt::standard_conversion_boolean(
   if(expr.get_bool(ID_C_lvalue))
     return false;
 
-  if(expr.type().id() != ID_signedbv
-     && expr.type().id() != ID_unsignedbv
-     && expr.type().id() != ID_pointer
-     && follow(expr.type()).id() != ID_c_enum)
-      return false;
+  if(expr.type().id() != ID_signedbv &&
+     expr.type().id() != ID_unsignedbv &&
+     expr.type().id() != ID_pointer && 
+     follow(expr.type()).id() != ID_c_enum)
+    return false;
 
   c_qualifierst qual_from;
   qual_from.read(expr.type());
 
-  typet Bool(ID_bool);
+  bool_typet Bool;
   qual_from.write(Bool);
 
-  new_expr = expr;
+  new_expr=expr;
   new_expr.make_typecast(Bool);
   return true;
 }
@@ -904,7 +902,7 @@ bool cpp_typecheckt::standard_conversion_sequence(
   curr_expr.swap(new_expr);
 
   // need to consider #cpp_type
-  if(curr_expr.type()!=type ||
+  if(follow(curr_expr.type())!=follow(type) ||
      curr_expr.type().get(ID_C_cpp_type)!=type.get(ID_C_cpp_type))
   {
     if(type.id()==ID_signedbv ||
@@ -2120,8 +2118,8 @@ bool cpp_typecheckt::reinterpret_typecast(
         return false;
     }
 
-    if(e.type().id() == ID_array ||
-       e.type().id() == ID_incomplete_array )
+    if(e.type().id()==ID_array ||
+       e.type().id()==ID_incomplete_array)
     {
       if(standard_conversion_array_to_pointer(e,tmp))
         e.swap(tmp);
