@@ -6,6 +6,8 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 \*******************************************************************/
 
+#include <cstdlib>
+
 #include <expr_util.h>
 #include <i2string.h>
 #include <std_types.h>
@@ -23,6 +25,7 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 #include "cpp_typecheck.h"
 #include "cpp_convert_type.h"
 #include "cpp_class_type.h"
+#include "cpp_exception_id.h"
 #include "expr2cpp.h"
 
 /*******************************************************************\
@@ -355,14 +358,14 @@ Purpose:
 
 void cpp_typecheckt::typecheck_expr_sizeof(exprt &expr)
 {
-  // we need to overload, as "sizeof-type" may be
-  // parsed as an expression!
-
+  // We need to overload, "sizeof-expression" can be mis-parsed
+  // as a type.
+  
   if(expr.operands().size()==0)
   {
     const typet &type=
       static_cast<const typet &>(expr.find(ID_type_arg));
-
+      
     if(type.id()==ID_cpp_name)
     {
       // this may be ambiguous -- it can be either a type or
@@ -381,8 +384,30 @@ void cpp_typecheckt::typecheck_expr_sizeof(exprt &expr)
         expr.remove(ID_type_arg);
       }
     }
-  }
+    else if(type.id()==ID_array)
+    {
+      // sizeof(expr[index]) can be parsed as an array type!
+      
+      if(type.subtype().id()==ID_cpp_name)
+      {
+        cpp_typecheck_fargst fargs;
 
+        exprt symbol_expr=resolve(
+          to_cpp_name(static_cast<const irept &>(type.subtype())),
+          cpp_typecheck_resolvet::BOTH,
+          fargs);
+
+        if(symbol_expr.id()!=ID_type)
+        {
+          // _NOT_ a type
+          index_exprt index_expr(symbol_expr, to_array_type(type).size());
+          expr.copy_to_operands(index_expr);
+          expr.remove(ID_type_arg);
+        }
+      }
+    }
+  }
+  
   c_typecheck_baset::typecheck_expr_sizeof(expr);
 }
 
@@ -859,14 +884,20 @@ void cpp_typecheckt::typecheck_expr_throw(exprt &expr)
   assert(expr.operands().size()==1 ||
          expr.operands().size()==0);
 
-  // nothing really to do; one can throw _almost_ anything
   if(expr.operands().size()==1)
   {
-    if(follow(expr.op0().type()).id()==ID_empty)
+    // nothing really to do; one can throw _almost_ anything
+    const typet &exception_type=expr.op0().type();
+  
+    if(follow(exception_type).id()==ID_empty)
     {
       err_location(expr.op0());
       throw "cannot throw void";
     }
+    
+    // annotate the relevant exception IDs
+    expr.set(ID_exception_list, 
+             cpp_exception_list(exception_type, *this));
   }
 }
 
@@ -2240,7 +2271,7 @@ void cpp_typecheckt::typecheck_side_effect_assignment(exprt &expr)
   if(expr.operands().size()!=2)
     throw "assignment side-effect expected to have two operands";
     
-  typet type0(expr.op0().type());
+  typet type0=expr.op0().type();
 
   if(is_reference(type0))
     type0=type0.subtype();
@@ -2268,7 +2299,6 @@ void cpp_typecheckt::typecheck_side_effect_assignment(exprt &expr)
       int left  = atoi(extractbits.op1().get(ID_C_cformat).c_str());
       int right = atoi(extractbits.op2().get(ID_C_cformat).c_str());
 
-
       std::string mask;
 
       for(int i = 0; i < right; i++)
@@ -2417,6 +2447,7 @@ void cpp_typecheckt::typecheck_side_effect_assignment(exprt &expr)
     return;
   }
 
+  // It's a non-POD.
   // Turn into an operator call
 
   std::string strop="operator";
