@@ -103,8 +103,6 @@ bool cmdline_optionst::doit()
 
   compilet compiler(cmdline);
 
-  transform(my_name.begin(), my_name.end(), my_name.begin(), tolower);
-
   if(has_prefix(my_name, "ld") ||
      has_prefix(my_name, "goto-ld") ||
      has_prefix(my_name, "link") ||
@@ -150,7 +148,7 @@ bool cmdline_optionst::doit()
 
   if(cmdline.isset("verbosity"))
     verbosity=atoi(cmdline.getval("verbosity"));
-  
+
   compiler.set_verbosity(verbosity);
   set_verbosity(verbosity);
 
@@ -330,6 +328,8 @@ bool cmdline_optionst::doit()
 
     compiler.doLink=!( cmdline.isset('E') || cmdline.isset('P') ||
                        cmdline.isset('c') );
+                       
+    compiler.echo_file_name=true;
 
     if(cmdline.isset("Fo"))
     {
@@ -525,7 +525,7 @@ int cmdline_optionst::main(int argc, const char **argv)
   try
   {
     if(doit())
-      return EX_USAGE;
+      return EX_USAGE; // error
     else
       return EX_OK;
   }
