@@ -8,26 +8,10 @@ Date: June 2006
 
 \*******************************************************************/
 
-#ifdef __linux__
-#include <unistd.h>
-#include <errno.h>
-#include <dirent.h>
-#endif
-
-#ifdef __MACH__
-#include <unistd.h>
-#include <errno.h>
-#include <dirent.h>
-#endif
-
-#ifdef __CYGWIN__
-#include <unistd.h>
-#include <errno.h>
-#include <dirent.h>
-#endif
-
 #include <fstream>
 #include <sstream>
+#include <cstdlib>
+#include <algorithm>
 
 #include <config.h>
 #include <tempdir.h>
@@ -35,6 +19,8 @@ Date: June 2006
 #include <base_type.h>
 #include <xml.h>
 #include <i2string.h>
+#include <cmdline.h>
+#include <file_util.h>
 
 #include <ansi-c/ansi_c_language.h>
 #include <ansi-c/c_link_class.h>
@@ -61,16 +47,6 @@ Date: June 2006
 #include "compile.h"
 #include "version.h"
 
-#ifdef _WIN32
-#include <io.h>
-#include <windows.h>
-#include <direct.h>
-#include <errno.h>
-#define chdir _chdir
-#define popen _popen
-#define pclose _pclose
-#endif
-
 #define DOTGRAPHSETTINGS  "color=black;" \
                           "orientation=portrait;" \
                           "fontsize=20;"\
@@ -80,6 +56,29 @@ Date: June 2006
 
 unsigned compilet::subgraphscount;
 
+// the following are for chdir
+
+#ifdef __linux__
+#include <unistd.h>
+#endif
+
+#ifdef __FreeBSD_kernel__
+#include <unistd.h>
+#endif
+
+#ifdef __MACH__
+#include <unistd.h>
+#endif
+
+#ifdef __CYGWIN__
+#include <unistd.h>
+#endif
+
+#ifdef _WIN32
+#include <direct.h>
+#define chdir _chdir
+#endif
+
 /*******************************************************************\
 
 Function: compilet::doit
@@ -115,7 +114,7 @@ bool compilet::doit()
 
   add_compiler_specific_defines(config);
 
-  // Parse commandline for source and object filenames
+  // Parse commandline for source and object file names
   for(unsigned i=0; i<_cmdline.args.size(); i++)
     if(add_input_file(_cmdline.args[i]))
       return true;
@@ -174,18 +173,18 @@ Function: compilet::add_input_file
 
  Outputs: false on success, true on error.
 
- Purpose: puts input filenames into a list and does preprocessing for
+ Purpose: puts input file names into a list and does preprocessing for
           libraries.
 
 \*******************************************************************/
 
-bool compilet::add_input_file(const std::string &filename)
+bool compilet::add_input_file(const std::string &file_name)
 {
-  size_t r=filename.rfind('.', filename.length()-1);
+  size_t r=file_name.rfind('.', file_name.length()-1);
 
   if(r!=std::string::npos)
   {
-    std::string ext = filename.substr(r+1, filename.length());
+    std::string ext = file_name.substr(r+1, file_name.length());
 
     if(ext=="a")
     {
@@ -195,11 +194,11 @@ bool compilet::add_input_file(const std::string &filename)
       char td[] = "goto-cc.XXXXXX";
       #endif
 
-      std::string tstr = get_temporary_directory(td);
+      std::string tstr=get_temporary_directory(td);
 
       if(tstr=="")
       {
-        error("Cannot create temporary directory.");
+        error("Cannot create temporary directory");
         return true;
       }
 
@@ -207,30 +206,30 @@ bool compilet::add_input_file(const std::string &filename)
       std::stringstream cmd("");
       if(chdir(tmp_dirs.back().c_str())!=0)
       {
-        error("Cannot switch to temporary directory.");
+        error("Cannot switch to temporary directory");
         return true;
       }
       
       #ifdef _WIN32
-      if (filename[0]!='/' && filename[1]!=':')
+      if(file_name[0]!='/' && file_name[1]!=':')
       #else
-      if (filename[0]!='/')
+      if(file_name[0]!='/')
       #endif
       {
         cmd << "ar t " <<
         #ifdef _WIN32
-          working_directory << "\\" <<  filename;
+          working_directory << "\\" << file_name;
         #else
-          working_directory << "/" <<  filename;
+          working_directory << "/" << file_name;
         #endif
       }
       else
       {
-        cmd << "ar t " << filename;
+        cmd << "ar t " << file_name;
       }
 
       FILE *stream = popen(cmd.str().c_str(), "r");
-      if (stream!=NULL)
+      if(stream!=NULL)
       {
         std::string line;
         char ch;
@@ -255,25 +254,26 @@ bool compilet::add_input_file(const std::string &filename)
       }
       pclose(stream);
       cmd.str("");
+
       #ifdef _WIN32
-      if(filename[0]!='/' && filename[1]!=':')
+      if(file_name[0]!='/' && file_name[1]!=':')
       #else
-      if(filename[0]!='/')
+      if(file_name[0]!='/')
       #endif
       {
         cmd << "ar x " <<
         #ifdef _WIN32
-          working_directory << "\\" <<  filename;
+          working_directory << "\\" << file_name;
         #else
-          working_directory << "/" <<  filename;
+          working_directory << "/" << file_name;
         #endif
       }
       else
       {
-        cmd << "ar x " << filename;
+        cmd << "ar x " << file_name;
       }
 
-      stream = popen(cmd.str().c_str(), "r");
+      stream=popen(cmd.str().c_str(), "r");
       pclose(stream);
 
       if(chdir(working_directory.c_str())!=0)
@@ -283,25 +283,25 @@ bool compilet::add_input_file(const std::string &filename)
     {
       if(cmdline.isset("xml"))
       {
-        if(is_xml_file(filename))
-          object_files.push_back(filename);
+        if(is_xml_file(file_name))
+          object_files.push_back(file_name);
       }
       else
       {
-        if(is_binary_file(filename))
-          object_files.push_back(filename);
+        if(is_binary_file(file_name))
+          object_files.push_back(file_name);
       }
     }
     else if(ext==object_file_extension ||
             ext=="la" || ext=="lo") // Object file recognized
-      object_files.push_back(filename);
+      object_files.push_back(file_name);
     else // assume source file
-      source_files.push_back(filename);
+      source_files.push_back(file_name);
   }
   else
   {
     // don't care about no extensions
-    source_files.push_back(filename);
+    source_files.push_back(file_name);
   }
 
   return false;
@@ -323,6 +323,7 @@ Function: compilet::find_library
 bool compilet::find_library(const std::string &name)
 {
   std::string tmp;
+
   for(std::list<std::string>::const_iterator
       it=library_paths.begin();
       it!=library_paths.end();
@@ -333,7 +334,9 @@ bool compilet::find_library(const std::string &name)
     #else
     tmp = *it + "/lib";
     #endif
+
     std::ifstream in((tmp+name+".a").c_str());
+
     if(in.is_open())
       add_input_file(tmp+name+".a");
     else
@@ -377,17 +380,17 @@ Function: compilet::is_xml_file
 
 \*******************************************************************/
 
-bool compilet::is_xml_file(const std::string &filename)
+bool compilet::is_xml_file(const std::string &file_name)
 {
   std::fstream in;
-  in.open(filename.c_str(), std::ios::in);
+  in.open(file_name.c_str(), std::ios::in);
 
   if(in.is_open())
   {
     char buf[5];
     for (unsigned i=0; i<5; i++)
       buf[i] = in.get();
-    if (buf[0]=='<' && buf[1]=='?' &&
+    if(buf[0]=='<' && buf[1]=='?' &&
         buf[2]=='x' && buf[3]=='m' && buf[4]=='l')
       return true;
   }
@@ -408,17 +411,17 @@ Function: compilet::is_binary_file
 
 \*******************************************************************/
 
-bool compilet::is_binary_file(const std::string &filename)
+bool compilet::is_binary_file(const std::string &file_name)
 {
   std::fstream in;
-  in.open(filename.c_str(), std::ios::in);
+  in.open(file_name.c_str(), std::ios::in);
 
   if(in.is_open())
   {
     char buf[3];
     for (unsigned i=0; i<3; i++)
       buf[i] = in.get();
-    if (buf[0]=='G' && buf[1]=='B' && buf[2]=='F')
+    if(buf[0]=='G' && buf[1]=='B' && buf[2]=='F')
       return true;
   }
   
@@ -438,17 +441,17 @@ Function: compilet::is_elf_file
 
 \*******************************************************************/
 
-bool compilet::is_elf_file(const std::string &filename)
+bool compilet::is_elf_file(const std::string &file_name)
 {
   std::fstream in;
 
-  in.open(filename.c_str(), std::ios::in);
+  in.open(file_name.c_str(), std::ios::in);
   if(in.is_open())
   {
     char buf[4];
     for (unsigned i=0; i<4; i++)
       buf[i] = in.get();
-    if (buf[0]==0x7f && buf[1]=='E' &&
+    if(buf[0]==0x7f && buf[1]=='E' &&
         buf[2]=='L' && buf[3]=='F')
       return true;
   }
@@ -489,10 +492,10 @@ bool compilet::link()
   // parse object files
   while(object_files.size()>0)
   {
-    std::string filename=object_files.front();
+    std::string file_name=object_files.front();
     object_files.pop_front();
 
-    if(parse_object(filename, compiled_functions))
+    if(parse_object(file_name, compiled_functions))
       return true;
   }
 
@@ -510,7 +513,7 @@ bool compilet::link()
 
   // finalize
   contextt::symbolst::iterator it_main=
-    context.symbols.find("c::main");
+    context.symbols.find(config.main.empty()?"c::main":"c::"+config.main);
 
   if((!act_as_ld || it_main!=context.symbols.end()) &&
      !cmdline.isset("static") && !cmdline.isset("shared"))
@@ -520,7 +523,7 @@ bool compilet::link()
     forall_symbols(it, context.symbols)
     {
       const symbolt &symbol = it->second;
-      if (symbol.mode!="" &&
+      if(symbol.mode!="" &&
             find(seen_modes.begin(), seen_modes.end(), symbol.mode)
             == seen_modes.end())
       {
@@ -545,16 +548,16 @@ bool compilet::link()
       std::list<irep_idt>::iterator c =
         find(seen_modes.begin(), seen_modes.end(), "C");
 
-      if (c!=seen_modes.end() && cpp!=seen_modes.end())
+      if(c!=seen_modes.end() && cpp!=seen_modes.end())
         seen_modes.erase(c);
 
       std::list<irep_idt>::iterator it = seen_modes.begin();
-      for (; it!=seen_modes.end(); it++)
+      for(; it!=seen_modes.end(); it++)
       {
         languaget *language=get_language_from_mode(*it);
-        if (language)
+        if(language)
         {
-          if (language->final(context, ui_message_handler))
+          if(language->final(context, ui_message_handler))
             return true;
         }
         else
@@ -602,10 +605,14 @@ bool compilet::compile()
 {
   while(!source_files.empty())
   {
-    std::string filename=source_files.front();
+    std::string file_name=source_files.front();
     source_files.pop_front();
+    
+    // Visual Studio always prints the name of the file it's doing
+    if(echo_file_name)
+      status(file_name);
 
-    bool r=parse_source(filename); // don't break the program!
+    bool r=parse_source(file_name); // don't break the program!
 
     if(!r && !doLink && !only_preprocess)
     {
@@ -631,9 +638,9 @@ bool compilet::compile()
       if(output_file_object=="")
       {
         if(cmdline.isset('S')) // compile, but don't assemble
-          cfn=get_base_name(filename) + ".s";
+          cfn=get_base_name(file_name) + ".s";
         else
-          cfn=get_base_name(filename) + "." + object_file_extension;
+          cfn=get_base_name(file_name) + "." + object_file_extension;
       }
       else
       {
@@ -657,7 +664,7 @@ bool compilet::compile()
 
 Function: compilet::parse
 
-  Inputs: filename
+  Inputs: file_name
 
  Outputs: true on error, false otherwise
 
@@ -665,23 +672,23 @@ Function: compilet::parse
 
 \*******************************************************************/
 
-bool compilet::parse(const std::string &filename)
+bool compilet::parse(const std::string &file_name)
 {
-  if(filename=="-") return parse_stdin();
+  if(file_name=="-") return parse_stdin();
 
-  std::ifstream infile(filename.c_str());
+  std::ifstream infile(file_name.c_str());
 
   if(!infile)
   {
-    error("failed to open input file", filename);
+    error("failed to open input file", file_name);
     return true;
   }
 
-  languaget *languagep=get_language_from_filename(filename);
+  languaget *languagep=get_language_from_filename(file_name);
 
   if(languagep==NULL)
   {
-    error("failed to figure out type of file", filename);
+    error("failed to figure out type of file", file_name);
     return true;
   }
 
@@ -690,13 +697,13 @@ bool compilet::parse(const std::string &filename)
 
   std::pair<language_filest::filemapt::iterator, bool>
   res=language_files.filemap.insert(
-    std::pair<std::string, language_filet>(filename, language_file));
+    std::pair<std::string, language_filet>(file_name, language_file));
 
   language_filet &lf=res.first->second;
-  lf.filename=filename;
+  lf.filename=file_name;
   lf.language=languagep;
 
-  print(8, "Parsing: "+filename);
+  print(8, "Parsing: "+file_name);
 
   if(only_preprocess)
   {
@@ -716,11 +723,11 @@ bool compilet::parse(const std::string &filename)
       }
     }
 
-    language.preprocess(infile, filename, *os, get_message_handler());
+    language.preprocess(infile, file_name, *os, get_message_handler());
   }
   else
   {
-    if(language.parse(infile, filename, get_message_handler()))
+    if(language.parse(infile, file_name, get_message_handler()))
     {
       if(get_ui()==ui_message_handlert::PLAIN)
         error("PARSING ERROR");
@@ -736,7 +743,7 @@ bool compilet::parse(const std::string &filename)
 
 Function: compilet::parse_stdin
 
-  Inputs: filename
+  Inputs: file_name
 
  Outputs: true on error, false otherwise
 
@@ -787,7 +794,7 @@ bool compilet::parse_stdin()
 
 Function: compilet::write_object_file
 
-  Inputs: filename, functions table
+  Inputs: file_name, functions table
 
  Outputs: true on error, false otherwise
 
@@ -797,21 +804,21 @@ Function: compilet::write_object_file
 \*******************************************************************/
 
 bool compilet::write_object_file(
-  const std::string &filename,
+  const std::string &file_name,
   const contextt &lcontext,
   goto_functionst &functions)
 {
   if(cmdline.isset("xml"))
-    return write_xml_object_file(filename, lcontext, functions);
+    return write_xml_object_file(file_name, lcontext, functions);
   else
-    return write_bin_object_file(filename, lcontext, functions);
+    return write_bin_object_file(file_name, lcontext, functions);
 }
 
 /*******************************************************************\
 
 Function: compilet::write_bin_object_file
 
-  Inputs: filename, functions table
+  Inputs: file_name, functions table
 
  Outputs: true on error, false otherwise
 
@@ -821,21 +828,21 @@ Function: compilet::write_bin_object_file
 \*******************************************************************/
 
 bool compilet::write_bin_object_file(
-  const std::string &filename,
+  const std::string &file_name,
   const contextt &lcontext,
   goto_functionst &functions)
 {
-  print(8, "Writing binary format object `" + filename + "'");
+  print(8, "Writing binary format object `" + file_name + "'");
 
   // symbols
   print(8, "Symbols in table: "+
            i2string((unsigned long)lcontext.symbols.size()));
 
-  std::ofstream outfile(filename.c_str(), std::ios::binary);
+  std::ofstream outfile(file_name.c_str(), std::ios::binary);
 
   if(!outfile.is_open())
   {
-    error("Error opening file `"+filename+"'");
+    error("Error opening file `"+file_name+"'");
     return true;
   }
 
@@ -852,15 +859,15 @@ bool compilet::write_bin_object_file(
   if(cmdline.isset("dot"))
   {
     std::ofstream dgf;
-    write_dot_header(filename, dgf);
+    write_dot_header(file_name, dgf);
 
-    for (goto_functionst::function_mapt::iterator it=
-           functions.function_map.begin();
-         it!=functions.function_map.end();
-         it++)
+    for(goto_functionst::function_mapt::iterator
+        it=functions.function_map.begin();
+        it!=functions.function_map.end();
+        it++)
     {
-      if (it->second.body_available)
-          write_dot_subgraph(dgf, id2string(it->first), it->second.body);
+      if(it->second.body_available)
+        write_dot_subgraph(dgf, id2string(it->first), it->second.body);
     }
 
     do_dot_function_calls(dgf);
@@ -875,7 +882,7 @@ bool compilet::write_bin_object_file(
 
 Function: compilet::write_xml_object_file
 
-  Inputs: filename, functions table
+  Inputs: file_name, functions table
 
  Outputs: true on error, false otherwise
 
@@ -885,16 +892,16 @@ Function: compilet::write_xml_object_file
 \*******************************************************************/
 
 bool compilet::write_xml_object_file(
-  const std::string &filename,
+  const std::string &file_name,
   const contextt &lcontext,
   goto_functionst &functions)
 {
-  print(8, "Writing xml format object " + filename);
+  print(8, "Writing xml format object " + file_name);
 
-  std::ofstream f(filename.c_str());
+  std::ofstream f(file_name.c_str());
   if(!f.is_open())
   {
-    error("Error opening file " + filename);
+    error("Error opening file " + file_name);
     return true;
   }
 
@@ -916,30 +923,32 @@ bool compilet::write_xml_object_file(
   }
 
   xmlt funs("functions");
-  if (verbosity>=9)
+  if(verbosity>=9)
   {
     std::cout << "Functions: " << functions.function_map.size() << "; ";
     std::cout << function_body_count(functions) << " have a body." << std::endl;
   }
 
   std::ofstream dgf;
-  if (cmdline.isset("dot"))
-    write_dot_header(filename, dgf);
-    for ( goto_functionst::function_mapt::iterator it=functions.function_map.begin();
-          it != functions.function_map.end();
-          it++)
-    {
-      if (it->second.body_available)
-      {
-        xmlt &fun = funs.new_element("function");
-        fun.set_attribute("name", id2string(it->first));
-        gfconverter.convert(it->second, fun);
-        if (dgf.is_open())
-          write_dot_subgraph(dgf, id2string(it->first), it->second.body);
-      }
-    }
 
-  if (dgf.is_open())
+  if(cmdline.isset("dot"))
+    write_dot_header(file_name, dgf);
+
+  for(goto_functionst::function_mapt::iterator it=functions.function_map.begin();
+      it != functions.function_map.end();
+      it++)
+  {
+    if(it->second.body_available)
+    {
+      xmlt &fun = funs.new_element("function");
+      fun.set_attribute("name", id2string(it->first));
+      gfconverter.convert(it->second, fun);
+      if(dgf.is_open())
+        write_dot_subgraph(dgf, id2string(it->first), it->second.body);
+    }
+  }
+
+  if(dgf.is_open())
   {
     do_dot_function_calls(dgf);
     dgf << "}" << std::endl;
@@ -983,7 +992,7 @@ void compilet::write_dot_subgraph(
   const goto_programt::instructionst& instructions =
     goto_program.instructions;  
   
-  if (instructions.size()==0)
+  if(instructions.size()==0)
   {
     out << "Node_" << subgraphscount << "_0 " <<
       "[shape=Mrecord,fontsize=22,label=\"?\"];" << std::endl;
@@ -994,7 +1003,7 @@ void compilet::write_dot_subgraph(
     goto_programt::const_targetst worklist;
     worklist.push_back(instructions.begin());
     
-    while (!worklist.empty())
+    while(!worklist.empty())
     {
       goto_programt::const_targett it=worklist.front();
       worklist.pop_front();
@@ -1005,7 +1014,7 @@ void compilet::write_dot_subgraph(
       std::stringstream tmp("");
       if(it->is_goto())
       {
-        if (it->guard.is_true())
+        if(it->guard.is_true())
           tmp.str("Goto");
         else
         {
@@ -1015,33 +1024,33 @@ void compilet::write_dot_subgraph(
           tmp << escape(t) << "?";
         }
       }
-      else if (it->is_assume())
+      else if(it->is_assume())
       {
         std::string t = from_expr(ns, "", it->guard);
         while (t[ t.size()-1 ]=='\n')
           t = t.substr(0,t.size()-1);
         tmp << "Assume\\n(" << escape(t) << ")";
       }
-      else if (it->is_assert())
+      else if(it->is_assert())
       {
         std::string t = from_expr(ns, "", it->guard);
         while (t[ t.size()-1 ]=='\n')
           t = t.substr(0,t.size()-1);
         tmp << "Assert\\n(" << escape(t) << ")";
       }
-      else if (it->is_skip())
+      else if(it->is_skip())
         tmp.str("Skip");
-      else if (it->is_end_function())            
+      else if(it->is_end_function())            
         tmp.str("End of Function");      
-      else if (it->is_location())
+      else if(it->is_location())
         tmp.str("Location");
-      else if (it->is_dead())
+      else if(it->is_dead())
         tmp.str("Dead");
-      else if (it->is_atomic_begin())
+      else if(it->is_atomic_begin())
         tmp.str("Atomic Begin");
-      else if (it->is_atomic_end())
+      else if(it->is_atomic_end())
         tmp.str("Atomic End");
-      else if (it->is_function_call())
+      else if(it->is_function_call())
       {
         std::string t = from_expr(ns, "", it->code);
         while (t[ t.size()-1 ]=='\n')
@@ -1055,7 +1064,7 @@ void compilet::write_dot_subgraph(
         fc.operands().push_back(it->code.op1());  
         function_calls.push_back(fc);
       }
-      else if (it->is_assign() ||
+      else if(it->is_assign() ||
                it->is_return() ||
                it->is_other())      
       {
@@ -1064,20 +1073,20 @@ void compilet::write_dot_subgraph(
           t = t.substr(0,t.size()-1);
         tmp.str(escape(t));
       }
-      else if (it->is_start_thread())
+      else if(it->is_start_thread())
         tmp.str("Start of Thread");
-      else if (it->is_end_thread())
+      else if(it->is_end_thread())
         tmp.str("End of Thread");
-      else if (it->is_throw())
+      else if(it->is_throw())
         tmp.str("THROW");
-      else if (it->is_catch())
+      else if(it->is_catch())
         tmp.str("CATCH");
       else
         tmp.str("UNKNOWN");
 
       out << "Node_" << subgraphscount << "_" << it->location_number;
       out << " [shape="; 
-      if (it->is_goto() && !it->guard.is_true() && !it->guard.is_false())
+      if(it->is_goto() && !it->guard.is_true() && !it->guard.is_false())
         out << "diamond";
       else
         out <<"Mrecord";
@@ -1091,7 +1100,7 @@ void compilet::write_dot_subgraph(
 
       std::string tlabel="true";
       std::string flabel="false";
-      if (fres.size()==0 || tres.size()==0)
+      if(fres.size()==0 || tres.size()==0)
       {
         tlabel="";
         flabel="";
@@ -1139,10 +1148,10 @@ void compilet::do_dot_function_calls(std::ostream &out)
   {
     std::list<exprt>::const_iterator cit=clusters.begin();
     for (;cit!=clusters.end();cit++)
-      if (cit->get("name")==it->op1().get(ID_identifier))
+      if(cit->get("name")==it->op1().get(ID_identifier))
         break;
 
-    if (cit!=clusters.end())
+    if(cit!=clusters.end())
     {
       out << it->op0().id() <<
         " -> " "Node_" << cit->get("nr") << "_0" <<
@@ -1190,18 +1199,18 @@ void compilet::find_next(
   std::set<goto_programt::const_targett> &tres,
   std::set<goto_programt::const_targett> &fres)
 {
-  if (it->is_goto() && !it->guard.is_false())
+  if(it->is_goto() && !it->guard.is_false())
   {
     goto_programt::targetst::const_iterator gtit = it->targets.begin();
     for (; gtit!=it->targets.end(); gtit++)
       tres.insert((*gtit));
   }
 
-  if (it->is_goto() && it->guard.is_true())
+  if(it->is_goto() && it->guard.is_true())
     return;
   
   goto_programt::const_targett next = it; next++;
-  if (next!=instructions.end())
+  if(next!=instructions.end())
     fres.insert(next);
 }
 
@@ -1227,10 +1236,10 @@ void compilet::write_edge(
   out << "Node_" << subgraphscount << "_" << from.location_number;
   out << " -> ";
   out << "Node_" << subgraphscount << "_" << to.location_number << " ";
-  if (label!="")
+  if(label!="")
     {
       out << "[fontsize=20,label=\"" << label << "\"";
-      if (from.is_backwards_goto() &&
+      if(from.is_backwards_goto() &&
           from.location_number > to.location_number)
         out << ",color=red";
       out << "]";
@@ -1280,7 +1289,7 @@ std::string &compilet::escape(std::string &str)
 
 Function: compilet::parse_source
 
-  Inputs: filename
+  Inputs: file_name
 
  Outputs: true on error, false otherwise
 
@@ -1288,16 +1297,16 @@ Function: compilet::parse_source
 
 \*******************************************************************/
 
-bool compilet::parse_source(const std::string &filename)
+bool compilet::parse_source(const std::string &file_name)
 {
-  if(parse(filename))
+  if(parse(file_name))
     return true;
     
   if(typecheck()) // we just want to typecheck this one file here
     return true;
     
   // so we remove it from the list afterwards
-  language_files.filemap.erase(filename);
+  language_files.filemap.erase(file_name);
   return false;
 }
 
@@ -1305,7 +1314,7 @@ bool compilet::parse_source(const std::string &filename)
 
 Function: compilet::parse_object
 
-  Inputs: a filename
+  Inputs: a file_name
 
  Outputs: true on error, false otherwise
 
@@ -1314,22 +1323,22 @@ Function: compilet::parse_object
 \*******************************************************************/
 
 bool compilet::parse_object(
-  const std::string &filename,
+  const std::string &file_name,
   goto_functionst &functions)
 {
   std::ifstream infile;
   if(cmdline.isset("xml"))
-    infile.open(filename.c_str());
+    infile.open(file_name.c_str());
   else
-    infile.open(filename.c_str(), std::ios::binary);
+    infile.open(file_name.c_str(), std::ios::binary);
 
   if(!infile)
   {
-    error("failed to open input file", filename);
+    error("failed to open input file", file_name);
     return true;
   }
 
-  print(8, "Parsing: " + filename);
+  print(8, "Parsing: " + file_name);
 
   // we parse to a temporary context
   contextt temp_context;
@@ -1338,13 +1347,13 @@ bool compilet::parse_object(
 
   if(cmdline.isset("xml"))
   {
-    if(read_goto_object(infile, filename, temp_context,
+    if(read_goto_object(infile, file_name, temp_context,
                          temp_functions, *message_handler))
       return true;
   }
   else
   {
-    if(read_bin_goto_object(infile, filename, temp_context,
+    if(read_bin_goto_object(infile, file_name, temp_context,
                              temp_functions, *message_handler))
       return true;
   }
@@ -1354,7 +1363,7 @@ bool compilet::parse_object(
       it!=temp_context.symbols.end();
       it++)
   {
-    if (  it->second.mode!="" &&
+    if(  it->second.mode!="" &&
           find(  seen_modes.begin(),
                  seen_modes.end(),
                  it->second.mode)
@@ -1435,21 +1444,12 @@ compilet::compilet(cmdlinet &_cmdline):
   ns(context),
   cmdline(_cmdline)
 {
-  doLink = false;
-  act_as_ld = false;
-  only_preprocess = false;
-  subgraphscount = 0;
-
-  unsigned bsize=50;
-  char *buf = (char*) malloc(sizeof(char)*bsize);
-  errno=0;
-  while(buf && getcwd(buf, bsize-1)==NULL && errno==ERANGE)
-  {
-    bsize*=2;
-    buf = (char*) realloc(buf, sizeof(char)*bsize);
-  }
-  working_directory = buf;
-  if(buf) free(buf);
+  doLink=false;
+  act_as_ld=false;
+  only_preprocess=false;
+  echo_file_name=false;
+  subgraphscount=0;
+  working_directory=get_current_working_directory();
 }
 
 /*******************************************************************\
@@ -1468,41 +1468,10 @@ compilet::~compilet()
 {
   // clean up temp dirs
 
-  for (std::list<std::string>::const_iterator it = tmp_dirs.begin();
-       it!=tmp_dirs.end();
-       it++)
-  {
-    #ifdef _WIN32
-    
-    std::string pattern=*it+"\\*";
-    
-    struct _finddata_t info;
-    
-    intptr_t handle=_findfirst(pattern.c_str(), &info);
-    
-    if(handle!=-1)
-    {
-      unlink(info.name);
-      
-      while(_findnext(handle, &info)!=-1)
-        unlink(info.name);
-    }
-    #else
-    DIR *dir=opendir(it->c_str());
-
-    if(dir!=NULL)
-    {
-      struct dirent *ent;
-
-      while((ent=readdir(dir))!=NULL)
-        remove((*it + "/" + ent->d_name).c_str());
-
-      closedir(dir);
-    }
-    #endif
-
-    rmdir(it->c_str());
-  }
+  for(std::list<std::string>::const_iterator it = tmp_dirs.begin();
+      it!=tmp_dirs.end();
+      it++)
+    delete_directory(*it);
 }
 
 /*******************************************************************\
@@ -1518,14 +1487,14 @@ Function: compilet::write_dot_file
 \*******************************************************************/
 
 bool compilet::write_dot_header(
-  const std::string &filename,
+  const std::string &file_name,
   std::ofstream &dgf)
 {
-  std::string dgfilename = filename + ".dot";
-  if (verbosity>=9)
+  std::string dgfilename = file_name + ".dot";
+  if(verbosity>=9)
     status("Writing dot graph to " + dgfilename);
   dgf.open(dgfilename.c_str());
-  if (!dgf.is_open())
+  if(!dgf.is_open())
   {
     error("Error opening file: " + dgfilename);
     return true;
@@ -1554,7 +1523,7 @@ unsigned compilet::function_body_count(const goto_functionst &functions)
           functions.function_map.begin();
         it != functions.function_map.end();
         it++)
-    if (it->second.body_available)
+    if(it->second.body_available)
       fbs++;
 
   return fbs;
@@ -1706,15 +1675,37 @@ void compilet::convert_symbols(goto_functionst &dest)
   goto_convert_functionst converter(context, options, dest,
                                     ui_message_handler);
 
-  Forall_symbols(it, context.symbols)
+  // the compilation may add symbols!
+
+  contextt::symbolst::size_type before=0;
+  
+  while(before!=context.symbols.size())
   {
-    if(it->second.type.id()==ID_code &&
-       it->second.value.id()!="compiled" &&
-       it->second.value.is_not_nil())
+    before=context.symbols.size();
+
+    typedef std::set<irep_idt> symbols_sett;
+    symbols_sett symbols;
+  
+    Forall_symbols(it, context.symbols)
+      symbols.insert(it->first);
+
+    // the symbol table itertors aren't stable
+    for(symbols_sett::const_iterator
+        it=symbols.begin();
+        it!=symbols.end();
+        ++it)
     {
-      print(9, "Compiling "+id2string(it->first));
-      converter.convert_function(it->first);
-      it->second.value=exprt("compiled");
+      contextt::symbolst::iterator s_it=context.symbols.find(*it);
+      assert(s_it!=context.symbols.end());
+
+      if(s_it->second.type.id()==ID_code &&
+          s_it->second.value.id()!="compiled" &&
+          s_it->second.value.is_not_nil())
+      {
+        print(9, "Compiling "+id2string(s_it->first));
+        converter.convert_function(s_it->first);
+        s_it->second.value=exprt("compiled");
+      }
     }
   }
 }
