@@ -19,6 +19,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <simplify_expr.h>
 #include <std_code.h>
 #include <std_expr.h>
+#include <symbol.h>
 
 #include <ansi-c/c_types.h>
 
@@ -27,86 +28,6 @@ Author: Daniel Kroening, kroening@kroening.com
 
 /*******************************************************************\
 
-Function: get_alloc_type_rec
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-static void get_alloc_type_rec(
-  const exprt &src,
-  typet &type,
-  exprt &size)
-{
-  const irept &sizeof_type=src.find("#c_sizeof_type");
-
-  if(!sizeof_type.is_nil())
-  {
-    type=static_cast<const typet &>(sizeof_type);
-  }
-  else if(src.id()==ID_mult)
-  {
-    forall_operands(it, src)
-      get_alloc_type_rec(*it, type, size);
-  }
-  else
-  {
-    size.copy_to_operands(src);
-  }
-}
-
-/*******************************************************************\
-
-Function: get_alloc_type
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-#if 0
-static void get_alloc_type(
-  const exprt &src,
-  typet &type,
-  exprt &size)
-{
-  type.make_nil();
-  size.make_nil();
-
-  get_alloc_type_rec(src, type, size);
-
-  if(type.is_nil())
-    type=char_type();
-
-  if(size.has_operands())
-  {
-    if(size.operands().size()==1)
-    {
-      exprt tmp;
-      tmp.swap(size.op0());
-      size.swap(tmp);
-    }
-    else
-    {
-      size.id(ID_mult);
-      size.type()=size.op0().type();
-    }
-    
-    if(size.type()!=size_type())
-      size.make_typecast(size_type());
-  }
-}
-#endif
-
-/*******************************************************************\
-
 Function: goto_convertt::do_prob_uniform
 
   Inputs:
@@ -852,16 +773,28 @@ void goto_convertt::do_function_call_symbol(
     throw "error: function `"+id2string(identifier)+"' type mismatch: expected code";
   }
   
-  bool is_assume=identifier==CPROVER_PREFIX "assume" ||
-                 identifier=="specc::__CPROVER_assume";
+  if(identifier==CPROVER_PREFIX "parameter_predicates" || identifier==CPROVER_PREFIX "return_predicates")
+  {
+        if(arguments.size() != 0)
+        {
+            err_location(function);
+            throw "`"+id2string(identifier)+"' expected to have no arguments";
+        }
+        goto_programt::targett t = dest.add_instruction(OTHER);
+        t->location = function.location();
+        t->location.set("user-provided", true);
+        if(identifier==CPROVER_PREFIX "parameter_predicates")
+        {
+                t->code = codet(ID_user_specified_parameter_predicates);
+                t->code.set_statement(ID_user_specified_parameter_predicates);
+        } else {
+                t->code = codet(ID_user_specified_return_predicates);
+                t->code.set_statement(ID_user_specified_return_predicates);
+        }
+        return;
+  }
 
-  bool is_assert=identifier=="c::assert" ||
-                 identifier=="specc::assert";
-
-  bool is_predicate=identifier==CPROVER_PREFIX "predicate" ||
-                 identifier=="specc::__CPROVER_predicate";
-
-  if(is_assume || is_assert || is_predicate)
+  if(identifier==CPROVER_PREFIX "predicate")
   {
     if(arguments.size()!=1)
     {
@@ -869,32 +802,61 @@ void goto_convertt::do_function_call_symbol(
       throw "`"+id2string(identifier)+"' expected to have one argument";
     }
 
-    if(is_predicate) {
-    	goto_programt::targett t = dest.add_instruction(OTHER);
-    	t->guard = arguments.front();
-    	t->location = function.location();
-    	t->location.set("user-provided", true);
-    	t->code = ID_user_specified_predicate;
-		t->code.set_statement(ID_user_specified_predicate);
-	  return;
-
-    }
-
-    if(is_assume && !options.get_bool_option("assumptions"))
-      return;
-
-    if(is_assert && !options.get_bool_option("assertions"))
-      return;
-
-    goto_programt::targett t=dest.add_instruction(
-      is_assume?ASSUME:ASSERT);
+    goto_programt::targett t=dest.add_instruction(OTHER);
     t->guard=arguments.front();
     t->location=function.location();
     t->location.set("user-provided", true);
+    t->code=codet(ID_user_specified_predicate);
+  }
+  else if(identifier==CPROVER_PREFIX "assume" ||
+          identifier=="c::__VERIFIER_assume")
+  {
+    if(arguments.size()!=1)
+    {
+      err_location(function);
+      throw "`"+id2string(identifier)+"' expected to have one argument";
+    }
 
-    if(is_assert)
-      t->location.set(ID_property, ID_assertion);
+    if(!options.get_bool_option("assumptions"))
+      return;
+
+    goto_programt::targett t=dest.add_instruction(ASSUME);
+    t->guard=arguments.front();
+    t->location=function.location();
+    t->location.set("user-provided", true);
     
+    // let's double-check the type of the argument
+    if(t->guard.type().id()!=ID_bool)
+      t->guard.make_typecast(bool_typet());
+
+    if(lhs.is_not_nil())
+    {
+      err_location(function);
+      throw id2string(identifier)+" expected not to have LHS";
+    }
+  }
+  else if(identifier=="c::assert")
+  {
+    if(arguments.size()!=1)
+    {
+      err_location(function);
+      throw "`"+id2string(identifier)+"' expected to have one argument";
+    }
+
+    if(!options.get_bool_option("assertions"))
+      return;
+
+    goto_programt::targett t=dest.add_instruction(ASSERT);
+    t->guard=arguments.front();
+    t->location=function.location();
+    t->location.set("user-provided", true);
+    t->location.set_priority(10); // high
+    t->location.set_property(ID_assertion);
+    
+    // let's double-check the type of the argument
+    if(t->guard.type().id()!=ID_bool)
+      t->guard.make_typecast(bool_typet());
+
     if(lhs.is_not_nil())
     {
       err_location(function);
@@ -919,9 +881,14 @@ void goto_convertt::do_function_call_symbol(
     t->guard=arguments[0];
     t->location=function.location();
     t->location.set("user-provided", true);
-    t->location.set(ID_property, ID_assertion);
-    t->location.set(ID_comment, description);
+    t->location.set_property(ID_assertion);
+    t->location.set_priority(10); // high
+    t->location.set_comment(description);
     
+    // let's double-check the type of the argument
+    if(t->guard.type().id()!=ID_bool)
+      t->guard.make_typecast(bool_typet());
+
     if(lhs.is_not_nil())
     {
       err_location(function);
@@ -961,7 +928,7 @@ void goto_convertt::do_function_call_symbol(
     do_prob_uniform(lhs, function, arguments, dest);
   }
   else if(has_prefix(id2string(identifier), "c::nondet_") ||
-          has_prefix(id2string(identifier), "cpp::nondet_"))
+          has_prefix(id2string(identifier), "c::__VERIFIER_nondet_"))
   {
     // make it a side effect if there is an LHS
     if(lhs.is_nil()) return;
@@ -1029,8 +996,9 @@ void goto_convertt::do_function_call_symbol(
     t->guard=false_exprt();
     t->location=function.location();
     t->location.set("user-provided", true);
-    t->location.set(ID_property, ID_assertion);
-    t->location.set(ID_comment, description);
+    t->location.set_property(ID_assertion);
+    t->location.set_priority(10); // high
+    t->location.set_comment(description);
     // we ignore any LHS
   }
   else if(identifier=="c::__assert_rtn")
@@ -1055,8 +1023,9 @@ void goto_convertt::do_function_call_symbol(
     t->guard=false_exprt();
     t->location=function.location();
     t->location.set("user-provided", true);
-    t->location.set(ID_property, ID_assertion);
-    t->location.set(ID_comment, description);
+    t->location.set_property(ID_assertion);
+    t->location.set_priority(10); // high
+    t->location.set_comment(description);
     // we ignore any LHS
   }
   else if(identifier=="c::_wassert")
@@ -1080,8 +1049,9 @@ void goto_convertt::do_function_call_symbol(
     t->guard=false_exprt();
     t->location=function.location();
     t->location.set("user-provided", true);
-    t->location.set(ID_property, ID_assertion);
-    t->location.set(ID_comment, description);
+    t->location.set_property(ID_assertion);
+    t->location.set_priority(10); // high
+    t->location.set_comment(description);
     // we ignore any LHS
   }
   else if(identifier=="c::__builtin_prefetch")
