@@ -9,11 +9,14 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <location.h>
 #include <i2string.h>
 #include <expr_util.h>
+#include <std_types.h>
 #include <guard.h>
 #include <simplify_expr.h>
 #include <array_name.h>
 #include <arith_tools.h>
 #include <base_type.h>
+#include <ieee_float.h>
+#include <std_expr.h>
 
 #include "goto_check.h"
 
@@ -110,42 +113,168 @@ void goto_checkt::overflow_check(
   const exprt &expr,
   const guardt &guard)
 {
-  if(!options.get_bool_option("overflow-check"))
+  bool signed_overflow_check=options.get_bool_option("signed-overflow-check");
+  bool unsigned_overflow_check=options.get_bool_option("unsigned-overflow-check");
+
+  if(!signed_overflow_check && !unsigned_overflow_check)
     return;
 
-  // first, check type
-  if(expr.type().id()!=ID_signedbv)
+  // First, check type.
+  const typet &type=ns.follow(expr.type());
+  
+  if(type.id()==ID_signedbv && !signed_overflow_check)
     return;
 
+  if(type.id()==ID_unsignedbv && !unsigned_overflow_check)
+    return;
+    
   // add overflow subgoal
 
+  if(expr.id()==ID_typecast)
+  {
+    // conversion to signed int may overflow
+  
+    if(expr.operands().size()!=1)
+      throw "typecast takes one operand";
+
+    const typet &old_type=ns.follow(expr.op0().type());
+    
+    if(type.id()==ID_signedbv)
+    {
+      if(old_type.id()==ID_signedbv)
+      {
+        unsigned new_width=expr.type().get_int(ID_width);
+        unsigned old_width=old_type.get_int(ID_width);
+        if(new_width>=old_width) return; // always ok
+
+        binary_relation_exprt no_overflow_upper(ID_le);
+        no_overflow_upper.lhs()=expr.op0();
+        no_overflow_upper.rhs()=from_integer(power(2, new_width-1)-1, old_type);
+
+        binary_relation_exprt no_overflow_lower(ID_ge);
+        no_overflow_lower.lhs()=expr.op0();
+        no_overflow_lower.rhs()=from_integer(-power(2, new_width-1), old_type);
+
+        add_guarded_claim(
+          and_exprt(no_overflow_lower, no_overflow_upper),
+          "arithmetic overflow on signed type conversion",
+          "overflow",
+          expr.find_location(),
+          guard);
+      }
+      else if(old_type.id()==ID_unsignedbv)
+      {
+        unsigned new_width=expr.type().get_int(ID_width);
+        unsigned old_width=old_type.get_int(ID_width);
+        if(new_width>=old_width+1) return; // always ok
+
+        binary_relation_exprt no_overflow_upper(ID_le);
+        no_overflow_upper.lhs()=expr.op0();
+        no_overflow_upper.rhs()=from_integer(power(2, new_width-1)-1, old_type);
+
+        add_guarded_claim(
+          no_overflow_upper,
+          "arithmetic overflow on unsigned to signed type conversion",
+          "overflow",
+          expr.find_location(),
+          guard);
+      }
+      else if(old_type.id()==ID_floatbv)
+      {
+        unsigned new_width=expr.type().get_int(ID_width);
+
+        // Note that the fractional part is truncated!
+        ieee_floatt upper(to_floatbv_type(old_type));
+        upper.from_integer(power(2, new_width-1));
+        binary_relation_exprt no_overflow_upper(ID_lt);
+        no_overflow_upper.lhs()=expr.op0();
+        no_overflow_upper.rhs()=upper.to_expr();
+
+        ieee_floatt lower(to_floatbv_type(old_type));
+        lower.from_integer(-power(2, new_width-1)-1);
+        binary_relation_exprt no_overflow_lower(ID_gt);
+        no_overflow_lower.lhs()=expr.op0();
+        no_overflow_lower.rhs()=lower.to_expr();
+
+        add_guarded_claim(
+          and_exprt(no_overflow_lower, no_overflow_upper),
+          "arithmetic overflow on float to signed integer type conversion",
+          "overflow",
+          expr.find_location(),
+          guard);
+      }
+    }
+    else if(type.id()==ID_unsignedbv)
+    {
+      // todo
+    }
+
+    return;
+  }
+  else if(expr.id()==ID_div)
+  {
+    assert(expr.operands().size()==2);
+
+    // undefined for signed division INT_MIN/-1
+    if(type.id()==ID_signedbv)
+    {
+      equal_exprt int_min_eq(
+        expr.op0(), to_signedbv_type(type).smallest_expr());
+
+      equal_exprt minus_one_eq(
+        expr.op1(), from_integer(-1, type));
+
+      add_guarded_claim(
+        not_exprt(and_exprt(int_min_eq, minus_one_eq)),
+        "arithmetic overflow on signed division",
+        "overflow",
+        expr.find_location(),
+        guard);
+    }
+    return;
+  }
+
   exprt overflow("overflow-"+expr.id_string(), bool_typet());
   overflow.operands()=expr.operands();
 
-  if(expr.id()==ID_typecast)
+  if(expr.operands().size()>=3)
   {
-    if(expr.operands().size()!=1)
-      throw "typecast takes one operand";
-
-    const typet &old_type=expr.op0().type();
-
-    unsigned new_width=atoi(expr.type().get(ID_width).c_str());
-    unsigned old_width=atoi(old_type.get(ID_width).c_str());
-
-    if(old_type.id()==ID_unsignedbv) new_width--;
-    if(new_width>=old_width) return;
-
-    overflow.id(overflow.id_string()+"-"+i2string(new_width));
+    // The overflow checks are binary!
+    // We break these up.
+  
+    for(unsigned i=1; i<expr.operands().size(); i++)
+    {
+      exprt tmp;
+      
+      if(i==1)
+        tmp=expr.op0();
+      else
+      {
+        tmp=expr;
+        tmp.operands().resize(i);
+      }
+      
+      overflow.operands().resize(2);
+      overflow.op0()=tmp;
+      overflow.op1()=expr.operands()[i];
+    
+      add_guarded_claim(
+        not_exprt(overflow),
+        "arithmetic overflow on "+expr.id_string(),
+        "overflow",
+        expr.find_location(),
+        guard);
+    }
+  }
+  else
+  {
+    add_guarded_claim(
+      not_exprt(overflow),
+      "arithmetic overflow on "+expr.id_string(),
+      "overflow",
+      expr.find_location(),
+      guard);
   }
-
-  overflow.make_not();
-
-  add_guarded_claim(
-    overflow,
-    "arithmetic overflow on "+expr.id_string(),
-    "overflow",
-    expr.find_location(),
-    guard);
 }
 
 /*******************************************************************\
@@ -453,12 +582,16 @@ void goto_checkt::add_guarded_claim(
   
   if(assertions.insert(new_expr).second)
   {
-    goto_programt::targett t=new_code.add_instruction(ASSERT);
+    goto_program_instruction_typet type=
+      options.get_bool_option("assert-to-assume")?ASSUME:ASSERT;
+
+    goto_programt::targett t=new_code.add_instruction(type);
 
     t->guard.swap(new_expr);
     t->location=location;
     t->location.set_comment(comment);
     t->location.set_property(property);
+    t->location.set_priority(2); // relatively low
   }
 }
 
@@ -577,13 +710,17 @@ void goto_checkt::check_rec(
   {
     div_by_zero_check(expr, guard);
     nan_check(expr, guard);
+    
+    if(expr.type().id()==ID_signedbv)
+      overflow_check(expr, guard);
   }
   else if(expr.id()==ID_plus || expr.id()==ID_minus ||
           expr.id()==ID_mult ||
           expr.id()==ID_unary_minus ||
           expr.id()==ID_typecast)
   {
-    if(expr.type().id()==ID_signedbv)
+    if(expr.type().id()==ID_signedbv ||
+       expr.type().id()==ID_unsignedbv)
       overflow_check(expr, guard);
     else if(expr.type().id()==ID_floatbv)
       nan_check(expr, guard);
