@@ -13,6 +13,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <expr_util.h>
 #include <prefix.h>
 #include <std_expr.h>
+#include <context.h>
 
 #include <ansi-c/c_types.h>
 
@@ -142,7 +143,7 @@ void goto_convertt::finish_computed_gotos(goto_programt &goto_program)
       exprt label_expr(ID_label, empty_typet());
       label_expr.set(ID_identifier, l_it->first);
 
-      equality_exprt guard;
+      equal_exprt guard;
       
       guard.lhs()=pointer;
       guard.rhs()=address_of_exprt(label_expr);
@@ -296,7 +297,14 @@ void goto_convertt::convert_label(
 
   if(!case_op.empty())
   {
-    exprt::operandst &case_op_dest=targets.cases[target];
+    cases_mapt::iterator cases_entry=targets.cases_map.find(target);
+    if(cases_entry==targets.cases_map.end())
+    {
+      targets.cases.push_back(std::make_pair(target, caset()));
+      cases_entry=targets.cases_map.insert(std::make_pair(
+            target, --targets.cases.end())).first;
+    }
+    exprt::operandst &case_op_dest=cases_entry->second->second;
     
     case_op_dest.reserve(case_op_dest.size()+case_op.size());
     
@@ -312,6 +320,25 @@ void goto_convertt::convert_label(
 
 /*******************************************************************\
 
+Function: goto_convert::convert_gcc_local_label
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void goto_convertt::convert_gcc_local_label(
+  const codet &code,
+  goto_programt &dest)
+{
+  // ignore for now
+}
+
+/*******************************************************************\
+
 Function: goto_convertt::convert
 
   Inputs:
@@ -348,6 +375,8 @@ void goto_convertt::convert(
     convert_function_call(to_code_function_call(code), dest);
   else if(statement==ID_label)
     convert_label(to_code_label(code), dest);
+  else if(statement==ID_gcc_local_label)
+    convert_gcc_local_label(code, dest);
   else if(statement==ID_for)
     convert_for(code, dest);
   else if(statement==ID_while)
@@ -399,9 +428,12 @@ void goto_convertt::convert(
     convert_msc_try_finally(code, dest);
   else if(statement==ID_msc_leave)
     convert_msc_leave(code, dest);
+  else if(statement==ID_catch) // C++ try/catch
+    convert_catch(code, dest);
   else
     copy(code, OTHER, dest);
 
+  // make sure dest is never empty
   if(dest.instructions.empty())
   {
     dest.add_instruction(SKIP);
@@ -517,11 +549,6 @@ void goto_convertt::convert_expression(
     tmp_code.else_case().location()=expr.location();
     convert_ifthenelse(tmp_code, dest);
   }
-  else if(expr.id()==ID_statement_expression)
-  {
-    assert(expr.operands().size()==1);
-    convert(to_code(expr.op0()), dest);
-  }
   else
   {
     clean_expr(expr, dest, false); // result _not_ used
@@ -1162,7 +1189,7 @@ void goto_convertt::case_guard(
 
   forall_expr(it, case_op)
   {
-    equality_exprt eq_expr;
+    equal_exprt eq_expr;
     eq_expr.lhs()=value;
     eq_expr.rhs()=*it;
     dest.move_to_operands(eq_expr);
@@ -1232,6 +1259,7 @@ void goto_convertt::convert_switch(
   targets.set_break(z);
   targets.set_default(z);
   targets.cases.clear();
+  targets.cases_map.clear();
 
   goto_programt tmp;
 
@@ -1886,6 +1914,71 @@ void goto_convertt::convert_msc_leave(
 
 /*******************************************************************\
 
+Function: goto_convertt::convert_catch
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void goto_convertt::convert_catch(
+  const codet &code,
+  goto_programt &dest)
+{
+  assert(code.operands().size()>=2);
+  
+  // add the CATCH-push instruction to 'dest'
+  goto_programt::targett catch_push_instruction=dest.add_instruction();
+  catch_push_instruction->make_catch();
+  catch_push_instruction->code.set_statement(ID_catch);
+  catch_push_instruction->location=code.location();
+  
+  // the CATCH-push instruction is annotated with a list of IDs,
+  // one per target
+  irept::subt &exception_list=
+    catch_push_instruction->code.add(ID_exception_list).get_sub();
+
+  // add a SKIP target for the end of everything
+  goto_programt end;
+  goto_programt::targett end_target=end.add_instruction();
+  end_target->make_skip();
+  
+  // the first operand is the 'try' block
+  convert(to_code(code.op0()), dest);
+  
+  // add the CATCH-pop to the end of the 'try' block
+  goto_programt::targett catch_pop_instruction=dest.add_instruction();
+  catch_pop_instruction->make_catch();
+  catch_pop_instruction->code.set_statement(ID_catch);
+  
+  // add a goto to the end of the 'try' block
+  dest.add_instruction()->make_goto(end_target);
+
+  for(unsigned i=1; i<code.operands().size(); i++)
+  {
+    const codet &block=to_code(code.operands()[i]);
+  
+    // grab the ID and add to CATCH instruction
+    exception_list.push_back(irept(block.get(ID_exception_id)));
+    
+    goto_programt tmp;
+    convert(block, tmp);
+    catch_push_instruction->targets.push_back(tmp.instructions.begin());
+    dest.destructive_append(tmp);
+
+    // add a goto to the end of the 'catch' block
+    dest.add_instruction()->make_goto(end_target);
+  }
+
+  // add end-target  
+  dest.destructive_append(end);
+}
+
+/*******************************************************************\
+
 Function: goto_convertt::convert_ifthenelse
 
   Inputs:
