@@ -9,12 +9,14 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_GOTO_PROGRAM_TEMPLATE_H
 #define CPROVER_GOTO_PROGRAM_TEMPLATE_H
 
-#include <assert.h>
+/*! \defgroup gr_goto_programs Goto programs */
 
-#include <iostream>
+#include <cassert>
+#include <ostream>
 #include <set>
 
 #include <namespace.h>
+#include <location.h>
 
 typedef enum { NO_INSTRUCTION_TYPE=0,
                GOTO=1,          // branch, possibly guarded
@@ -40,166 +42,113 @@ typedef enum { NO_INSTRUCTION_TYPE=0,
 
 std::ostream &operator<<(std::ostream &, goto_program_instruction_typet);
 
+/*! \brief A generic container class for a control flow graph
+           for one function, in the form of a goto-program
+    \ingroup gr_goto_programs
+*/
 template <class codeT, class guardT>
 class goto_program_templatet
 {
 public:
-  // DO NOT COPY ME! I HAVE POINTERS IN ME!
-  goto_program_templatet(const goto_program_templatet &src)
+  /*! \brief copy constructor
+      \param[in] src an empty goto program
+      \remark Use copy_from to copy non-empty goto-programs
+  */
+  inline goto_program_templatet(const goto_program_templatet &src)
   {
+    // DO NOT COPY ME! I HAVE POINTERS IN ME!
     assert(src.instructions.empty());
   }
 
-  // DO NOT COPY ME! I HAVE POINTERS IN ME!
-  goto_program_templatet &operator=(const goto_program_templatet &src)
+  /*! \brief assignment operator
+      \param[in] src an empty goto program
+      \remark Use copy_from to copy non-empty goto-programs
+  */
+  inline goto_program_templatet &operator=(const goto_program_templatet &src)
   {
+    // DO NOT COPY ME! I HAVE POINTERS IN ME!
     assert(src.instructions.empty());
     instructions.clear();
     update();
     return *this;
   }
 
+  /*! \brief Container for an instruction of the goto-program
+  */
   class instructiont
   {
   public:
     codeT code;
     
-    // function this belongs to
+    //! function this belongs to
     irep_idt function;
     
-    // keep track of the location in the source file
+    //! the location of the instruction in the source file
     locationt location;
     
-    // what kind of instruction?
+    //! what kind of instruction?
     goto_program_instruction_typet type;
 
-    // for gotos, assume, assert
+    //! guard for gotos, assume, assert
     guardT guard;
     
-    // for gotos
+    //! the target for gotos and for start_thread nodes
     typedef typename std::list<class instructiont>::iterator targett;
     typedef typename std::list<class instructiont>::const_iterator const_targett;
     typedef std::list<targett> targetst;
     typedef std::list<const_targett> const_targetst;
 
     targetst targets;
-    
+
+    //! goto target labels    
     typedef std::list<irep_idt> labelst;
     labelst labels;
 
     std::set<targett> incoming_edges;
 
-    bool is_target() const
+    //! is this node a branch target?
+    inline bool is_target() const
     { return target_number!=unsigned(-1); }
-    
-    void make_goto()
+
+    //! clear the node    
+    inline void clear(goto_program_instruction_typet _type)
     {
-      type=GOTO;
+      type=_type;
       targets.clear();
       guard.make_true();
       code.make_nil();
     }
      
-    void make_return()
-    {
-      type=RETURN;
-      targets.clear();
-      guard.make_true();
-      code.make_nil();
-    }
-     
-    void make_function_call(const codeT &_code)
-    {
-      type=FUNCTION_CALL;
-      targets.clear();
-      guard.make_true();
-      code=_code;
-    }
-    
-    void make_skip()
-    {
-      type=SKIP;
-      targets.clear();
-      guard.make_true();
-      code.make_nil();
-    }
-     
-    void make_throw()
-    {
-      type=THROW;
-      targets.clear();
-      guard.make_true();
-      code.make_nil();
-    }
-     
-    void make_catch()
-    {
-      type=CATCH;
-      targets.clear();
-      guard.make_true();
-      code.make_nil();
-    }
-     
-    void make_assertion(const guardT &g)
-    {
-      type=ASSERT;
-      targets.clear();
-      guard=g;
-      code.make_nil();
-    }
-     
-    void make_assumption(const guardT &g)
-    {
-      type=ASSUME;
-      targets.clear();
-      guard=g;
-      code.make_nil();
-    }
-     
-    void make_assignment()
-    {
-      type=ASSIGN;
-      targets.clear();
-      guard.make_nil();
-      code.make_nil();
-    }
-     
-    void make_goto(typename std::list<class instructiont>::iterator _target)
+    inline void make_goto() { clear(GOTO); }
+    inline void make_return() { clear(RETURN); }
+    inline void make_function_call(const codeT &_code) { clear(FUNCTION_CALL); code=_code; }
+    inline void make_skip() { clear(SKIP); }
+    inline void make_throw() { clear(THROW); }
+    inline void make_catch() { clear(CATCH); }
+    inline void make_assertion(const guardT &g) { clear(ASSERT); guard=g; }
+    inline void make_assumption(const guardT &g) { clear(ASSUME); guard=g; }
+    inline void make_assignment() { clear(ASSIGN); }
+    inline void make_other() { clear(OTHER); }
+    inline void make_decl() { clear(DECL); }
+    inline void make_dead() { clear(DEAD); }
+    inline void make_atomic_begin() { clear(ATOMIC_BEGIN); }
+    inline void make_atomic_end() { clear(ATOMIC_END); }
+
+    inline void make_goto(
+      typename std::list<class instructiont>::iterator _target)
     {
       make_goto();
       targets.push_back(_target);
     }
     
-    void make_goto(typename std::list<class instructiont>::iterator _target, const guardT &g)
+    inline void make_goto(
+      typename std::list<class instructiont>::iterator _target,
+      const guardT &g)
     {
       make_goto(_target);
       guard=g;
     }
-    
-    void make_other()
-    {
-      type=OTHER;
-      targets.clear();
-      guard.make_true();
-      code.make_nil();
-    }
-    
-    void make_decl()
-    {
-      type=DECL;
-      targets.clear();
-      guard.make_true();
-      code.make_nil();
-    }
-    
-    void make_dead()
-    {
-      type=DEAD;
-      targets.clear();
-      guard.make_true();
-      code.make_nil();
-    }
-    
+
     inline bool is_goto         () const { return type==GOTO;          }
     inline bool is_return       () const { return type==RETURN;        }
     inline bool is_assign       () const { return type==ASSIGN;        }
@@ -219,7 +168,7 @@ public:
     inline bool is_end_thread   () const { return type==END_THREAD;    }
     inline bool is_end_function () const { return type==END_FUNCTION;  }
 
-    instructiont():
+    inline instructiont():
       location(static_cast<const locationt &>(get_nil_irep())),
       type(NO_INSTRUCTION_TYPE),
       location_number(0),
@@ -228,7 +177,7 @@ public:
       guard.make_true();
     }
 
-    instructiont(goto_program_instruction_typet _type):
+    inline instructiont(goto_program_instruction_typet _type):
       location(static_cast<const locationt &>(get_nil_irep())),
       type(_type),
       location_number(0),
@@ -236,7 +185,8 @@ public:
     {
       guard.make_true();
     }
-    
+
+    //! swap two instructions    
     void swap(instructiont &instruction)
     {
       instruction.code.swap(code);
@@ -247,18 +197,19 @@ public:
       instruction.function.swap(function);
     }
     
-    // a globally unique number to identify a program location
-    // it's guaranteed to be ordered in program order within
-    // one goto_program
+    //! A globally unique number to identify a program location.
+    //! It's guaranteed to be ordered in program order within
+    //! one goto_program.
     unsigned location_number;
     
-    // number unique per function to identify loops
+    //! Number unique per function to identify loops
     unsigned loop_number;
     
-    // a number to identify branch targets
-    // this is -1 if it's not a target
+    //! A number to identify branch targets.
+    //! This is -1 if it's not a target.
     unsigned target_number;
 
+    //! Returns true if the instruction is a backwards branch.
     bool is_backwards_goto() const
     {
       if(!is_goto()) return false;
@@ -281,192 +232,118 @@ public:
   typedef typename std::list<targett> targetst;
   typedef typename std::list<const_targett> const_targetst;
 
+  //! The list of instructions in the goto program
   instructionst instructions;
   
   void get_successors(
     targett target,
-    targetst &successors)
-  {
-    successors.clear();
-    if(target==instructions.end()) return;
-
-    targett next=target;
-    next++;
-    
-    const instructiont &i=*target;
-  
-    if(i.is_goto())
-    {
-      for(typename targetst::const_iterator
-          t_it=i.targets.begin();
-          t_it!=i.targets.end();
-          t_it++)
-        successors.push_back(*t_it);
-  
-      if(!i.guard.is_true())
-        successors.push_back(next);
-    }
-    else if(i.is_start_thread())
-    {
-      for(typename targetst::const_iterator
-          t_it=i.targets.begin();
-          t_it!=i.targets.end();
-          t_it++)
-        successors.push_back(*t_it);
-  
-      successors.push_back(next);
-    }
-    else if(i.is_end_thread())
-    {
-      // no successors
-    }
-    else if(i.is_return())
-    {
-      // the successor is the end_function at the end
-      successors.push_back(--instructions.end());
-    }
-    else if(i.is_assume())
-    {
-      if(!i.guard.is_false())
-        successors.push_back(next);
-    }
-    else
-      successors.push_back(next);
-  }
+    targetst &successors);
 
   void get_successors(
     const_targett target,
-    const_targetst &successors) const
-  {
-    successors.clear();
-    if(target==instructions.end()) return;
-
-    const_targett next=target;
-    next++;
-    
-    const instructiont &i=*target;
-  
-    if(i.is_goto())
-    {
-      for(typename targetst::const_iterator
-          t_it=i.targets.begin();
-          t_it!=i.targets.end();
-          t_it++)
-        successors.push_back(*t_it);
-  
-      if(!i.guard.is_true())
-        successors.push_back(next);
-    }
-    else if(i.is_start_thread())
-    {
-      for(typename targetst::const_iterator
-          t_it=i.targets.begin();
-          t_it!=i.targets.end();
-          t_it++)
-        successors.push_back(*t_it);
-  
-      successors.push_back(next);
-    }
-    else if(i.is_end_thread())
-    {
-      // no successors
-    }
-    else if(i.is_return())
-    {
-      // the successor is the end_function at the end
-      successors.push_back(--instructions.end());
-    }
-    else if(i.is_assume())
-    {
-      if(!i.guard.is_false())
-        successors.push_back(next);
-    }
-    else
-      successors.push_back(next);
-  }
+    const_targetst &successors) const;
 
   void compute_incoming_edges();
 
-  // insertion that preserves jumps to "target"
+  //! Insertion that preserves jumps to "target".
+  //! The instruction is destroyed.
   void insert_before_swap(targett target, instructiont &instruction)
   {
+    assert(target!=instructions.end());
     targett next=target;
     next++;
     instructions.insert(next, instructiont())->swap(*target);
     target->swap(instruction);
   }
   
-  // insertion that preserves jumps to "target"
+  //! Insertion that preserves jumps to "target".
+  //! The program p is destroyed.
   void insert_before_swap(targett target, goto_program_templatet<codeT, guardT> &p)
   {
+    assert(target!=instructions.end());
     if(p.instructions.empty()) return;
     insert_before_swap(target, p.instructions.front());
     targett next=target;
     next++;
     p.instructions.erase(p.instructions.begin());
     instructions.splice(next, p.instructions);
+    // BUG: The iterators to p-instructions are invalidated!
   }
   
-  // this does not preserve jumps to target -- these move
-  targett insert_before(targett target)
+  //! Insertion before the given target
+  //! \return newly inserted location
+  inline targett insert_before(targett target)
   {
     return instructions.insert(target, instructiont());
   }
   
-  targett insert_after(targett target)
+  //! Insertion after the given target
+  //! \return newly inserted location
+  inline targett insert_after(targett target)
   {
     targett t=target;
     t++;
     return instructions.insert(t, instructiont());
   }
   
+  //! Appends the given program, which is destroyed
   inline void destructive_append(goto_program_templatet<codeT, guardT> &p)
   {
     instructions.splice(instructions.end(),
                         p.instructions);
+    // BUG: The iterators to p-instructions are invalidated!
   }
 
+  //! Inserts the given program at the given location.
+  //! The program is destroyed.
   inline void destructive_insert(
     targett target,
     goto_program_templatet<codeT, guardT> &p)
   {
     instructions.splice(target,
                         p.instructions);
+    // BUG: The iterators to p-instructions are invalidated!
   }
 
-  targett add_instruction()
+  //! Adds an instruction at the end.
+  //! \return The newly added instruction.
+  inline targett add_instruction()
   {
     instructions.push_back(instructiont());
     return --instructions.end();
   }
 
-  targett add_instruction(goto_program_instruction_typet type)
+  //! Adds an instruction of given type at the end.
+  //! \return The newly added instruction.
+  inline targett add_instruction(goto_program_instruction_typet type)
   {
     instructions.push_back(instructiont(type));
     return --instructions.end();
   }
 
-  // these assume that the targets are computed and numbered
+  //! Output goto program to given stream
   std::ostream &output(
-    const class namespacet &ns,
+    const namespacet &ns,
     const irep_idt &identifier,
     std::ostream &out) const;
-  
-  std::ostream& output(std::ostream &out) const
+
+  //! Output goto-program to given stream  
+  inline std::ostream &output(std::ostream &out) const
   {
     return output(namespacet(contextt()), "", out);
   }
-  
-  virtual std::ostream& output_instruction(
-    const class namespacet &ns,
+
+  //! Output a single instruction  
+  virtual std::ostream &output_instruction(
+    const namespacet &ns,
     const irep_idt &identifier,
     std::ostream &out,
     typename instructionst::const_iterator it) const=0;
 
-  // compute the target numbers
+  //! Compute the target numbers
   void compute_target_numbers();
 
-  // compute location numbers
+  //! Compute location numbers
   void compute_location_numbers(unsigned &nr)
   {
     for(typename instructionst::iterator
@@ -476,34 +353,26 @@ public:
       it->location_number=nr++;
   }
   
-  // compute location numbers
-  void compute_location_numbers()
+  //! Compute location numbers
+  inline void compute_location_numbers()
   {
     unsigned nr=0;
     compute_location_numbers(nr);
   }
   
-  // compute loop numbers
-  void compute_loop_numbers()
-  {
-    unsigned nr=0;
-    for(typename instructionst::iterator
-        it=instructions.begin();
-        it!=instructions.end();
-        it++)
-      if(it->is_backwards_goto())
-        it->loop_number=nr++;
-  }
-  
+  //! Compute loop numbers
+  void compute_loop_numbers();
+
+  //! Update all indices  
   void update();
   
-  // empty program?
-  bool empty() const
+  //! Is the program empty?
+  inline bool empty() const
   {
     return instructions.empty();
   }
 
-  // constructor/destructor
+  //! Constructor
   goto_program_templatet()
   {
   }
@@ -511,22 +380,146 @@ public:
   virtual ~goto_program_templatet()
   {
   }
-   
-  void swap(goto_program_templatet<codeT, guardT> &program)
+
+  //! Swap the goto program   
+  inline void swap(goto_program_templatet<codeT, guardT> &program)
   {
     program.instructions.swap(instructions);
   }
-  
-  void clear()
+
+  //! Clear the goto program  
+  inline void clear()
   {
     instructions.clear();
   }
-  
+
+  //! Copy a full goto program, preserving targets  
   void copy_from(const goto_program_templatet<codeT, guardT> &src);
-  
+
+  //! Does the goto program have an assertion?  
   bool has_assertion() const;
 }; 
 
+template <class codeT, class guardT>
+void goto_program_templatet<codeT, guardT>::compute_loop_numbers()
+{
+  unsigned nr=0;
+  for(typename instructionst::iterator
+      it=instructions.begin();
+      it!=instructions.end();
+      it++)
+    if(it->is_backwards_goto())
+      it->loop_number=nr++;
+}
+
+template <class codeT, class guardT>
+void goto_program_templatet<codeT, guardT>::get_successors(
+  targett target,
+  targetst &successors)
+{
+  successors.clear();
+  if(target==instructions.end()) return;
+
+  targett next=target;
+  next++;
+  
+  const instructiont &i=*target;
+
+  if(i.is_goto())
+  {
+    for(typename targetst::const_iterator
+        t_it=i.targets.begin();
+        t_it!=i.targets.end();
+        t_it++)
+      successors.push_back(*t_it);
+
+    if(!i.guard.is_true())
+      successors.push_back(next);
+  }
+  else if(i.is_start_thread())
+  {
+    for(typename targetst::const_iterator
+        t_it=i.targets.begin();
+        t_it!=i.targets.end();
+        t_it++)
+      successors.push_back(*t_it);
+
+    successors.push_back(next);
+  }
+  else if(i.is_end_thread())
+  {
+    // no successors
+  }
+  else if(i.is_throw())
+  {
+    // the successors are non-obvious
+  }
+  else if(i.is_return())
+  {
+    // the successor is the end_function at the end of the function
+    successors.push_back(--instructions.end());
+  }
+  else if(i.is_assume())
+  {
+    if(!i.guard.is_false())
+      successors.push_back(next);
+  }
+  else
+    successors.push_back(next);
+}
+
+template <class codeT, class guardT>
+void goto_program_templatet<codeT, guardT>::get_successors(
+  const_targett target,
+  const_targetst &successors) const
+{
+  successors.clear();
+  if(target==instructions.end()) return;
+
+  const_targett next=target;
+  next++;
+  
+  const instructiont &i=*target;
+
+  if(i.is_goto())
+  {
+    for(typename targetst::const_iterator
+        t_it=i.targets.begin();
+        t_it!=i.targets.end();
+        t_it++)
+      successors.push_back(*t_it);
+
+    if(!i.guard.is_true())
+      successors.push_back(next);
+  }
+  else if(i.is_start_thread())
+  {
+    for(typename targetst::const_iterator
+        t_it=i.targets.begin();
+        t_it!=i.targets.end();
+        t_it++)
+      successors.push_back(*t_it);
+
+    successors.push_back(next);
+  }
+  else if(i.is_end_thread())
+  {
+    // no successors
+  }
+  else if(i.is_return())
+  {
+    // the successor is the end_function at the end
+    successors.push_back(--instructions.end());
+  }
+  else if(i.is_assume())
+  {
+    if(!i.guard.is_false())
+      successors.push_back(next);
+  }
+  else
+    successors.push_back(next);
+}
+
 #include <langapi/language_util.h>
 #include <iomanip>
 
@@ -593,7 +586,10 @@ void goto_program_templatet<codeT, guardT>::compute_target_numbers()
       it++)
   {
     if(it->is_target())
+    {
       it->target_number=++cnt;
+      assert(it->target_number!=0);
+    }
   }
 
   // check the targets!
@@ -611,7 +607,10 @@ void goto_program_templatet<codeT, guardT>::compute_target_numbers()
     {
       targett t=*t_it;
       if(t!=instructions.end())
+      {
         assert(t->target_number!=0);
+        assert(t->target_number!=unsigned(-1));
+      }
     }
   }
   
