@@ -11,6 +11,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <rename.h>
 #include <cprover_prefix.h>
 #include <i2string.h>
+#include <symbol.h>
 
 #include <ansi-c/c_types.h>
 
@@ -618,6 +619,7 @@ void goto_convertt::remove_statement_expression(
   if(!result_is_used)
   {
     convert(code, dest);
+    expr.make_nil();
     return;
   }
   
@@ -708,10 +710,10 @@ void goto_convertt::remove_side_effect(
           statement==ID_predecrement)
     remove_pre(expr, dest);
   else if(statement==ID_cpp_new ||
-          statement=="cpp_new[]")
+          statement==ID_cpp_new_array)
     remove_cpp_new(expr, dest, result_is_used);
   else if(statement==ID_cpp_delete ||
-          statement=="cpp_delete[]")
+          statement==ID_cpp_delete_array)
     remove_cpp_delete(expr, dest, result_is_used);
   else if(statement==ID_malloc)
     remove_malloc(expr, dest, result_is_used);
@@ -725,12 +727,15 @@ void goto_convertt::remove_side_effect(
   }
   else if(statement==ID_throw)
   {
-    // the result can't be used, these are void
     goto_programt::targett t=dest.add_instruction(THROW);
     t->code=codet(ID_throw);
     t->code.operands().swap(expr.operands());
     t->code.location()=expr.location();
     t->location=expr.location();
+    t->code.set(ID_exception_list, expr.find(ID_exception_list));
+ 
+    // the result can't be used, these are void
+    expr.make_nil();
   }
   else
   {
