@@ -11,9 +11,12 @@ Date: May 2007
 #ifndef READ_BIN_GOTO_OBJECT_H_
 #define READ_BIN_GOTO_OBJECT_H_
 
-#include <message.h>
-#include <context.h>
-#include <goto-programs/goto_functions.h>
+#include <istream>
+#include <string>
+
+class contextt;
+class goto_functionst;
+class message_handlert;
 
 bool read_bin_goto_object(
   std::istream &in,
