@@ -9,10 +9,16 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_GOTO_SYMEX_GOTO_SYMEX_H
 #define CPROVER_GOTO_SYMEX_GOTO_SYMEX_H
 
-#include <std_types.h>
+/*! \defgroup goto_symex Symbolic execution of goto programs
+*/
+
 #include <goto-programs/goto_functions.h>
 
 #include "basic_symex.h"
+#include "goto_symex_state.h"
+
+class typet;
+class code_typet;
 
 class goto_symext:
   public basic_symext
@@ -89,6 +95,13 @@ protected:
     exprt &expr,
     statet &state,
     guardt &guard);
+    
+  static bool is_index_member_symbol_if(const exprt &expr);
+  
+  exprt address_arithmetic(
+    const exprt &expr,
+    statet &state,
+    guardt &guard);
   
   // guards
   
@@ -107,6 +120,7 @@ protected:
   virtual void claim(
     const exprt &expr,
     const std::string &msg,
+    unsigned priority,
     statet &state);
     
   // gotos
@@ -173,6 +187,11 @@ protected:
   std::map<irep_idt, unsigned> function_unwind;
   std::map<irep_idt, unsigned> function_frame;
   std::map<symex_targett::sourcet, unsigned> unwind_map;
+  
+  // exceptions
+  
+  void symex_throw(statet &state);
+  void symex_catch(statet &state);
 };
 
 #endif
