@@ -9,9 +9,11 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_GOTO_SYMEX_GOTO_SYMEX_STATE_H
 #define CPROVER_GOTO_SYMEX_GOTO_SYMEX_STATE_H
 
-#include <assert.h>
+#include <cassert>
 
 #include <guard.h>
+#include <std_expr.h>
+#include <i2string.h>
 
 #include <pointer-analysis/value_set.h>
 #include <goto-programs/goto_functions.h>
@@ -43,50 +45,104 @@ public:
   struct renaming_levelt
   {
   public:
-    virtual const irep_idt &get_original_name(const irep_idt &identifier) const;
-    virtual void get_original_name(exprt &expr) const;
-    virtual void rename(exprt &expr)=0;
-    virtual void rename(typet &type);
-    virtual void remove(const irep_idt &identifier)=0;
-
-    virtual std::string operator()(const irep_idt &identifier) const=0;
-    
+    virtual irep_idt current_name(const irep_idt &identifier) const=0;
+    virtual irep_idt name(const irep_idt &identifier, unsigned count) const=0;
     virtual ~renaming_levelt() { }
+
+    typedef std::map<irep_idt, unsigned> current_namest;
+    current_namest current_names;
     
-    virtual void print(std::ostream &out) const { }
+    void remove(const irep_idt &identifier) { current_names.erase(identifier); }
+    const irep_idt &get_original_name(const irep_idt &identifier) const;
+    void get_original_name(exprt &expr) const;
+    void get_original_name(typet &type) const;
+    void print(std::ostream &out) const;
+    unsigned current_count(const irep_idt &identifier) const;
+
+    irep_idt operator()(const irep_idt &identifier)
+    {
+      // see if it's already renamed
+      if(is_renamed(identifier)) return identifier;
+
+      // record
+      irep_idt i=current_name(identifier);
+      original_identifiers[i]=identifier;
+      return i;
+    }
+
+    inline void operator()(symbol_exprt &expr)
+    {
+      expr.set_identifier(operator()(expr.get_identifier()));
+    }
+
+    void rename(const irep_idt &identifier, unsigned count)
+    {
+      current_names[identifier]=count;
+      original_identifiers[name(identifier, count)]=identifier;
+    }
+    
+    inline bool is_renamed(const irep_idt &identifier) const
+    {
+      return original_identifiers.find(identifier)!=original_identifiers.end();
+    }
 
   protected:
     original_identifierst original_identifiers;
   };
   
+  // level 0 -- threads!
+  // renaming built for one particular interleaving
+  struct level0t:public renaming_levelt
+  {
+  public:
+    virtual irep_idt name(const irep_idt &identifier, unsigned thread_nr) const
+    {
+      return id2string(identifier)+"!"+i2string(thread_nr);
+    }
+
+    virtual irep_idt current_name(const irep_idt &identifier) const
+    {
+      assert(false);
+    }
+
+    irep_idt operator()(
+      const irep_idt &identifier,
+      const namespacet &ns,
+      unsigned thread_nr);
+
+    level0t() { }
+    virtual ~level0t() { }
+  } level0;
+
   // level 1 -- function frames
   // this is to preserve locality in case of recursion
   
   struct level1t:public renaming_levelt
   {
   public:
-    std::string name(
-      const irep_idt &identifier,
-      unsigned frame) const;
-      
-    typedef std::map<irep_idt, unsigned> current_namest;
-    current_namest current_names;
-
-    virtual void rename(exprt &expr);
-    virtual void rename(typet &type) { renaming_levelt::rename(type); }
-    virtual std::string operator()(const irep_idt &identifier) const;
-    virtual void remove(const irep_idt &identifier) { current_names.erase(identifier); }
-    
-    void rename(const irep_idt &identifier, unsigned frame)
+    virtual irep_idt name(const irep_idt &identifier, unsigned frame) const
     {
-      current_names[identifier]=frame;
-      original_identifiers[name(identifier, frame)]=identifier;
+      return id2string(identifier)+"@"+i2string(frame);
+    }
+    
+    virtual irep_idt current_name(const irep_idt &identifier) const
+    {
+      // see if it's already renamed
+      if(is_renamed(identifier))
+        return identifier;
+
+      // rename only if needed
+      const current_namest::const_iterator it=
+        current_names.find(identifier);
+    
+      if(it==current_names.end())
+        return identifier;
+      else
+        return name(identifier, it->second);
     }
 
     level1t() { }
     virtual ~level1t() { }
-
-    virtual void print(std::ostream &out) const;
   };
   
   // level 2 -- SSA
@@ -94,35 +150,9 @@ public:
   struct level2t:public renaming_levelt
   {
   public:
-    virtual void rename(exprt &expr);
-    virtual void rename(typet &type) { renaming_levelt::rename(type); }
-    virtual std::string operator()(const irep_idt &identifier) const;
-    virtual void remove(const irep_idt &identifier) { current_names.erase(identifier); }
-
-    std::string name(
-      const irep_idt &identifier,
-      unsigned count) const;
-
-    struct valuet
+    virtual irep_idt name(const irep_idt &identifier, unsigned count) const
     {
-      unsigned count;
-      exprt constant;
-      
-      valuet():
-        count(0),
-        constant(static_cast<const exprt &>(get_nil_irep()))
-      {
-      }
-    };
-    
-    typedef std::map<irep_idt, valuet> current_namest;
-    current_namest current_names;
-    
-    void rename(const irep_idt &identifier, unsigned count)
-    {
-      valuet &entry=current_names[identifier];
-      entry.count=count;
-      original_identifiers[name(identifier, entry.count)]=identifier;
+      return id2string(identifier)+"#"+i2string(count);
     }
 
     void get_variables(std::set<irep_idt> &vars) const
@@ -133,21 +163,47 @@ public:
         vars.insert(it->first);
     }
 
-    unsigned current_number(const irep_idt &identifier) const;
+    virtual irep_idt current_name(const irep_idt &identifier) const
+    {
+      // see if it's already renamed
+      if(is_renamed(identifier))
+        return identifier;
+
+      // _always_ rename
+      return name(identifier, current_count(identifier));
+    }
 
     level2t() { }
     virtual ~level2t() { }
-
-    virtual void print(std::ostream &out) const;
   } level2;
+  
+  // this maps L1 names to (L2) constants
+  class propagationt
+  {
+  public:
+    typedef std::map<irep_idt, exprt> valuest;
+    valuest values;
+    void operator()(exprt &expr);
 
-  void rename(exprt &expr, const namespacet &ns);
-  void rename_address(exprt &expr, const namespacet &ns);
-  void rename(typet &type, const namespacet &ns);
+    void remove(const irep_idt &identifier)
+    {
+      values.erase(identifier);
+    }
+    
+  } propagation;
+  
+  typedef enum { L0=0, L1=1, L2=2 } levelt;
+
+  // performs renaming _up to_ the given level
+  irep_idt rename(const irep_idt &identifier, const namespacet &ns, levelt level=L2);
+  void rename(exprt &expr, const namespacet &ns, levelt level=L2);
+  void rename(typet &type, const namespacet &ns, levelt level=L2);
+  
+  void rename_address(exprt &expr, const namespacet &ns, levelt level);
   
   void assignment(
-    exprt &lhs,
-    const exprt &rhs,
+    symbol_exprt &lhs, // L0/L1
+    const exprt &rhs,  // L2
     const namespacet &ns,
     bool record_value);
 
@@ -155,21 +211,27 @@ public:
   bool constant_propagation(const exprt &expr) const;
   bool constant_propagation_reference(const exprt &expr) const;
 
-  // undoes both levels of renaming
+  // undoes all levels of renaming
   const irep_idt &get_original_name(const irep_idt &identifier) const;
   void get_original_name(exprt &expr) const;
+  void get_original_name(typet &type) const;
   
-  // does both levels of renaming
-  std::string current_name(const irep_idt &identifier) const
+  // does all levels of renaming
+  irep_idt current_name(
+      const namespacet &ns,
+      const irep_idt &identifier)
   {
-    return current_name(level2, identifier);
+    return current_name(level2, ns, identifier);
   }
 
-  std::string current_name(
+  irep_idt current_name(
     const level2t &level2,
-    const irep_idt &identifier) const
+    const namespacet &ns,
+    const irep_idt &identifier)
   {
-    return level2(top().level1(identifier));
+    return level2.current_name(
+           top().level1.current_name(
+           level0(identifier, ns, source.thread_nr)));
   }
   
   // uses level 1 names, and is used to
@@ -193,21 +255,25 @@ public:
     }
   };
 
-  std::string current_name(
+  irep_idt current_name(
     const goto_statet &goto_state,
-    const irep_idt &identifier) const
+    const namespacet &ns,
+    const irep_idt &identifier)
   {
-    return current_name(goto_state.level2, identifier);
+    return current_name(goto_state.level2, ns, identifier);
   }
   
   // gotos
   typedef std::list<goto_statet> goto_state_listt;
   typedef std::map<goto_programt::const_targett, goto_state_listt> goto_state_mapt;
 
-  // function calls
+  // stack frames -- these are used for function calls and
+  // for exceptions
   class framet
   {
   public:
+    // function calls
+  
     irep_idt function_identifier;
     goto_state_mapt goto_state_map;
     level1t level1;
@@ -223,6 +289,10 @@ public:
       return_value(static_cast<const exprt &>(get_nil_irep()))
     {
     }
+
+    // exceptions
+    typedef std::map<irep_idt, goto_programt::targett> catch_mapt;
+    catch_mapt catch_map;
   };
   
   typedef std::vector<framet> call_stackt;
@@ -239,7 +309,7 @@ public:
     assert(!call_stack.empty());
     return call_stack.back();
   }
-
+  
   inline framet &new_frame() { call_stack.push_back(framet()); return call_stack.back(); }
   inline void pop_frame() { call_stack.pop_back(); }
   inline const framet &previous_frame() { return *(--(--call_stack.end())); }
