@@ -6,7 +6,8 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
+#include <cstdlib>
 
 #include <expr_util.h>
 #include <c_misc.h>
@@ -19,6 +20,9 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <std_expr.h>
 #include <cprover_prefix.h>
 #include <pointer_offset_size.h>
+#include <context.h>
+#include <guard.h>
+#include <options.h>
 
 #include <ansi-c/c_types.h>
 #include <ansi-c/c_typecast.h>
@@ -46,7 +50,7 @@ Function: dereferencet::has_dereference
 
 \*******************************************************************/
 
-bool dereferencet::has_dereference(const exprt &expr) const
+bool dereferencet::has_dereference(const exprt &expr)
 {
   forall_operands(it, expr)
     if(has_dereference(*it))
@@ -346,7 +350,7 @@ dereferencet::valuet dereferencet::build_reference_to(
       
       if(o.offset().is_zero())
       {
-        tmp_guard.add(equality_exprt(pointer_expr, null_pointer));
+        tmp_guard.add(equal_exprt(pointer_expr, null_pointer));
 
         dereference_callback.dereference_failure(
           "pointer dereference",
@@ -507,7 +511,7 @@ dereferencet::valuet dereferencet::build_reference_to(
     
     if(o.offset().is_zero())
     {
-      equality_exprt equality(pointer_expr, object_pointer);
+      equal_exprt equality(pointer_expr, object_pointer);
 
       if(ns.follow(equality.lhs().type())!=ns.follow(equality.rhs().type()))
         equality.lhs().make_typecast(equality.rhs().type());
@@ -543,9 +547,9 @@ dereferencet::valuet dereferencet::build_reference_to(
     else if(root_object_type.id()==ID_array &&
             dereference_type_compare(root_object_type.subtype(), dereference_type))
     {
-      // we have an array with a subtype that matches
-      // the dereferencing type
-      // we will require well-alignedness!
+      // We have an array with a subtype that matches
+      // the dereferencing type.
+      // We will require well-alignedness!
       
       exprt offset;
 
@@ -595,7 +599,7 @@ dereferencet::valuet dereferencet::build_reference_to(
       // this is relative to the root object
       const exprt offset=
         unary_exprt(ID_pointer_offset, pointer_expr, index_type());
-    
+
       if(memory_model(result.value, dereference_type, tmp_guard, offset))
       {
         // ok, done
@@ -815,18 +819,6 @@ bool dereferencet::memory_model(
     if(bv_width(from_type)==bv_width(to_type))
       return memory_model_conversion(value, to_type, guard, offset);
   }
-  else
-  {
-    // we may allow this on _anything_ that has a size
-
-    #if 0    
-    mp_integer from_size=pointer_offset_size(ns, from_type);
-    mp_integer to_size=pointer_offset_size(ns, to_type);
-    
-    if(from_size>=1 && from_size==to_size)
-      return memory_model_conversion(value, to_type, guard, offset);
-    #endif
-  }
 
   // otherwise, we will stich it together from bytes
   
@@ -872,7 +864,7 @@ bool dereferencet::memory_model_conversion(
 
   if(options.get_bool_option("pointer-check"))
   {
-    equality_exprt offset_not_zero(offset, gen_zero(offset.type()));
+    equal_exprt offset_not_zero(offset, gen_zero(offset.type()));
     offset_not_zero.make_not();
   
     guardt tmp_guard(guard);
@@ -905,20 +897,42 @@ bool dereferencet::memory_model_bytes(
 {
   const typet from_type=value.type();
   
-  // we refuse to convert to/from code
+  // We simply refuse to convert to/from code.
   if(from_type.id()==ID_code || to_type.id()==ID_code)
     return false;
 
-  // won't do this without a committment to an endianness
+  // We won't do this without a committment to an endianness.
   if(config.ansi_c.endianness==configt::ansi_ct::NO_ENDIANNESS)
     return false; 
 
   // But everything else we will try!
   // We just rely on byte_extract to do the job!
+  
+  exprt result;
 
-  exprt byte_extract(byte_extract_id(), to_type);
-  byte_extract.copy_to_operands(value, offset);
-  value=byte_extract;
+  // See if we have an array of bytes already,
+  // and we want something byte-sized.
+  if(ns.follow(from_type).id()==ID_array &&
+     pointer_offset_size(ns, ns.follow(from_type).subtype())==1 &&
+     pointer_offset_size(ns, to_type)==1 &&
+     is_a_bv_type(ns.follow(from_type).subtype()) &&
+     is_a_bv_type(to_type))
+  {
+    // yes, can use 'index'
+    result=index_exprt(value, offset, ns.follow(from_type).subtype());
+    
+    // possibly need to convert
+    if(!base_type_eq(result.type(), to_type, ns))
+      result.make_typecast(to_type);
+  }
+  else
+  {
+    // no, use 'byte_extract'
+    result=exprt(byte_extract_id(), to_type);
+    result.copy_to_operands(value, offset);
+  }
+  
+  value=result;
 
   // are we within the bounds?
   if(options.get_bool_option("pointer-check"))
