@@ -10,16 +10,21 @@ Author: Daniel Kroening, kroening@kroening.com
 #define CPROVER_POINTER_ANALYSIS_DEREFERENCE_H
 
 #include <set>
+#include <string>
 
-#include <expr.h>
 #include <hash_cont.h>
-#include <guard.h>
-#include <namespace.h>
-#include <options.h>
 #include <std_expr.h>
 
 #include "value_sets.h"
 
+class contextt;
+class guardt;
+class optionst;
+class modet;
+class symbolt;
+
+/*! \brief TO_BE_DOCUMENTED
+*/
 class dereference_callbackt
 {
 public:
@@ -43,9 +48,18 @@ public:
     const symbolt *&symbol)=0;
 };
 
+/*! \brief TO_BE_DOCUMENTED
+*/
 class dereferencet
 {
 public:
+  /*! \brief Constructor 
+   * \param _ns Namespace
+   * \param _new_context A context to store new symbols in
+   * \param _options Options, in particular whether pointer checks are
+            to be performed
+   * \param _dereference_callback Callback object for error reporting
+  */
   dereferencet(
     const namespacet &_ns,
     contextt &_new_context,
@@ -60,13 +74,29 @@ public:
   virtual ~dereferencet() { }
   
   typedef enum { READ, WRITE } modet;
+  
+  /*! 
+   * The method 'dereference' dereferences the
+   * given pointer-expression. Any errors are
+   * reported to the callback method given in the
+   * constructor.
+   *
+   * \param pointer A pointer-typed expression, to
+            be dereferenced.
+   * \param guard A guard, which is assumed to hold when
+            dereferencing.
+   * \param mode Indicates whether the dereferencing
+            is a load or store.
+  */
 
   virtual exprt dereference(
     const exprt &pointer,
     const guardt &guard,
     const modet mode);
     
-  bool has_dereference(const exprt &expr) const;
+  /*! \brief Returns 'true' iff the given expression contains unary '*'
+  */
+  static bool has_dereference(const exprt &expr);
 
   typedef hash_set_cont<exprt, irep_hash> expr_sett;
 
@@ -109,7 +139,7 @@ private:
              
   static const exprt &get_symbol(const exprt &object);
   
-  void bounds_check(const class index_exprt &expr, const guardt &guard);
+  void bounds_check(const index_exprt &expr, const guardt &guard);
   void valid_check(const exprt &expr, const guardt &guard, const modet mode);
   
   void invalid_pointer(const exprt &expr, const guardt &guard);
