@@ -9,7 +9,8 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_POINTER_ANALYSIS_REWRITE_INDEX_H
 #define CPROVER_POINTER_ANALYSIS_REWRITE_INDEX_H
 
-#include <std_expr.h>
+class dereference_exprt;
+class index_exprt;
 
 // rewrite a[i] to *(a+i)
 
