@@ -10,11 +10,10 @@ Author: Daniel Kroening, kroening@kroening.com
 #define CPROVER_GOTO_PROGRAMS_SHOW_VALUE_SETS_H
 
 #include <ui_message.h>
-#include <namespace.h>
 
-#include <goto-programs/goto_functions.h>
-
-#include "value_set_analysis.h"
+class goto_functionst;
+class goto_programt;
+class value_set_analysist;
 
 void show_value_sets(
   ui_message_handlert::uit ui,
