@@ -12,12 +12,13 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <set>
 
 #include <mp_arith.h>
-#include <namespace.h>
 #include <reference_counting.h>
 
 #include "object_numbering.h"
 #include "value_sets.h"
 
+class namespacet;
+
 class value_sett
 {
 public:
@@ -110,8 +111,6 @@ public:
   
   typedef std::set<exprt> expr_sett;
 
-  static void add_objects(const entryt &src, expr_sett &dest);
-
   #ifdef USE_DSTRING   
   typedef std::map<idt, entryt> valuest;
   #else
