@@ -11,6 +11,8 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <prefix.h>
 #include <cprover_prefix.h>
 #include <xml_irep.h>
+#include <context.h>
+#include <xml.h>
 
 #include <langapi/language_util.h>
 
