@@ -9,13 +9,13 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_POINTER_ANALYSIS_VALUE_PROPAGATION_H
 #define CPROVER_POINTER_ANALYSIS_VALUE_PROPAGATION_H
 
-#include <xml.h>
-
 #include <goto-programs/static_analysis.h>
 
 #include "value_set_domain.h"
 #include "value_sets.h"
 
+class xmlt;
+
 class value_set_analysist:
   public value_setst,
   public static_analysist<value_set_domaint>
