@@ -6,8 +6,9 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
 #include <ctype.h>
+#include <cstdlib>
 
 #include <arith_tools.h>
 #include <std_types.h>
@@ -92,8 +93,8 @@ Function: dplib_convt::gen_array_index_type
 
 typet dplib_convt::gen_array_index_type()
 {
-  typet t("signedbv");
-  t.set("width", 32);
+  typet t(ID_signedbv);
+  t.set(ID_width, 32);
   return t;
 }
 
@@ -134,7 +135,7 @@ void dplib_convt::convert_array_index(const exprt &expr)
   }
   else
   {
-    exprt tmp("typecast", gen_array_index_type());
+    exprt tmp(ID_typecast, gen_array_index_type());
     tmp.copy_to_operands(expr);
     convert_dplib_expr(tmp);
   }
@@ -154,8 +155,8 @@ Function: dplib_convt::convert_address_of_rec
 
 void dplib_convt::convert_address_of_rec(const exprt &expr)
 {
-  if(expr.id()=="symbol" ||
-     expr.id()=="constant" ||
+  if(expr.id()==ID_symbol ||
+     expr.id()==ID_constant ||
      expr.id()==ID_string_constant)
   {
     dplib_prop.out
@@ -164,7 +165,7 @@ void dplib_convt::convert_address_of_rec(const exprt &expr)
       << ", offset:="
       << bin_zero(config.ansi_c.pointer_width) << " #)";
   }
-  else if(expr.id()=="index")
+  else if(expr.id()==ID_index)
   {
     if(expr.operands().size()!=2)
       throw "index takes two operands";
@@ -174,9 +175,9 @@ void dplib_convt::convert_address_of_rec(const exprt &expr)
 
     if(index.is_zero())
     {
-      if(array.type().id()=="pointer")
+      if(array.type().id()==ID_pointer)
         convert_dplib_expr(array);
-      else if(array.type().id()=="array")
+      else if(array.type().id()==ID_array)
         convert_address_of_rec(array);
       else
         assert(false);
@@ -187,9 +188,9 @@ void dplib_convt::convert_address_of_rec(const exprt &expr)
       dplib_prop.out << dplib_pointer_type();
       dplib_prop.out << " = ";
       
-      if(array.type().id()=="pointer")
+      if(array.type().id()==ID_pointer)
         convert_dplib_expr(array);
-      else if(array.type().id()=="array")
+      else if(array.type().id()==ID_array)
         convert_address_of_rec(array);
       else
         assert(false);
@@ -201,7 +202,7 @@ void dplib_convt::convert_address_of_rec(const exprt &expr)
       dplib_prop.out << "))";
     }
   }
-  else if(expr.id()=="member")
+  else if(expr.id()==ID_member)
   {
     if(expr.operands().size()!=1)
       throw "member takes one operand";
@@ -220,8 +221,8 @@ void dplib_convt::convert_address_of_rec(const exprt &expr)
     mp_integer offset=member_offset(ns,
       to_struct_type(struct_op.type()), component_name);
     
-    typet index_type("unsignedbv");
-    index_type.set("width", config.ansi_c.pointer_width);
+    typet index_type(ID_unsignedbv);
+    index_type.set(ID_width, config.ansi_c.pointer_width);
 
     exprt index=from_integer(offset, index_type);
 
@@ -255,13 +256,13 @@ literalt dplib_convt::convert_rest(const exprt &expr)
   
   find_symbols(expr);
 
-  if(expr.id()=="=" || expr.id()=="notequal")
+  if(expr.id()==ID_equal || expr.id()==ID_notequal)
   {
     assert(expr.operands().size()==2);
 
     dplib_prop.out << "ASSERT " << dplib_prop.dplib_literal(l) << " <=> (";
     convert_dplib_expr(expr.op0());
-    dplib_prop.out << ((expr.id()=="=")?"=":"/=");
+    dplib_prop.out << ((expr.id()==ID_equal)?"=":"/=");
     convert_dplib_expr(expr.op1());
     dplib_prop.out << ");" << std::endl;
   }
@@ -331,7 +332,7 @@ Function: dplib_convt::convert_as_bv
 
 void dplib_convt::convert_as_bv(const exprt &expr)
 {
-  if(expr.type().id()=="bool")
+  if(expr.type().id()==ID_bool)
   {
     dplib_prop.out << "IF ";
     convert_dplib_expr(expr);
@@ -372,24 +373,24 @@ Function: dplib_convt::convert_dplib_expr
 
 void dplib_convt::convert_dplib_expr(const exprt &expr)
 {
-  if(expr.id()=="symbol")
+  if(expr.id()==ID_symbol)
   {
-    convert_identifier(expr.get_string("identifier"));
+    convert_identifier(expr.get_string(ID_identifier));
   }
-  else if(expr.id()=="nondet_symbol")
+  else if(expr.id()==ID_nondet_symbol)
   {
-    convert_identifier("nondet$"+expr.get_string("identifier"));
+    convert_identifier("nondet$"+expr.get_string(ID_identifier));
   }
-  else if(expr.id()=="typecast")
+  else if(expr.id()==ID_typecast)
   {
     assert(expr.operands().size()==1);
     const exprt &op=expr.op0();
     
-    if(expr.type().id()=="bool")
+    if(expr.type().id()==ID_bool)
     {
-      if(op.type().id()=="signedbv" ||
-         op.type().id()=="unsignedbv" ||
-         op.type().id()=="pointer")
+      if(op.type().id()==ID_signedbv ||
+         op.type().id()==ID_unsignedbv ||
+         op.type().id()==ID_pointer)
       {
         convert_dplib_expr(op);
         dplib_prop.out << "/=";
@@ -400,14 +401,14 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
         throw "TODO typecast1 "+op.type().id_string()+" -> bool";
       }
     }
-    else if(expr.type().id()=="signedbv" ||
-            expr.type().id()=="unsignedbv")
+    else if(expr.type().id()==ID_signedbv ||
+            expr.type().id()==ID_unsignedbv)
     {
-      unsigned to_width=atoi(expr.type().get("width").c_str());
+      unsigned to_width=atoi(expr.type().get(ID_width).c_str());
       
-      if(op.type().id()=="signedbv")
+      if(op.type().id()==ID_signedbv)
       {
-        unsigned from_width=atoi(op.type().get("width").c_str());
+        unsigned from_width=atoi(op.type().get(ID_width).c_str());
         
         if(from_width==to_width)
           convert_dplib_expr(op);
@@ -424,9 +425,9 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
           dplib_prop.out << ")[" << (to_width-1) << ":0]";
         }
       }
-      else if(op.type().id()=="unsignedbv")
+      else if(op.type().id()==ID_unsignedbv)
       {
-        unsigned from_width=atoi(op.type().get("width").c_str());
+        unsigned from_width=atoi(op.type().get(ID_width).c_str());
         
         if(from_width==to_width)
           convert_dplib_expr(op);
@@ -450,7 +451,7 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
           dplib_prop.out << ")[" << (to_width-1) << ":0]";
         }
       }
-      else if(op.type().id()=="bool")
+      else if(op.type().id()==ID_bool)
       {
         if(to_width>1)
         {
@@ -478,9 +479,9 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
               " -> "+expr.type().id_string();
       }
     }
-    else if(expr.type().id()=="pointer")
+    else if(expr.type().id()==ID_pointer)
     {
-      if(op.type().id()=="pointer")
+      if(op.type().id()==ID_pointer)
       {
         convert_dplib_expr(op);
       }
@@ -490,7 +491,7 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     else
       throw "TODO typecast4 ? -> "+expr.type().id_string();
   }
-  else if(expr.id()=="struct")
+  else if(expr.id()==ID_struct)
   {
     dplib_prop.out << "(# ";
     
@@ -508,24 +509,24 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
         it++, i++)
     {
       if(i!=0) dplib_prop.out << ", ";
-      dplib_prop.out << it->get("name");
+      dplib_prop.out << it->get(ID_name);
       dplib_prop.out << ":=";
       convert_dplib_expr(expr.operands()[i]);
     }
     
     dplib_prop.out << " #)";
   }
-  else if(expr.id()=="constant")
+  else if(expr.id()==ID_constant)
   {
-    if(expr.type().id()=="unsignedbv" ||
-       expr.type().id()=="signedbv" ||
-       expr.type().id()=="bv")
+    if(expr.type().id()==ID_unsignedbv ||
+       expr.type().id()==ID_signedbv ||
+       expr.type().id()==ID_bv)
     {
-      dplib_prop.out << "0bin" << expr.get("value");
+      dplib_prop.out << "0bin" << expr.get(ID_value);
     }
-    else if(expr.type().id()=="pointer")
+    else if(expr.type().id()==ID_pointer)
     {
-      const irep_idt &value=expr.get("value");
+      const irep_idt &value=expr.get(ID_value);
       
       if(value=="NULL")
       {
@@ -537,7 +538,7 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
       else
         throw "unknown pointer constant: "+id2string(value);
     }
-    else if(expr.type().id()=="bool")
+    else if(expr.type().id()==ID_bool)
     {
       if(expr.is_true())
         dplib_prop.out << "TRUE";
@@ -546,7 +547,7 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
       else
         throw "unknown boolean constant";
     }
-    else if(expr.type().id()=="array")
+    else if(expr.type().id()==ID_array)
     {
       dplib_prop.out << "ARRAY (i: " << array_index_type() << "):";
       
@@ -569,11 +570,11 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
       convert_dplib_expr(expr.op0());
       dplib_prop.out << "\n  ENDIF";
     }
-    else if(expr.type().id()=="integer" ||
-            expr.type().id()=="natural" ||
-            expr.type().id()=="range")
+    else if(expr.type().id()==ID_integer ||
+            expr.type().id()==ID_natural ||
+            expr.type().id()==ID_range)
     {
-      dplib_prop.out << expr.get("value");
+      dplib_prop.out << expr.get(ID_value);
     }
     else
     {
@@ -581,9 +582,9 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
       throw "unknown constant: "+expr.type().id_string();
     }
   }
-  else if(expr.id()=="concatenation" || 
-          expr.id()=="bitand" ||
-          expr.id()=="bitor")
+  else if(expr.id()==ID_concatenation || 
+          expr.id()==ID_bitand ||
+          expr.id()==ID_bitor)
   {
     dplib_prop.out << "(";
 
@@ -591,11 +592,11 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     {
       if(it!=expr.operands().begin())
       {
-        if(expr.id()=="concatenation")
+        if(expr.id()==ID_concatenation)
           dplib_prop.out << " @ ";
-        else if(expr.id()=="bitand")
+        else if(expr.id()==ID_bitand)
           dplib_prop.out << " & ";
-        else if(expr.id()=="bitor")
+        else if(expr.id()==ID_bitor)
           dplib_prop.out << " | ";
       }
 
@@ -604,7 +605,7 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
 
     dplib_prop.out << ")";
   }
-  else if(expr.id()=="bitxor")
+  else if(expr.id()==ID_bitxor)
   {
     assert(!expr.operands().empty());
   
@@ -634,7 +635,7 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
       dplib_prop.out << ")";
     }
   }
-  else if(expr.id()=="bitnand")
+  else if(expr.id()==ID_bitnand)
   {
     assert(expr.operands().size()==2);
 
@@ -644,18 +645,18 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     convert_dplib_expr(expr.op1());
     dplib_prop.out << ")";
   }
-  else if(expr.id()=="bitnot")
+  else if(expr.id()==ID_bitnot)
   {
     assert(expr.operands().size()==1);
     dplib_prop.out << "~(";
     convert_dplib_expr(expr.op0());
     dplib_prop.out << ")";
   }
-  else if(expr.id()=="unary-")
+  else if(expr.id()==ID_unary_minus)
   {
     assert(expr.operands().size()==1);
-    if(expr.type().id()=="unsignedbv" ||
-       expr.type().id()=="signedbv")
+    if(expr.type().id()==ID_unsignedbv ||
+       expr.type().id()==ID_signedbv)
     {
       dplib_prop.out << "BVUMINUS(";
       convert_dplib_expr(expr.op0());
@@ -664,7 +665,7 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     else
       throw "unsupported type for unary-: "+expr.type().id_string();
   }
-  else if(expr.id()=="if")
+  else if(expr.id()==ID_if)
   {
     assert(expr.operands().size()==3);
     dplib_prop.out << "IF ";
@@ -675,11 +676,11 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     convert_dplib_expr(expr.op2());
     dplib_prop.out << " ENDIF";
   }
-  else if(expr.id()=="and" ||
-          expr.id()=="or" ||
-          expr.id()=="xor")
+  else if(expr.id()==ID_and ||
+          expr.id()==ID_or ||
+          expr.id()==ID_xor)
   {
-    assert(expr.type().id()=="bool");
+    assert(expr.type().id()==ID_bool);
     
     if(expr.operands().size()>=2)
     {
@@ -687,11 +688,11 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
       {
         if(it!=expr.operands().begin())
         {
-          if(expr.id()=="and")
+          if(expr.id()==ID_and)
             dplib_prop.out << " AND ";
-          else if(expr.id()=="or")
+          else if(expr.id()==ID_or)
             dplib_prop.out << " OR ";
-          else if(expr.id()=="xor")
+          else if(expr.id()==ID_xor)
             dplib_prop.out << " XOR ";
         }
         
@@ -707,58 +708,58 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     else
       assert(false);
   }
-  else if(expr.id()=="not")
+  else if(expr.id()==ID_not)
   {
     assert(expr.operands().size()==1);
     dplib_prop.out << "NOT (";
     convert_dplib_expr(expr.op0());
     dplib_prop.out << ")";
   }
-  else if(expr.id()=="=" ||
-          expr.id()=="notequal")
+  else if(expr.id()==ID_equal ||
+          expr.id()==ID_notequal)
   {
     assert(expr.operands().size()==2);
     assert(expr.op0().type()==expr.op1().type());
 
-    if(expr.op0().type().id()=="bool")
+    if(expr.op0().type().id()==ID_bool)
     {
-      if(expr.id()=="notequal") dplib_prop.out << "NOT (";
+      if(expr.id()==ID_notequal) dplib_prop.out << "NOT (";
       dplib_prop.out << "(";
       convert_dplib_expr(expr.op0());
       dplib_prop.out << ") <=> (";
       convert_dplib_expr(expr.op1());
       dplib_prop.out << ")";
-      if(expr.id()=="notequal") dplib_prop.out << ")";
+      if(expr.id()==ID_notequal) dplib_prop.out << ")";
     }
     else
     {
       dplib_prop.out << "(";
       convert_dplib_expr(expr.op0());
       dplib_prop.out << ")";
-      dplib_prop.out << (expr.id()=="="?"=":"/=");
+      dplib_prop.out << (expr.id()==ID_equal?"=":"/=");
       dplib_prop.out << "(";
       convert_dplib_expr(expr.op1());
       dplib_prop.out << ")";
     }
   }
-  else if(expr.id()=="<=" ||
-          expr.id()=="<" ||
-          expr.id()==">=" ||
-          expr.id()==">")
+  else if(expr.id()==ID_le ||
+          expr.id()==ID_lt ||
+          expr.id()==ID_ge ||
+          expr.id()==ID_gt)
   {
     assert(expr.operands().size()==2);
     
     const typet &op_type=expr.op0().type();
 
-    if(op_type.id()=="unsignedbv")
+    if(op_type.id()==ID_unsignedbv)
     {
-      if(expr.id()=="<=")
+      if(expr.id()==ID_le)
         dplib_prop.out << "BVLE";
-      else if(expr.id()=="<")
+      else if(expr.id()==ID_lt)
         dplib_prop.out << "BVLT";
-      else if(expr.id()==">=")
+      else if(expr.id()==ID_ge)
         dplib_prop.out << "BVGE";
-      else if(expr.id()==">")
+      else if(expr.id()==ID_gt)
         dplib_prop.out << "BVGT";
       
       dplib_prop.out << "(";
@@ -767,15 +768,15 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
       convert_dplib_expr(expr.op1());
       dplib_prop.out << ")";
     }
-    else if(op_type.id()=="signedbv")
+    else if(op_type.id()==ID_signedbv)
     {
-      if(expr.id()=="<=")
+      if(expr.id()==ID_le)
         dplib_prop.out << "SBVLE";
-      else if(expr.id()=="<")
+      else if(expr.id()==ID_lt)
         dplib_prop.out << "SBVLT";
-      else if(expr.id()==">=")
+      else if(expr.id()==ID_ge)
         dplib_prop.out << "SBVGE";
-      else if(expr.id()==">")
+      else if(expr.id()==ID_gt)
         dplib_prop.out << "SBVGT";
       
       dplib_prop.out << "(";
@@ -787,14 +788,14 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     else
       throw "unsupported type for "+expr.id_string()+": "+expr.type().id_string();
   }
-  else if(expr.id()=="+")
+  else if(expr.id()==ID_plus)
   {
     if(expr.operands().size()>=2)
     {
-      if(expr.type().id()=="unsignedbv" ||
-         expr.type().id()=="signedbv")
+      if(expr.type().id()==ID_unsignedbv ||
+         expr.type().id()==ID_signedbv)
       {
-        dplib_prop.out << "BVPLUS(" << expr.type().get("width");
+        dplib_prop.out << "BVPLUS(" << expr.type().get(ID_width);
 
         forall_operands(it, expr)
         {
@@ -804,19 +805,19 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
           
         dplib_prop.out << ")";
       }
-      else if(expr.type().id()=="pointer")
+      else if(expr.type().id()==ID_pointer)
       {
         if(expr.operands().size()!=2)
           throw "pointer arithmetic with more than two operands";
         
         const exprt *p, *i;
         
-        if(expr.op0().type().id()=="pointer")
+        if(expr.op0().type().id()==ID_pointer)
         {
           p=&expr.op0();
           i=&expr.op1();
         }
-        else if(expr.op1().type().id()=="pointer")
+        else if(expr.op1().type().id()==ID_pointer)
         {
           p=&expr.op1();
           i=&expr.op0();
@@ -842,14 +843,14 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     else
       assert(false);
   }
-  else if(expr.id()=="-")
+  else if(expr.id()==ID_minus)
   {
     if(expr.operands().size()==2)
     {
-      if(expr.type().id()=="unsignedbv" ||
-         expr.type().id()=="signedbv")
+      if(expr.type().id()==ID_unsignedbv ||
+         expr.type().id()==ID_signedbv)
       {
-        dplib_prop.out << "BVSUB(" << expr.type().get("width") << ", ";
+        dplib_prop.out << "BVSUB(" << expr.type().get(ID_width) << ", ";
         convert_dplib_expr(expr.op0());
         dplib_prop.out << ", ";
         convert_dplib_expr(expr.op1());
@@ -865,19 +866,19 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     else
       assert(false);
   }
-  else if(expr.id()=="/")
+  else if(expr.id()==ID_div)
   {
     assert(expr.operands().size()==2);
 
-    if(expr.type().id()=="unsignedbv" ||
-       expr.type().id()=="signedbv")
+    if(expr.type().id()==ID_unsignedbv ||
+       expr.type().id()==ID_signedbv)
     {
-      if(expr.type().id()=="unsignedbv")
+      if(expr.type().id()==ID_unsignedbv)
         dplib_prop.out << "BVDIV";
       else
         dplib_prop.out << "SBVDIV";
 
-      dplib_prop.out << "(" << expr.type().get("width") << ", ";
+      dplib_prop.out << "(" << expr.type().get(ID_width) << ", ";
       convert_dplib_expr(expr.op0());
       dplib_prop.out << ", ";
       convert_dplib_expr(expr.op1());
@@ -886,19 +887,19 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     else
       throw "unsupported type for /: "+expr.type().id_string();
   }
-  else if(expr.id()=="mod")
+  else if(expr.id()==ID_mod)
   {
     assert(expr.operands().size()==2);
 
-    if(expr.type().id()=="unsignedbv" ||
-       expr.type().id()=="signedbv")
+    if(expr.type().id()==ID_unsignedbv ||
+       expr.type().id()==ID_signedbv)
     {
-      if(expr.type().id()=="unsignedbv")
+      if(expr.type().id()==ID_unsignedbv)
         dplib_prop.out << "BVMOD";
       else
         dplib_prop.out << "SBVMOD";
 
-      dplib_prop.out << "(" << expr.type().get("width") << ", ";
+      dplib_prop.out << "(" << expr.type().get(ID_width) << ", ";
       convert_dplib_expr(expr.op0());
       dplib_prop.out << ", ";
       convert_dplib_expr(expr.op1());
@@ -907,14 +908,14 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     else
       throw "unsupported type for mod: "+expr.type().id_string();
   }
-  else if(expr.id()=="*")
+  else if(expr.id()==ID_mult)
   {
     if(expr.operands().size()==2)
     {
-      if(expr.type().id()=="unsignedbv" ||
-         expr.type().id()=="signedbv")
+      if(expr.type().id()==ID_unsignedbv ||
+         expr.type().id()==ID_signedbv)
       {
-        dplib_prop.out << "BVMULT(" << expr.type().get("width") << ", ";
+        dplib_prop.out << "BVMULT(" << expr.type().get(ID_width) << ", ";
         convert_dplib_expr(expr.op0());
         dplib_prop.out << ", ";
         convert_dplib_expr(expr.op1());
@@ -930,23 +931,23 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     else
       assert(false);
   }
-  else if(expr.id()=="address_of" ||
+  else if(expr.id()==ID_address_of ||
           expr.id()=="implicit_address_of" ||
           expr.id()=="reference_to")
   {
     assert(expr.operands().size()==1);
-    assert(expr.type().id()=="pointer");
+    assert(expr.type().id()==ID_pointer);
     convert_address_of_rec(expr.op0());
   }
-  else if(expr.id()=="array_of")
+  else if(expr.id()==ID_array_of)
   {
-    assert(expr.type().id()=="array");
+    assert(expr.type().id()==ID_array);
     assert(expr.operands().size()==1);
     dplib_prop.out << "(ARRAY (i: " << array_index_type() << "): ";
     convert_array_value(expr.op0());
     dplib_prop.out << ")";
   }
-  else if(expr.id()=="index")
+  else if(expr.id()==ID_index)
   {
     assert(expr.operands().size()==2);
     dplib_prop.out << "(";
@@ -955,25 +956,25 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     convert_array_index(expr.op1());
     dplib_prop.out << "]";
   }
-  else if(expr.id()=="ashr" ||
-          expr.id()=="lshr" ||
-          expr.id()=="shl")
+  else if(expr.id()==ID_ashr ||
+          expr.id()==ID_lshr ||
+          expr.id()==ID_shl)
   {
     assert(expr.operands().size()==2);
 
-    if(expr.type().id()=="unsignedbv" ||
-       expr.type().id()=="signedbv")
+    if(expr.type().id()==ID_unsignedbv ||
+       expr.type().id()==ID_signedbv)
     {
-      if(expr.id()=="ashr")
+      if(expr.id()==ID_ashr)
         dplib_prop.out << "BVASHR";
-      else if(expr.id()=="lshr")
+      else if(expr.id()==ID_lshr)
         dplib_prop.out << "BVLSHR";
-      else if(expr.id()=="shl")
+      else if(expr.id()==ID_shl)
         dplib_prop.out << "BVSHL";
       else
         assert(false);
 
-      dplib_prop.out << "(" << expr.type().get("width") << ", ";
+      dplib_prop.out << "(" << expr.type().get(ID_width) << ", ";
       convert_dplib_expr(expr.op0());
       dplib_prop.out << ", ";
       convert_dplib_expr(expr.op1());
@@ -982,7 +983,7 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     else
       throw "unsupported type for "+expr.id_string()+": "+expr.type().id_string();
   }
-  else if(expr.id()=="with")
+  else if(expr.id()==ID_with)
   {
     assert(expr.operands().size()>=1);
     dplib_prop.out << "(";
@@ -995,21 +996,21 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
       const exprt &index=expr.operands()[i];
       const exprt &value=expr.operands()[i+1];
 
-      if(expr.type().id()=="struct")
+      if(expr.type().id()==ID_struct)
       {
-        dplib_prop.out << " WITH ." << index.get("component_name");
+        dplib_prop.out << " WITH ." << index.get(ID_component_name);
         dplib_prop.out << ":=(";
         convert_array_value(value);
         dplib_prop.out << ")";
       }
-      else if(expr.type().id()=="union")
+      else if(expr.type().id()==ID_union)
       {
-        dplib_prop.out << " WITH ." << index.get("component_name");
+        dplib_prop.out << " WITH ." << index.get(ID_component_name);
         dplib_prop.out << ":=(";
         convert_array_value(value);
         dplib_prop.out << ")";
       }
-      else if(expr.type().id()=="array")
+      else if(expr.type().id()==ID_array)
       {
         dplib_prop.out << " WITH [";
         convert_array_index(index);
@@ -1021,14 +1022,14 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
         throw "with expects struct or array type, but got "+expr.type().id_string();
     }
   }
-  else if(expr.id()=="member")
+  else if(expr.id()==ID_member)
   {
     assert(expr.operands().size()==1);
     convert_dplib_expr(expr.op0());
     dplib_prop.out << ".";
-    dplib_prop.out << expr.get("component_name");
+    dplib_prop.out << expr.get(ID_component_name);
   }
-  else if(expr.id()=="pointer_offset")
+  else if(expr.id()==ID_pointer_offset)
   {
     assert(expr.operands().size()==1);
     dplib_prop.out << "(";
@@ -1036,7 +1037,7 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     dplib_prop.out << ").offset";
   }
   #if 0
-  else if(expr.id()=="pointer_object")
+  else if(expr.id()==ID_pointer_object)
   {
     assert(expr.operands().size()==1);
     dplib_prop.out << "(";
@@ -1045,7 +1046,7 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     // TODO, this has the wrong type
   }
   #endif
-  else if(expr.id()=="same-object")
+  else if(expr.id()==ID_same_object)
   {
     assert(expr.operands().size()==2);
     dplib_prop.out << "(";
@@ -1058,12 +1059,12 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
   {
     convert_dplib_expr(to_string_constant(expr).to_array_expr());
   }
-  else if(expr.id()=="extractbit")
+  else if(expr.id()==ID_extractbit)
   {
     assert(expr.operands().size()==2);
 
-    if(expr.op0().type().id()=="unsignedbv" ||
-       expr.op0().type().id()=="signedbv")
+    if(expr.op0().type().id()==ID_unsignedbv ||
+       expr.op0().type().id()==ID_signedbv)
     {
       convert_dplib_expr(expr.op0());
       
@@ -1076,7 +1077,7 @@ void dplib_convt::convert_dplib_expr(const exprt &expr)
     else
       throw "unsupported type for "+expr.id_string()+": "+expr.op0().type().id_string();
   }
-  else if(expr.id()=="replication")
+  else if(expr.id()==ID_replication)
   {
     assert(expr.operands().size()==2);
   
@@ -1116,7 +1117,7 @@ Function: dplib_convt::set_to
 
 void dplib_convt::set_to(const exprt &expr, bool value)
 {
-  if(value && expr.id()=="and")
+  if(value && expr.id()==ID_and)
   {
     forall_operands(it, expr)
       set_to(*it, true);
@@ -1128,13 +1129,13 @@ void dplib_convt::set_to(const exprt &expr, bool value)
 
   dplib_prop.out << "// set_to " << (value?"true":"false") << std::endl;
 
-  if(expr.id()=="=" && value)
+  if(expr.id()==ID_equal && value)
   {
     assert(expr.operands().size()==2);
     
-    if(expr.op0().id()=="symbol")
+    if(expr.op0().id()==ID_symbol)
     {
-      const irep_idt &identifier=expr.op0().get("identifier");
+      const irep_idt &identifier=expr.op0().get(ID_identifier);
       
       identifiert &id=identifier_map[identifier];
 
@@ -1197,12 +1198,12 @@ void dplib_convt::find_symbols(const exprt &expr)
   forall_operands(it, expr)
     find_symbols(*it);
     
-  if(expr.id()=="symbol")
+  if(expr.id()==ID_symbol)
   {
-    if(expr.type().id()=="code")
+    if(expr.type().id()==ID_code)
       return;
 
-    const irep_idt &identifier=expr.get("identifier");
+    const irep_idt &identifier=expr.get(ID_identifier);
 
     identifiert &id=identifier_map[identifier];
 
@@ -1216,12 +1217,12 @@ void dplib_convt::find_symbols(const exprt &expr)
       dplib_prop.out << ";" << std::endl;
     }
   }  
-  else if(expr.id()=="nondet_symbol")
+  else if(expr.id()==ID_nondet_symbol)
   {
-    if(expr.type().id()=="code")
+    if(expr.type().id()==ID_code)
       return;
 
-    const irep_idt identifier="nondet$"+expr.get_string("identifier");
+    const irep_idt identifier="nondet$"+expr.get_string(ID_identifier);
 
     identifiert &id=identifier_map[identifier];
 
@@ -1251,24 +1252,24 @@ Function: dplib_convt::convert_dplib_type
 
 void dplib_convt::convert_dplib_type(const typet &type)
 {
-  if(type.id()=="array")
+  if(type.id()==ID_array)
   {
     const array_typet &array_type=to_array_type(type);
     
     dplib_prop.out << "ARRAY " << array_index_type()
                  << " OF ";
                  
-    if(array_type.subtype().id()=="bool")
+    if(array_type.subtype().id()==ID_bool)
       dplib_prop.out << "BITVECTOR(1)";
     else
       convert_dplib_type(array_type.subtype());
   }
-  else if(type.id()=="bool")
+  else if(type.id()==ID_bool)
   {
     dplib_prop.out << "boolean";
   }
-  else if(type.id()=="struct" ||
-          type.id()=="union")
+  else if(type.id()==ID_struct ||
+          type.id()==ID_union)
   {
     const struct_typet &struct_type=to_struct_type(type);
   
@@ -1284,23 +1285,23 @@ void dplib_convt::convert_dplib_type(const typet &type)
     {
       if(it!=components.begin()) dplib_prop.out << ",";
       dplib_prop.out << " ";
-      dplib_prop.out << it->get("name");
+      dplib_prop.out << it->get(ID_name);
       dplib_prop.out << ": ";
       convert_dplib_type(it->type());
     }
     
     dplib_prop.out << " #]";
   }
-  else if(type.id()=="pointer" ||
-          type.id()=="reference")
+  else if(type.id()==ID_pointer ||
+          type.id()==ID_reference)
   {
     dplib_prop.out << dplib_pointer_type();
   }
-  else if(type.id()=="integer")
+  else if(type.id()==ID_integer)
   {
     dplib_prop.out << "int";
   }
-  else if(type.id()=="signedbv")
+  else if(type.id()==ID_signedbv)
   {
     unsigned width=to_signedbv_type(type).get_width();
 
@@ -1309,7 +1310,7 @@ void dplib_convt::convert_dplib_type(const typet &type)
   
     dplib_prop.out << "signed[" << width << "]";
   }
-  else if(type.id()=="unsignedbv")
+  else if(type.id()==ID_unsignedbv)
   {
     unsigned width=to_unsignedbv_type(type).get_width();
       
@@ -1318,7 +1319,7 @@ void dplib_convt::convert_dplib_type(const typet &type)
   
     dplib_prop.out << "unsigned[" << width << "]";
   }
-  else if(type.id()=="bv")
+  else if(type.id()==ID_bv)
   {
     unsigned width=to_bv_type(type).get_width();
       
@@ -1345,13 +1346,13 @@ Function: dplib_convt::find_symbols
 
 void dplib_convt::find_symbols(const typet &type)
 {
-  if(type.id()=="array")
+  if(type.id()==ID_array)
   {
     const array_typet &array_type=to_array_type(type);
     find_symbols(array_type.size());
   }
-  else if(type.id()=="struct" ||
-          type.id()=="union")
+  else if(type.id()==ID_struct ||
+          type.id()==ID_union)
   {
   }
 }
