@@ -6,10 +6,11 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
-
+#include <cassert>
+#include <cstdlib>
 #include <map>
 #include <set>
+#include <iostream>
 
 #include <symbol.h>
 #include <mp_arith.h>
@@ -19,6 +20,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <std_types.h>
 #include <prefix.h>
 #include <std_expr.h>
+#include <threeval.h>
 
 #include <ansi-c/string_constant.h>
 
@@ -283,7 +285,7 @@ void boolbvt::convert_bitvector(const exprt &expr, bvt &bv)
   else if(expr.id()==ID_replication)
     return convert_replication(expr, bv);
   else if(expr.id()==ID_extractbits)
-    return convert_extractbits(expr, bv);
+    return convert_extractbits(to_extractbits_expr(expr), bv);
   else if(expr.id()==ID_bitnot || expr.id()==ID_bitand ||
           expr.id()==ID_bitor || expr.id()==ID_bitxor)
     return convert_bitwise(expr, bv);
@@ -329,6 +331,22 @@ void boolbvt::convert_bitvector(const exprt &expr, bvt &bv)
     
     return;
   }
+  #ifdef HAVE_FLOATBV
+  else if(expr.id()==ID_float_debug1 ||
+          expr.id()==ID_float_debug2)
+  {
+    assert(expr.operands().size()==2);
+    bvt bv0, bv1;
+    convert_bitvector(expr.op0(), bv0);
+    convert_bitvector(expr.op1(), bv1);
+    float_utilst float_utils(prop);
+    float_utils.spec=to_floatbv_type(expr.type());
+    bv=expr.id()==ID_float_debug1?
+      float_utils.debug1(bv0, bv1):
+      float_utils.debug2(bv0, bv1);
+    return;
+  }
+  #endif
 
   return conversion_failed(expr, bv);
 }
@@ -481,9 +499,6 @@ void boolbvt::convert_symbol(const exprt &expr, bvt &bv)
 {
   unsigned width=boolbv_width(expr.type());
 
-  if(width==0)
-    return conversion_failed(expr, bv);
-
   bv.resize(width);
   
   const irep_idt &identifier=expr.get(ID_identifier);
@@ -491,12 +506,20 @@ void boolbvt::convert_symbol(const exprt &expr, bvt &bv)
   if(identifier.empty())
     throw "convert_symbol got empty identifier";
 
-  for(unsigned i=0; i<width; i++)
-    bv[i]=map.get_literal(identifier, i, expr.type());
+  if(width==0)
+  {
+    // just put in map
+    map.get_map_entry(identifier, expr.type());
+  }
+  else
+  {
+    for(unsigned i=0; i<width; i++)
+      bv[i]=map.get_literal(identifier, i, expr.type());
 
-  for(unsigned i=0; i<width; i++)
-    if(bv[i].var_no()>=prop.no_variables() &&
-      !bv[i].is_constant()) { std::cout << identifier << std::endl; abort(); }
+    for(unsigned i=0; i<width; i++)
+      if(bv[i].var_no()>=prop.no_variables() &&
+        !bv[i].is_constant()) { std::cout << identifier << std::endl; abort(); }
+  }
 }
    
 /*******************************************************************\
@@ -518,20 +541,21 @@ literalt boolbvt::convert_rest(const exprt &expr)
     std::cerr << expr << std::endl;
     throw "boolbvt::convert_rest got non-boolean operand";
   }
-
+  
   const exprt::operandst &operands=expr.operands();
 
   if(expr.id()==ID_typecast)
     return convert_typecast(expr);
   else if(expr.id()==ID_equal)
-    return convert_equality(to_equality_expr(expr));
+    return convert_equality(to_equal_expr(expr));
   else if(expr.id()==ID_notequal)
   {
     if(expr.operands().size()!=2)
       throw "notequal expects two operands";
-
-    equality_exprt e(expr.op0(), expr.op1());
-    return prop.lnot(convert_equality(e));
+    
+    return prop.lnot(
+      convert_equality(
+        equal_exprt(expr.op0(), expr.op1())));
   }
   else if(expr.id()==ID_ieee_float_equal ||
           expr.id()==ID_ieee_float_notequal)
@@ -540,7 +564,7 @@ literalt boolbvt::convert_rest(const exprt &expr)
           expr.id()==ID_lt  || expr.id()==ID_gt)
     return convert_bv_rel(expr);
   else if(expr.id()==ID_extractbit)
-    return convert_extractbit(expr);
+    return convert_extractbit(to_extractbit_expr(expr));
   else if(expr.id()==ID_forall)
     return convert_quantifier(expr);
   else if(expr.id()==ID_exists)
@@ -713,12 +737,14 @@ bool boolbvt::boolbv_set_equality_to_true(const exprt &expr)
 
   if(operands.size()==2)
   {
+    const typet &type=ns.follow(operands[0].type());
+
     if(operands[0].id()==ID_symbol &&
-       operands[0].type()==operands[1].type() &&
-       operands[0].type().id()!=ID_bool)
+       type==ns.follow(operands[1].type()) &&
+       type.id()!=ID_bool)
     {
       // see if it is an unbounded array
-      if(is_unbounded_array(operands[0].type()))
+      if(is_unbounded_array(type))
         return true;
 
       bvt bv1;
@@ -727,8 +753,6 @@ bool boolbvt::boolbv_set_equality_to_true(const exprt &expr)
       const irep_idt &identifier=
         operands[0].get(ID_identifier);
 
-      const typet &type=operands[0].type();
-
       for(unsigned i=0; i<bv1.size(); i++)
         map.set_literal(identifier, i, type, bv1[i]);
 
@@ -785,7 +809,7 @@ Function: boolbvt::make_bv_expr
 
 void boolbvt::make_bv_expr(const typet &type, const bvt &bv, exprt &dest)
 {
-  dest=exprt("bv_literals", type);
+  dest=exprt(ID_bv_literals, type);
   irept::subt &bv_sub=dest.add(ID_bv).get_sub();
 
   bv_sub.resize(bv.size());
