@@ -28,7 +28,7 @@ void boolbvt::convert_array_of(const exprt &expr, bvt &bv)
   if(expr.operands().size()!=1)
     throw "array_of takes one operand";
 
-  if(expr.type().id()!="array")
+  if(expr.type().id()!=ID_array)
     throw "array_of takes array-typed operand";
   
   const array_typet &array_type=to_array_type(expr.type());
