@@ -14,6 +14,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <std_expr.h>
 #include <expr_util.h>
 #include <pointer_offset_size.h>
+#include <threeval.h>
 
 #include "bv_pointers.h"
 
@@ -90,7 +91,7 @@ literalt bv_pointerst::convert_rest(const exprt &expr)
       }
     }
   }
-  else if(expr.id()=="same-object")
+  else if(expr.id()==ID_same_object)
   {
     if(operands.size()==2 &&
        is_ptr(operands[0].type()) &&
@@ -374,10 +375,15 @@ void bv_pointerst::convert_pointer_type(const exprt &expr, bvt &bv)
   }
   else if(expr.id()==ID_constant)
   {
-    if(expr.get(ID_value)!=ID_NULL)
-      throw "found non-NULL pointer constant";
+    irep_idt value=to_constant_expr(expr).get_value();
 
-    encode(pointer_logic.get_null_object(), bv);
+    if(value==ID_NULL)
+      encode(pointer_logic.get_null_object(), bv);
+    else
+    {
+      mp_integer i=string2integer(id2string(value), 2);
+      bv=bv_utils.build_constant(i, bits);
+    }
 
     return;
   }
@@ -466,6 +472,18 @@ void bv_pointerst::convert_pointer_type(const exprt &expr, bvt &bv)
 
     return;
   }
+  else if(expr.id()==ID_lshr || 
+          expr.id()==ID_shl)
+    return SUB::convert_shift(expr, bv);
+  else if(expr.id()==ID_bitand ||
+          expr.id()==ID_bitor ||
+          expr.id()==ID_bitnot)
+    return SUB::convert_bitwise(expr, bv);
+  else if(expr.id()==ID_concatenation)
+    return SUB::convert_concatenation(expr, bv);
+  else if(expr.id()==ID_byte_extract_little_endian ||
+          expr.id()==ID_byte_extract_big_endian)
+    return SUB::convert_byte_extract(expr, bv);
 
   return conversion_failed(expr, bv);
 }
@@ -536,8 +554,8 @@ void bv_pointerst::convert_bitvector(const exprt &expr, bvt &bv)
     // we need to strip off the object part
     op0.resize(offset_bits);
 
-    // we add zeros if needed
-    bv=bv_utils.zero_extension(op0, width);
+    // we do a sign extension to permit negative offsets
+    bv=bv_utils.sign_extension(op0, width);
 
     return;
   }
