@@ -86,6 +86,9 @@ public:
   literalt is_zero(const bvt &op)
   { return prop.lnot(prop.lor(op)); }
 
+  literalt is_not_zero(const bvt &op)
+  { return prop.lor(op); }
+
   literalt is_one(const bvt &op);
 
   literalt is_all_ones(const bvt &op)
