@@ -11,6 +11,8 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <set>
 
+#include <std_expr.h>
+
 #include <solvers/prop/prop_conv.h>
 
 class functionst
@@ -24,7 +26,7 @@ public:
   }
 
   void record(
-    const class function_application_exprt &function_application);
+    const function_application_exprt &function_application);
 
   virtual void post_process()
   {
