@@ -35,14 +35,14 @@ typedef satcheck_booleforcet satcheckt;
 
 #include "satcheck_minisat.h"
 
-typedef satcheck_minisatt satcheckt;
+typedef satcheck_minisat1t satcheckt;
 
 #else
 #ifdef SATCHECK_MINISAT2
 
 #include "satcheck_minisat2.h"
 
-typedef satcheck_minisat_coret satcheckt;
+typedef satcheck_minisat_simplifiert satcheckt;
 
 #else
 #error NO SAT CHECKER
