@@ -9,6 +9,8 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_PROP_SMT2_DEC_H
 #define CPROVER_PROP_SMT2_DEC_H
 
+/*! \defgroup gr_smt2 SMT-LIB 2.x Interface */
+
 #include <fstream>
 
 #include "smt2_conv.h"
@@ -24,10 +26,13 @@ protected:
   std::string temp_out_filename, temp_result_filename;
 };
 
+/*! \brief TO_BE_DOCUMENTED
+    \ingroup gr_smt2
+*/
 class smt2_dect:protected smt2_temp_filet, public smt2_convt
 {
 public:
-  typedef enum { BOOLECTOR, CVC3, YICES, Z3 } solvert;
+  typedef enum { BOOLECTOR, CVC3, MATHSAT, YICES, Z3 } solvert;
 
   smt2_dect(
     const namespacet &_ns,
@@ -51,6 +56,7 @@ protected:
 
   resultt read_result_boolector(std::istream &in);
   resultt read_result_cvc3(std::istream &in);
+  resultt read_result_mathsat(std::istream &in);
   resultt read_result_yices(std::istream &in);
   resultt read_result_z3(std::istream &in);
   
