@@ -26,14 +26,16 @@ Function: smt2_propt::smt2_propt
 
 smt2_propt::smt2_propt(
   const std::string &benchmark,
-  const std::string &notes,
+  const std::string &source,
   const std::string &logic,
   std::ostream &_out):out(_out)
 {
-  out << "(benchmark " << benchmark << std::endl;
-  out << ":notes \"" << notes << "\"" << std::endl;
-  out << ":status unknown" << std::endl;
-  out << ":logic " << logic << " ; SMT 2" << std::endl;
+  out << "; SMT 2" << std::endl;
+  
+  out << "(set-info :source \"" << source << "\")" << std::endl;
+  out << "(set-option :produce-models true)" << std::endl;
+  out << "(set-logic " << logic << ")" << std::endl;
+
   _no_variables=0;
 }
 
@@ -68,8 +70,18 @@ Function: smt2_propt::finalize
 void smt2_propt::finalize()
 {
   out << std::endl;
-  out << ":formula true" << std::endl;
-  out << ") ; benchmark" << std::endl;
+  out << "(check-sat)" << std::endl;
+  out << std::endl;
+  
+  for(smt2_identifierst::const_iterator
+      it=smt2_identifiers.begin();
+      it!=smt2_identifiers.end();
+      it++)
+    out << "(get-value (" << *it << "))" << std::endl;
+  
+  out << std::endl;
+  
+  out << "; end of SMT2 file" << std::endl;
 }
 
 /*******************************************************************\
@@ -88,10 +100,10 @@ literalt smt2_propt::land(const bvt &bv)
 {
   out << std::endl;
 
-  literalt l=new_variable();
+  literalt l=define_new_variable();
 
-  out << ":assumption ; land" << std::endl;
-  out << " (= " << smt2_literal(l) << " (and";
+  out << "; land" << std::endl;
+  out << " (and";
   
   for(unsigned int i=0; i<bv.size(); i++)
     out << " " << smt2_literal(bv[i]);
@@ -117,10 +129,10 @@ literalt smt2_propt::lor(const bvt &bv)
 {
   out << std::endl;
 
-  literalt l=new_variable();
+  literalt l=define_new_variable();
 
-  out << ":assumption ; lor" << std::endl;
-  out << " (= " << smt2_literal(l) << " (or";
+  out << "; lor" << std::endl;
+  out << " (or";
 
   for(unsigned int i=0; i<bv.size(); i++)
     out << " " << smt2_literal(bv[i]);
@@ -149,10 +161,10 @@ literalt smt2_propt::lxor(const bvt &bv)
 
   out << std::endl;
 
-  literalt l=new_variable();
+  literalt l=define_new_variable();
 
-  out << ":assumption ; lxor" << std::endl;
-  out << " (= " << smt2_literal(l) << " (xor";
+  out << "; lxor" << std::endl;
+  out << " (xor";
 
   for(unsigned int i=0; i<bv.size(); i++)
     out << " " << smt2_literal(bv[i]);
@@ -184,10 +196,10 @@ literalt smt2_propt::land(literalt a, literalt b)
 
   out << std::endl;
 
-  literalt l=new_variable();
+  literalt l=define_new_variable();
 
-  out << ":assumption ; land" << std::endl;
-  out << " (= " << smt2_literal(l) << " (and";
+  out << "; land" << std::endl;
+  out << " (and";
   out << " " << smt2_literal(a);
   out << " " << smt2_literal(b);
   out << "))" << std::endl;
@@ -217,10 +229,10 @@ literalt smt2_propt::lor(literalt a, literalt b)
   
   out << std::endl;
 
-  literalt l=new_variable();
+  literalt l=define_new_variable();
 
-  out << ":assumption ; lor" << std::endl;
-  out << " (= " << smt2_literal(l) << " (or";
+  out << "; lor" << std::endl;
+  out << " (or";
   out << " " << smt2_literal(a);
   out << " " << smt2_literal(b);
   out << "))" << std::endl;
@@ -269,8 +281,8 @@ literalt smt2_propt::lxor(literalt a, literalt b)
 
   literalt l=new_variable();
 
-  out << ":assumption ; lxor" << std::endl;
-  out << " (= " << smt2_literal(l) << " (xor";
+  out << "; lxor" << std::endl;
+  out << " (xor";
   out << " " << smt2_literal(a);
   out << " " << smt2_literal(b);
   out << "))" << std::endl;
@@ -371,12 +383,12 @@ literalt smt2_propt::lselect(literalt a, literalt b, literalt c)
 
   out << std::endl;
 
-  literalt l=new_variable();
+  literalt l=define_new_variable();
 
-  out << ":assumption ; lselect" << std::endl;
-  out << " (= " << smt2_literal(l) << "(if_then_else "
+  out << "; lselect" << std::endl;
+  out << " (if_then_else "
       << smt2_literal(a) << " " << smt2_literal(b) << " "
-      << smt2_literal(c) << ")" << std::endl;
+      << smt2_literal(c) << "))" << std::endl;
 
   return l;
 }
@@ -399,7 +411,32 @@ literalt smt2_propt::new_variable()
   l.set(_no_variables, false);
   _no_variables++;
   
-  out << ":extrafuns((" << smt2_literal(l) << " Bool))" << std::endl;
+  out << "(declare-fun " << smt2_literal(l) << " () Bool)" << std::endl;
+
+  return l;
+}
+
+/*******************************************************************\
+
+Function: smt2_propt::define_new_variable
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+literalt smt2_propt::define_new_variable()
+{
+  literalt l;
+  l.set(_no_variables, false);
+  _no_variables++;
+  
+  out << "(define-fun " << smt2_literal(l) << " () Bool ";
+  // The command is continued elsewhere, and the
+  // closing parenthesis is missing!
 
   return l;
 }
@@ -419,11 +456,11 @@ Function: smt2_propt::lcnf
 void smt2_propt::lcnf(const bvt &bv)
 {
   out << std::endl;
-  out << ":assumption ; lcnf" << std::endl;
+  out << "(assert ; lcnf" << std::endl;
   out << " ";
 
   if(bv.empty())
-    out << "false ; the empty clause";
+    out << "false";
   else if(bv.size()==1)
     out << smt2_literal(bv.front());
   else
@@ -436,7 +473,7 @@ void smt2_propt::lcnf(const bvt &bv)
     out << ")";
   }
 
-  out << std::endl;
+  out << ")" <<  std::endl;
 }
 
 /*******************************************************************\
@@ -459,6 +496,8 @@ std::string smt2_propt::smt2_literal(literalt l)
     return "true";
     
   std::string v="B"+i2string(l.var_no());
+  
+  smt2_identifiers.insert(v);
 
   if(l.sign())
     return "(not "+v+")";  
