@@ -1,4 +1,4 @@
-#include <type.h>
+class typet;
 
 typedef enum { BV_UNKNOWN, BV_NONE, BV_SIGNED, BV_UNSIGNED } bv_semt;
 
