@@ -10,6 +10,8 @@ Author: Daniel Kroening, kroening@kroening.com
 #include "config.h"
 #include "context.h"
 #include "arith_tools.h"
+#include "cmdline.h"
+#include "simplify_expr.h"
 
 configt config;
 
@@ -259,7 +261,14 @@ bool configt::set(const cmdlinet &cmdline)
     ansi_c.set_32();
     
   if(cmdline.isset("64"))
-    ansi_c.set_64();
+  {
+    // there are numerous flavors!
+    #ifdef _WIN32
+    ansi_c.set_LLP64();
+    #else
+    ansi_c.set_LP64();
+    #endif
+  }
 
   if(cmdline.isset("LP64"))
     ansi_c.set_LP64();  // int=32, long=64, pointer=64
@@ -322,7 +331,8 @@ bool configt::set(const cmdlinet &cmdline)
     ansi_c.arch=configt::ansi_ct::ARCH_X86_64;
     ansi_c.endianness=configt::ansi_ct::IS_LITTLE_ENDIAN;
     ansi_c.lib=configt::ansi_ct::LIB_FULL;
-    ansi_c.set_64();
+    // note that sizeof(void *)==8, but sizeof(long)==4!
+    ansi_c.set_LLP64();
     ansi_c.wchar_t_width=2*8;
   }
 
@@ -424,10 +434,13 @@ int configt::ansi_ct::from_ns(const namespacet &ns, const std::string &what)
 
   if(ns.lookup(id, symbol))
     throw "failed to find "+id2string(id);
+    
+  exprt tmp=symbol->value;
+  simplify(tmp, ns);
   
   mp_integer int_value;
   
-  if(to_integer(symbol->value, int_value))
+  if(to_integer(tmp, int_value))
     throw "failed to convert "+id2string(id);
     
   return integer2long(int_value);
