@@ -354,6 +354,12 @@ void ieee_floatt::extract(
   mp_integer &_fraction,
   mp_integer &_exponent) const
 {
+  if(is_zero() || is_NaN() || is_infinity())
+  {
+    _fraction=_exponent=0;
+    return;
+  }
+
   _exponent=exponent;
   _fraction=fraction;
 
@@ -985,7 +991,10 @@ Function: operator ==
 bool operator ==(const ieee_floatt &a, const ieee_floatt &b)
 {
   // packed equality!
-  if(a.NaN && b.NaN) return true;
+  if(a.NaN && b.NaN)
+    return true;
+  else if(a.NaN || b.NaN)
+    return false;
 
   if(a.infinity && b.infinity &&
      a.sign == b.sign) return true;
@@ -1456,6 +1465,14 @@ void ieee_floatt::next_representable(bool greater)
   
   bool old_sign = get_sign();
 
+  if(is_zero())
+  {
+    unpack(1);
+    set_sign(!greater);
+
+    return;
+  }
+
   if(is_infinity())
   {
     if(get_sign() == greater)
@@ -1466,28 +1483,45 @@ void ieee_floatt::next_representable(bool greater)
     return;
   }
   
-  int dir;
+  bool dir;
   if(greater)
   {
     if(get_sign())
-      dir = -1;
+      dir = false;
     else
-      dir = 1;
+      dir = true;
   } 
   else
   {
     if(get_sign())
-      dir = 1;
+      dir = true;
     else
-      dir = -1;
+      dir = false;
   }
 
-  mp_integer new_exp = exponent;
-  mp_integer new_frac = fraction + dir;
-  
-  if(get_sign())
-    new_frac.negate();
+  set_sign(false);
 
-  build(new_frac, new_exp);
+  mp_integer old = pack();
+  if(dir)
+    ++old;
+  else
+    --old;
+
+  unpack(old);
+
+  //sign change impossible (zero case caught earler)
+  set_sign(old_sign);
+
+  //mp_integer new_exp = exponent;
+  //mp_integer new_frac = fraction + dir;
+
+  //std::cout << exponent << ":" << fraction << std::endl;
+  //std::cout << new_exp << ":" << new_frac << std::endl;
+  
+  //if(get_sign())
+  //  new_frac.negate();
+
+  //new_exp -= spec.f;
+  //build(new_frac, new_exp);
 }
 
