@@ -9,10 +9,14 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_CBMC_PRETTY_NAMES_H
 #define CPROVER_CBMC_PRETTY_NAMES_H
 
+// THIS FILE WILL GO AWAY
+
 #include <set>
 #include <map>
 
-#include <namespace.h>
+#include <irep.h>
+
+class namespacet;
 
 class pretty_namest
 {
