@@ -27,6 +27,8 @@ Author: Daniel Kroening, kroening@kroening.com
 #include "rational_tools.h"
 #include "config.h"
 #include "base_type.h"
+#include "namespace.h"
+#include "threeval.h"
 
 //#define DEBUGX
 
@@ -103,7 +105,7 @@ bool simplify_exprt::simplify_typecast(exprt &expr)
   // elminiate casts to bool
   if(expr_type==bool_typet())
   {
-    equality_exprt equality;
+    equal_exprt equality;
     equality.location()=expr.location();
     equality.lhs()=expr.op0();
     equality.rhs()=gen_zero(ns.follow(expr.op0().type()));
@@ -186,8 +188,10 @@ bool simplify_exprt::simplify_typecast(exprt &expr)
         return false;
       }
 
-      if(expr_type_id==ID_unsignedbv || expr_type_id==ID_signedbv)
+      if(expr_type_id==ID_unsignedbv ||
+         expr_type_id==ID_signedbv)
       {
+        unsigned expr_width=to_bitvector_type(expr_type).get_width();
         new_expr.set(ID_value, integer2binary(int_value, expr_width));
         expr.swap(new_expr);
         return false;
@@ -271,6 +275,7 @@ bool simplify_exprt::simplify_typecast(exprt &expr)
          expr_type_id==ID_signedbv ||
          expr_type_id==ID_bv)
       {
+        unsigned expr_width=to_bitvector_type(expr_type).get_width();
         new_expr.set(ID_value, integer2binary(int_value, expr_width));
         expr.swap(new_expr);
 
@@ -368,6 +373,7 @@ bool simplify_exprt::simplify_typecast(exprt &expr)
          expr_type_id==ID_signedbv ||
          expr_type_id==ID_floatbv)
       {
+        unsigned expr_width=to_bitvector_type(expr_type).get_width();
         mp_integer int_value=binary2integer(
           id2string(value), false);
         new_expr.set(ID_value, integer2binary(int_value, expr_width));
@@ -574,6 +580,23 @@ bool simplify_exprt::simplify_address_of_arg(exprt &expr)
       if(!simplify_rec(expr.op0())) result=false;
       if(!simplify_address_of_arg(expr.op1())) result=false;
       if(!simplify_address_of_arg(expr.op2())) result=false;
+
+      // op0 is a constant?
+      if(expr.op0().is_true())
+      {
+        result=false;
+        exprt tmp;
+        tmp.swap(expr.op1());
+        expr.swap(tmp);
+      }
+      else if(expr.op0().is_false())
+      {
+        result=false;
+        exprt tmp;
+        tmp.swap(expr.op2());
+        expr.swap(tmp);
+      }
+      
       return result;
     }
   }
@@ -670,10 +693,11 @@ bool simplify_exprt::simplify_pointer_offset(exprt &expr)
     if(ptr.operands().size()!=1) return true;
     
     const typet &op_type=ns.follow(ptr.op0().type());
-
+    
     if(op_type.id()==ID_pointer)
     {
-      // this just passes through, remove typecast
+      // Cast from pointer to pointer.
+      // This just passes through, remove typecast.
       exprt tmp=ptr.op0();
       ptr=tmp;
     
@@ -681,14 +705,53 @@ bool simplify_exprt::simplify_pointer_offset(exprt &expr)
       simplify_node(expr);
       return false;
     }
-    else if(op_type.id()==ID_signedbv || op_type.id()==ID_unsignedbv)
+    else if(op_type.id()==ID_signedbv ||
+            op_type.id()==ID_unsignedbv)
     {
-      exprt tmp=ptr.op0();
-      if(op_type!=ns.follow(expr.type()))
+      // Cast from integer to pointer, say (int *)x.
+      
+      if(ptr.op0().is_constant())
+      {
+        // (T *)0x1234 -> 0x1234
+        exprt tmp=ptr.op0();
         tmp.make_typecast(expr.type());
-      expr=tmp;
-      simplify_node(expr);
-      return false;
+        simplify_node(tmp);
+        expr.swap(tmp);
+        return false;
+      }
+      else
+      {
+        // We do a bit of special treatment for (TYPE *)(a+(int)&o),
+        // which is re-written to 'a'.
+
+        typet type=ns.follow(expr.type());      
+        exprt tmp=ptr.op0();
+        if(tmp.id()==ID_plus && tmp.operands().size()==2)
+        {
+          if(tmp.op0().id()==ID_typecast &&
+             tmp.op0().operands().size()==1 &&
+             tmp.op0().op0().id()==ID_address_of)
+          {
+            expr=tmp.op1();
+            if(type!=expr.type())
+              expr.make_typecast(type);
+
+            simplify_node(expr);
+            return false;
+          }
+          else if(tmp.op1().id()==ID_typecast &&
+                  tmp.op1().operands().size()==1 &&
+                  tmp.op1().op0().id()==ID_address_of)
+          {
+            expr=tmp.op0();
+            if(type!=expr.type())
+              expr.make_typecast(type);
+
+            simplify_node(expr);
+            return false;
+          }
+        }
+      }
     }
   }
   else if(ptr.id()==ID_plus) // pointer arithmetic
@@ -1114,7 +1177,7 @@ bool simplify_exprt::simplify_addition(exprt &expr)
   
   assert(expr.id()==ID_plus);
   
-  // count constants
+  // count the constants
   unsigned count=0;
   forall_operands(it, expr)
     if(is_number(it->type()) && it->is_constant())
@@ -1148,7 +1211,7 @@ bool simplify_exprt::simplify_addition(exprt &expr)
   }
   
   // delete non-float zeros 
-  // (for float's the result of 0.0 + (-0.0) may not be -0.0 in std rounding)
+  // (for floats the result of 0.0 + (-0.0) need not be -0.0 in std rounding)
   for(exprt::operandst::iterator
       it=operands.begin();
       it!=operands.end();
@@ -1320,7 +1383,7 @@ bool simplify_exprt::simplify_bitwise(exprt &expr)
     
   // try to merge constants
   
-  unsigned width=bv_width(expr.type());
+  unsigned width=to_bitvector_type(expr.type()).get_width();
     
   while(expr.operands().size()>=2)
   {
@@ -1421,7 +1484,7 @@ bool simplify_exprt::simplify_extractbit(exprt &expr)
   if(!is_bitvector_type(op0_type))
     return true;
   
-  unsigned width=bv_width(op0_type);
+  unsigned width=to_bitvector_type(op0_type).get_width();
 
   assert(expr.operands().size()==2);
 
@@ -2436,7 +2499,7 @@ bool simplify_exprt::simplify_inequality_address_of(exprt &expr)
   {
     bool equal=
        expr.op0().op0().get(ID_identifier)==
-       expr.op0().op0().get(ID_identifier);
+       expr.op1().op0().get(ID_identifier);
        
     expr.make_bool(expr.id()==ID_equal?equal:!equal);
     
@@ -2454,7 +2517,7 @@ Function: simplify_exprt::simplify_inequality
 
  Outputs:
 
- Purpose:
+ Purpose: simplifies inequalities !=, <=, <, >=, >, and also ==
 
 \*******************************************************************/
 
@@ -2485,7 +2548,8 @@ bool simplify_exprt::simplify_inequality(exprt &expr)
   exprt tmp1=expr.op1();
   ns.follow_symbol(tmp0.type());
   ns.follow_symbol(tmp1.type());
-  
+
+  // are _both_ constant?  
   if(op0_is_const && op1_is_const)
   {
     if(tmp0.id()==ID_bool)
@@ -2618,15 +2682,20 @@ bool simplify_exprt::simplify_inequality(exprt &expr)
     
     expr.op0().swap(expr.op1());
     
+    // one is constant
     simplify_inequality_constant(expr);
     return false;
   }
   else if(op1_is_const)
   {
+    // one is constant
     return simplify_inequality_constant(expr); 
   }
   else
+  {
+    // both are not constant
     return simplify_inequality_not_constant(expr);
+  }
     
   assert(false);
   return false;
@@ -2819,7 +2888,7 @@ bool simplify_exprt::simplify_inequality_not_constant(exprt &expr)
 
 Function: simplify_exprt::simplify_inequality_constant
 
-  Inputs:
+  Inputs: an inequality with a constant on the RHS
 
  Outputs:
 
@@ -2867,9 +2936,10 @@ bool simplify_exprt::simplify_inequality_constant(exprt &expr)
               expr.op0().type().id()==ID_pointer)
       {
         // (type)ptr == NULL -> ptr == NULL
+        // note that 'ptr' may be an integer
         exprt op=expr.op0().op0();
         expr.op0().swap(op);
-        expr.op1().type()=expr.op0().type(); // fix type
+        expr.op1()=gen_zero(expr.op0().type());
         simplify_inequality(expr); // do again!
         return false;
       }
@@ -2976,13 +3046,21 @@ bool simplify_exprt::simplify_inequality_constant(exprt &expr)
      expr.op0().op0().type().id()==ID_bool)
   {
     // we re-write (TYPE)boolean == 0 -> !boolean
-    if(expr.op1().is_zero())
+    if(expr.op1().is_zero() && expr.id()==ID_equal)
     {
       exprt tmp=expr.op0().op0();
       tmp.make_not();
       expr.swap(tmp);
       return false;
     }
+
+    // we re-write (TYPE)boolean != 0 -> boolean
+    if(expr.op1().is_zero() && expr.id()==ID_notequal)
+    {
+      exprt tmp=expr.op0().op0();
+      expr.swap(tmp);
+      return false;
+    }
   }
 
   return true;
@@ -3235,7 +3313,7 @@ bool simplify_exprt::simplify_index(index_exprt &expr)
     {
       // turn (a with i:=x)[j] into (i==j)?x:a[j]
       // watch out that the type of i and j might be different
-      equality_exprt equality_expr(expr.op1(), with_expr.op1());
+      equal_exprt equality_expr(expr.op1(), with_expr.op1());
       
       if(equality_expr.lhs().type()!=equality_expr.rhs().type())
         equality_expr.rhs().make_typecast(equality_expr.lhs().type());
@@ -3348,15 +3426,45 @@ bool simplify_exprt::simplify_object(exprt &expr)
   }
   else if(expr.id()==ID_typecast)
   {
-    if(expr.operands().size()==1 &&
-       ns.follow(expr.op0().type()).id()==ID_pointer)
+    const typet &op_type=ns.follow(expr.op0().type());
+  
+    assert(expr.operands().size()==1);
+    
+    if(op_type.id()==ID_pointer)
     {
+      // cast from pointer to pointer
       exprt tmp;
       tmp.swap(expr.op0());
       expr.swap(tmp);
       simplify_object(expr);
       return false;
     }
+    else if(op_type.id()==ID_signedbv || op_type.id()==ID_unsignedbv)
+    {
+      // cast from integer to pointer
+
+      // We do a bit of special treatment for (TYPE *)(a+(int)&o),
+      // which is re-written to '&o'.
+
+      exprt tmp=expr.op0();
+      if(tmp.id()==ID_plus && tmp.operands().size()==2)
+      {
+        if(tmp.op0().id()==ID_typecast &&
+           tmp.op0().operands().size()==1 &&
+           tmp.op0().op0().id()==ID_address_of)
+        {
+          expr=tmp.op0().op0();
+          return false;
+        }
+        else if(tmp.op1().id()==ID_typecast &&
+                tmp.op1().operands().size()==1 &&
+                tmp.op1().op0().id()==ID_address_of)
+        {
+          expr=tmp.op1().op0();
+          return false;
+        }
+      }
+    }
   }
 
   return true;
@@ -3568,7 +3676,11 @@ bool simplify_exprt::simplify_member(member_exprt &expr)
 
   if(op.id()==ID_with)
   {
-    if(op.operands().size()>=3)
+    // the following optimization only works on structs,
+    // and not on unions
+  
+    if(op.operands().size()>=3 &&
+       op_type.id()==ID_struct)
     {
       exprt::operandst &operands=op.operands();
 
@@ -3770,7 +3882,7 @@ bool simplify_exprt::simplify_extractbits(exprt &expr)
 
   if(expr.op0().is_constant())
   {
-    unsigned width=bv_width(op0_type);
+    unsigned width=to_bitvector_type(op0_type).get_width();
     mp_integer start, end;
     
     if(to_integer(expr.op1(), start))
@@ -3969,6 +4081,15 @@ bool simplify_exprt::simplify_node(exprt &expr)
     result=simplify_multiplication(expr) && result;
   else if(expr.id()==ID_unary_minus)
     result=simplify_unary_minus(expr) && result;
+  else if(expr.id()==ID_unary_plus)
+  {
+    if(expr.operands().size()==1)
+    {
+      // simply remove, this is always 'nop'
+      expr=expr.op0();
+      result=false;
+    }
+  }
   else if(expr.id()==ID_not)
     result=simplify_not(expr) && result;
   else if(expr.id()==ID_implies || expr.id()=="<=>" ||
@@ -4019,7 +4140,8 @@ Function: simplify_exprt::simplify
 
   Inputs:
 
- Outputs:
+ Outputs: returns true if expression unchanged;
+          returns false if changed
 
  Purpose:
 
