@@ -9,12 +9,26 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_STD_TYPES_H
 #define CPROVER_STD_TYPES_H
 
-#include <assert.h>
+/*! \file util/std_types.h
+ * \brief API to type classes
+ *
+ * \author Daniel Kroening <kroening@kroening.com>
+ * \date   Sun Jul 31 21:54:44 BST 2011
+*/
+
+#include <cassert>
 
-#include <type.h>
 #include <expr.h>
 #include <mp_arith.h>
 
+class constant_exprt;
+
+/*! \defgroup gr_std_types Conversion to specific types
+ *  Conversion to subclasses of @ref typet
+*/
+
+/*! \brief The Booleans
+*/
 class bool_typet:public typet
 {
 public:
@@ -23,6 +37,8 @@ public:
   }
 };
 
+/*! \brief The NIL type
+*/
 class nil_typet:public typet
 {
 public:
@@ -31,6 +47,8 @@ public:
   }
 };
 
+/*! \brief The empty type
+*/
 class empty_typet:public typet
 {
 public:
@@ -39,6 +57,8 @@ public:
   }
 };
 
+/*! \brief Unbounded, signed integers
+*/
 class integer_typet:public typet
 {
 public:
@@ -47,6 +67,8 @@ public:
   }
 };
 
+/*! \brief Unbounded, signed rational numbers
+*/
 class rational_typet:public typet
 {
 public:
@@ -55,6 +77,8 @@ public:
   }
 };
 
+/*! \brief Unbounded, signed real numbers
+*/
 class real_typet:public typet
 {
 public:
@@ -63,6 +87,8 @@ public:
   }
 };
 
+/*! \brief A reference into the symbol table
+*/
 class symbol_typet:public typet
 {
 public:
@@ -86,20 +112,33 @@ public:
   }  
 };
 
+/*! \brief Cast a generic typet to a \ref symbol_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * symbol_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref symbol_typet
+ *
+ * \ingroup gr_std_types
+*/
 extern inline const symbol_typet &to_symbol_type(const typet &type)
 {
   assert(type.id()==ID_symbol);
   return static_cast<const symbol_typet &>(type);
 }
 
+/*! \copydoc to_symbol_type(const typet &)
+ * \ingroup gr_std_types
+*/
 extern inline symbol_typet &to_symbol_type(typet &type)
 {
   assert(type.id()==ID_symbol);
   return static_cast<symbol_typet &>(type);
 }
 
-// structs
-
+/*! \brief Base type of C structs and unions, and C++ classes
+*/
 class struct_union_typet:public typet
 {
 public:
@@ -169,6 +208,26 @@ public:
     {
       return set(ID_C_is_padding, is_padding);
     }
+
+    inline const bool get_is_bit_field() const
+    {
+      return get_bool(ID_C_is_bit_field);
+    }
+
+    inline void set_is_bit_field(bool is_bit_field)
+    {
+      return set(ID_C_is_bit_field, is_bit_field);
+    }
+
+    inline const typet &get_bit_field_type() const
+    {
+      return static_cast<const typet &>(find(ID_C_bit_field_type));
+    }
+
+    inline void set_bit_field_type(const typet &_type)
+    {
+      set(ID_C_bit_field_type, _type);
+    }
   };
 
   typedef std::vector<componentt> componentst;
@@ -195,6 +254,16 @@ public:
   typet component_type(const irep_idt &component_name) const;
 };
 
+/*! \brief Cast a generic typet to a \ref struct_union_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * struct_union_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref struct_union_typet
+ *
+ * \ingroup gr_std_types
+*/
 extern inline const struct_union_typet &to_struct_union_type(const typet &type)
 {
   assert(type.id()==ID_struct ||
@@ -203,6 +272,9 @@ extern inline const struct_union_typet &to_struct_union_type(const typet &type)
   return static_cast<const struct_union_typet &>(type);
 }
 
+/*! \copydoc to_struct_union_type(const typet &)
+ * \ingroup gr_std_types
+*/
 extern inline struct_union_typet &to_struct_union_type(typet &type)
 {
   assert(type.id()==ID_struct ||
@@ -211,16 +283,29 @@ extern inline struct_union_typet &to_struct_union_type(typet &type)
   return static_cast<struct_union_typet &>(type);
 }
 
+/*! \brief Structure type
+*/
 class struct_typet:public struct_union_typet
 {
 public:
   struct_typet():struct_union_typet(ID_struct)
   {
   }
-    
+
+  /// returns true if the object is a prefix of \a other    
   bool is_prefix_of(const struct_typet &other) const;
 };
 
+/*! \brief Cast a generic typet to a \ref struct_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * struct_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref struct_typet
+ *
+ * \ingroup gr_std_types
+*/
 extern inline const struct_typet &to_struct_type(const typet &type)
 {
   assert(type.id()==ID_struct ||
@@ -229,6 +314,9 @@ extern inline const struct_typet &to_struct_type(const typet &type)
   return static_cast<const struct_typet &>(type);
 }
 
+/*! \copydoc to_struct_type(const typet &)
+ * \ingroup gr_std_types
+*/
 extern inline struct_typet &to_struct_type(typet &type)
 {
   assert(type.id()==ID_struct ||
@@ -237,6 +325,8 @@ extern inline struct_typet &to_struct_type(typet &type)
   return static_cast<struct_typet &>(type);
 }
 
+/*! \brief The union type
+*/
 class union_typet:public struct_union_typet
 {
 public:
@@ -245,20 +335,33 @@ public:
   }
 };
 
+/*! \brief Cast a generic typet to a \ref union_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * union_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref union_typet
+ *
+ * \ingroup gr_std_types
+*/
 extern inline const union_typet &to_union_type(const typet &type)
 {
   assert(type.id()==ID_union);
   return static_cast<const union_typet &>(type);
 }
 
+/*! \copydoc to_union_type(const typet &)
+ * \ingroup gr_std_types
+*/
 extern inline union_typet &to_union_type(typet &type)
 {
   assert(type.id()==ID_union);
   return static_cast<union_typet &>(type);
 }
 
-// functions
-
+/*! \brief Base type of functions
+*/
 class code_typet:public typet
 {
 public:
@@ -356,18 +459,33 @@ public:
   }
 };
 
+/*! \brief Cast a generic typet to a \ref code_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * code_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref code_typet
+ *
+ * \ingroup gr_std_types
+*/
 extern inline const code_typet &to_code_type(const typet &type)
 {
   assert(type.id()==ID_code);
   return static_cast<const code_typet &>(type);
 }
 
+/*! \copydoc to_code_type(const typet &)
+ * \ingroup gr_std_types
+*/
 extern inline code_typet &to_code_type(typet &type)
 {
   assert(type.id()==ID_code);
   return static_cast<code_typet &>(type);
 }
 
+/*! \brief arrays with given size
+*/
 class array_typet:public typet
 {
 public:
@@ -391,22 +509,46 @@ public:
   {
     return static_cast<exprt &>(add(ID_size));
   }
+  
+  inline bool is_complete() const
+  {
+    return size().is_not_nil();
+  }
+
+  inline bool is_incomplete() const
+  {
+    return size().is_nil();
+  }
 
 };
 
+/*! \brief Cast a generic typet to an \ref array_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * array_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref array_typet
+ *
+ * \ingroup gr_std_types
+*/
 extern inline const array_typet &to_array_type(const typet &type)
 {
   assert(type.id()==ID_array);
   return static_cast<const array_typet &>(type);
 }
 
+/*! \copydoc to_array_type(const typet &)
+ * \ingroup gr_std_types
+*/
 extern inline array_typet &to_array_type(typet &type)
 {
   assert(type.id()==ID_array);
   return static_cast<array_typet &>(type);
 }
 
-// generic base class
+/*! \brief Base class of bitvector types
+*/
 class bitvector_typet:public typet
 {
 public:
@@ -422,11 +564,23 @@ public:
   }
 };
 
+/*! \brief Cast a generic typet to a \ref bitvector_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * bitvector_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref bitvector_typet
+ *
+ * \ingroup gr_std_types
+*/
 inline const bitvector_typet &to_bitvector_type(const typet &type)
 {
   return static_cast<const bitvector_typet &>(type);
 }
 
+/*! \brief The pointer type
+*/
 class pointer_typet:public bitvector_typet
 {
 public:
@@ -447,18 +601,33 @@ public:
   }
 };
 
+/*! \brief Cast a generic typet to a \ref pointer_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * pointer_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref pointer_typet
+ *
+ * \ingroup gr_std_types
+*/
 extern inline const pointer_typet &to_pointer_type(const typet &type)
 {
   assert(type.id()==ID_pointer);
   return static_cast<const pointer_typet &>(type);
 }
 
+/*! \copydoc to_pointer_type(const typet &)
+ * \ingroup gr_std_types
+*/
 extern inline pointer_typet &to_pointer_type(typet &type)
 {
   assert(type.id()==ID_pointer);
   return static_cast<pointer_typet &>(type);
 }
 
+/*! \brief The reference type
+*/
 class reference_typet:public pointer_typet
 {
 public:
@@ -468,9 +637,15 @@ public:
   }
 };
 
+/*! \brief TO_BE_DOCUMENTED
+*/
 bool is_reference(const typet &type);
+/*! \brief TO_BE_DOCUMENTED
+*/
 bool is_rvalue_reference(const typet &type);
 
+/*! \brief fixed-width bit-vector without numerical interpretation
+*/
 class bv_typet:public bitvector_typet
 {
 public:
@@ -484,18 +659,33 @@ public:
   }
 };
 
+/*! \brief Cast a generic typet to a \ref bv_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * bv_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref bv_typet
+ *
+ * \ingroup gr_std_types
+*/
 inline const bv_typet &to_bv_type(const typet &type)
 {
   assert(type.id()==ID_bv);
   return static_cast<const bv_typet &>(type);
 }
 
+/*! \copydoc to_bv_type(const typet &)
+ * \ingroup gr_std_types
+*/
 inline bv_typet &to_bv_type(typet &type)
 {
   assert(type.id()==ID_bv);
   return static_cast<bv_typet &>(type);
 }
 
+/*! \brief Fixed-width bit-vector with unsigned binary interpretation
+*/
 class unsignedbv_typet:public bitvector_typet
 {
 public:
@@ -507,20 +697,40 @@ public:
   {
     set_width(width);
   }
+  
+  mp_integer smallest() const;
+  mp_integer largest() const;
+  constant_exprt smallest_expr() const;
+  constant_exprt largest_expr() const;
 };
 
+/*! \brief Cast a generic typet to an \ref unsignedbv_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * unsignedbv_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref unsignedbv_typet
+ *
+ * \ingroup gr_std_types
+*/
 inline const unsignedbv_typet &to_unsignedbv_type(const typet &type)
 {
   assert(type.id()==ID_unsignedbv);
   return static_cast<const unsignedbv_typet &>(type);
 }
 
+/*! \copydoc to_unsignedbv_type(const typet &)
+ * \ingroup gr_std_types
+*/
 inline unsignedbv_typet &to_unsignedbv_type(typet &type)
 {
   assert(type.id()==ID_unsignedbv);
   return static_cast<unsignedbv_typet &>(type);
 }
 
+/*! \brief Fixed-width bit-vector with two's complement interpretation
+*/
 class signedbv_typet:public bitvector_typet
 {
 public:
@@ -532,20 +742,40 @@ public:
   {
     set_width(width);
   }
+
+  mp_integer smallest() const;
+  mp_integer largest() const;
+  constant_exprt smallest_expr() const;
+  constant_exprt largest_expr() const;
 };
 
+/*! \brief Cast a generic typet to a \ref signedbv_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * signedbv_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref signedbv_typet
+ *
+ * \ingroup gr_std_types
+*/
 inline const signedbv_typet &to_signedbv_type(const typet &type)
 {
   assert(type.id()==ID_signedbv);
   return static_cast<const signedbv_typet &>(type);
 }
 
+/*! \copydoc to_signedbv_type(const typet &)
+ * \ingroup gr_std_types
+*/
 inline signedbv_typet &to_signedbv_type(typet &type)
 {
   assert(type.id()==ID_signedbv);
   return static_cast<signedbv_typet &>(type);
 }
 
+/*! \brief Fixed-width bit-vector with signed fixed-point interpretation
+*/
 class fixedbv_typet:public bitvector_typet
 {
 public:
@@ -572,8 +802,20 @@ public:
   }
 };
 
+/*! \brief Cast a generic typet to a \ref fixedbv_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * fixedbv_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref fixedbv_typet
+ *
+ * \ingroup gr_std_types
+*/
 const fixedbv_typet &to_fixedbv_type(const typet &type);
 
+/*! \brief Fixed-width bit-vector with IEEE floating-point interpretation
+*/
 class floatbv_typet:public bitvector_typet
 {
 public:
@@ -583,7 +825,8 @@ public:
 
   unsigned get_e() const
   {
-    return get_width()-get_f();
+    // subtract one for sign bit
+    return get_width()-get_f()-1;
   }
 
   unsigned get_f() const;
@@ -600,8 +843,20 @@ public:
   }
 };
 
+/*! \brief Cast a generic typet to a \ref floatbv_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * floatbv_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref floatbv_typet
+ *
+ * \ingroup gr_std_types
+*/
 const floatbv_typet &to_floatbv_type(const typet &type);
 
+/*! \brief TO_BE_DOCUMENTED
+*/
 class string_typet:public typet
 {
 public:
@@ -616,8 +871,20 @@ public:
   }
 };
 
+/*! \brief Cast a generic typet to a \ref string_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * string_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref string_typet
+ *
+ * \ingroup gr_std_types
+*/
 const string_typet &to_string_type(const typet &type);
 
+/*! \brief TO_BE_DOCUMENTED
+*/
 class range_typet:public typet
 {
 public:
@@ -638,8 +905,20 @@ public:
   void set_to(const mp_integer &to);
 };
 
+/*! \brief Cast a generic typet to a \ref range_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * range_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref range_typet
+ *
+ * \ingroup gr_std_types
+*/
 const range_typet &to_range_type(const typet &type);
 
+/*! \brief TO_BE_DOCUMENTED
+*/
 class vector_typet:public typet
 {
 public:
@@ -666,12 +945,25 @@ public:
 
 };
 
+/*! \brief Cast a generic typet to a \ref vector_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * vector_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref vector_typet
+ *
+ * \ingroup gr_std_types
+*/
 extern inline const vector_typet &to_vector_type(const typet &type)
 {
   assert(type.id()==ID_vector);
   return static_cast<const vector_typet &>(type);
 }
 
+/*! \copydoc to_vector_type(const typet &)
+ * \ingroup gr_std_types
+*/
 extern inline vector_typet &to_vector_type(typet &type)
 {
   assert(type.id()==ID_vector);
