@@ -9,7 +9,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_STR_GETLINE_H
 #define CPROVER_STR_GETLINE_H
 
-#include <iostream>
+#include <istream>
 #include <string>
 
 std::istream &str_getline(std::istream &in, std::string &dest);
