@@ -6,9 +6,12 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
 
 #include "type_eq.h"
+#include "type.h"
+#include "symbol.h"
+#include "namespace.h"
 
 /*******************************************************************\
 
